@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -32,23 +32,30 @@
 #include <xapian/unicode.h>
 
 #include "omassert.h"
-#include "omdebug.h"
+#include "debuglog.h"
 #include "../backends/multi/multi_postlist.h"
 #include "../backends/multi/multi_termlist.h"
 #include "alltermslist.h"
 #include "multialltermslist.h"
+#include "multivaluelist.h"
 #include "database.h"
 #include "editdistance.h"
 #include "ortermlist.h"
 #include "noreturn.h"
 
-#include <stdlib.h> // For abs().
+#include <cstdlib> // For abs().
 
 #include <cstring>
 #include <vector>
 
 using namespace std;
 
+XAPIAN_NORETURN(static void docid_zero_invalid());
+static void docid_zero_invalid()
+{
+    throw Xapian::InvalidArgumentError("Document ID 0 is invalid");
+}
+
 XAPIAN_NORETURN(static void no_subdatabases());
 static void no_subdatabases()
 {
@@ -59,28 +66,28 @@ namespace Xapian {
 
 Database::Database()
 {
-    DEBUGAPICALL(void, "Database::Database", "");
+    LOGCALL_CTOR(API, "Database", NO_ARGS);
 }
 
 Database::Database(Database::Internal *internal_)
 {
-    DEBUGAPICALL(void, "Database::Database", "Database::Internal");
+    LOGCALL_CTOR(API, "Database", internal_);
     Xapian::Internal::RefCntPtr<Database::Internal> newi(internal_);
     internal.push_back(newi);
 }
 
 Database::Database(const Database &other)
 {
-    DEBUGAPICALL(void, "Database::Database", "Database");
+    LOGCALL_CTOR(API, "Database", other);
     internal = other.internal;
 }
 
 void
 Database::operator=(const Database &other)
 {
-    DEBUGAPICALL(void, "Database::operator=", "Database");
+    LOGCALL_VOID(API, "Database::operator=", other);
     if (this == &other) {
-	DEBUGLINE(API, "Database assigned to itself");
+	LOGLINE(API, "Database assigned to itself");
 	return;
     }
 
@@ -89,27 +96,36 @@ Database::operator=(const Database &other)
 
 Database::~Database()
 {
-    DEBUGAPICALL(void, "Database::~Database", "");
+    LOGCALL_DTOR(API, "Database");
 }
 
 void
 Database::reopen()
 {
-    DEBUGAPICALL(void, "Database::reopen", "");
+    LOGCALL_VOID(API, "Database::reopen", NO_ARGS);
     vector<Xapian::Internal::RefCntPtr<Database::Internal> >::iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	(*i)->reopen();
     }
 }
 
+void
+Database::close()
+{
+    LOGCALL_VOID(API, "Database::close", NO_ARGS);
+    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::iterator i;
+    for (i = internal.begin(); i != internal.end(); ++i) {
+	(*i)->close();
+    }
+}
+
 void
 Database::add_database(const Database & database)
 {
-    DEBUGAPICALL(void, "Database::add_database", "Database");
+    LOGCALL_VOID(API, "Database::add_database", database);
     if (this == &database) {
-	DEBUGLINE(API, "Database added to itself");
-	throw Xapian::InvalidArgumentError("Can't add an Database to itself");
-	return;
+	LOGLINE(API, "Database added to itself");
+	throw Xapian::InvalidArgumentError("Can't add a Database to itself");
     }
     vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
     for (i = database.internal.begin(); i != database.internal.end(); ++i) {
@@ -120,7 +136,7 @@ Database::add_database(const Database & database)
 PostingIterator
 Database::postlist_begin(const string &tname) const
 {
-    DEBUGAPICALL(PostingIterator, "Database::postlist_begin", tname);
+    LOGCALL(API, PostingIterator, "Database::postlist_begin", tname);
 
     // Don't bother checking that the term exists first.  If it does, we
     // just end up doing more work, and if it doesn't, we save very little
@@ -156,8 +172,9 @@ Database::postlist_begin(const string &tname) const
 TermIterator
 Database::termlist_begin(Xapian::docid did) const
 {
-    DEBUGAPICALL(TermIterator, "Database::termlist_begin", did);
-    if (did == 0) throw InvalidArgumentError("Document ID 0 is invalid");
+    LOGCALL(API, TermIterator, "Database::termlist_begin", did);
+    if (did == 0)
+	docid_zero_invalid();
 
     unsigned int multiplier = internal.size();
     if (rare(multiplier == 0))
@@ -186,7 +203,7 @@ Database::allterms_begin() const
 TermIterator
 Database::allterms_begin(const std::string & prefix) const
 {
-    DEBUGAPICALL(TermIterator, "Database::allterms_begin", "");
+    LOGCALL(API, TermIterator, "Database::allterms_begin", NO_ARGS);
     TermList * tl;
     if (rare(internal.size() == 0)) {
 	tl = NULL;
@@ -201,7 +218,7 @@ Database::allterms_begin(const std::string & prefix) const
 bool
 Database::has_positions() const
 {
-    DEBUGAPICALL(bool, "Database::has_positions", "");
+    LOGCALL(API, bool, "Database::has_positions", NO_ARGS);
     // If any sub-database has positions, the combined database does.
     vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
@@ -213,11 +230,11 @@ Database::has_positions() const
 PositionIterator
 Database::positionlist_begin(Xapian::docid did, const string &tname) const
 {
-    DEBUGAPICALL(PositionIterator, "Database::positionlist_begin",
-		 did << ", " << tname);
+    LOGCALL(API, PositionIterator, "Database::positionlist_begin", did | tname);
     if (tname.empty())
 	throw InvalidArgumentError("Zero length terms are invalid");
-    if (did == 0) throw InvalidArgumentError("Document ID 0 is invalid");
+    if (did == 0)
+	docid_zero_invalid();
 
     unsigned int multiplier = internal.size();
     if (rare(multiplier == 0))
@@ -230,7 +247,7 @@ Database::positionlist_begin(Xapian::docid did, const string &tname) const
 Xapian::doccount
 Database::get_doccount() const
 {
-    DEBUGAPICALL(Xapian::doccount, "Database::get_doccount", "");
+    LOGCALL(API, Xapian::doccount, "Database::get_doccount", NO_ARGS);
     Xapian::doccount docs = 0;
     vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
@@ -242,7 +259,7 @@ Database::get_doccount() const
 Xapian::docid
 Database::get_lastdocid() const
 {
-    DEBUGAPICALL(Xapian::docid, "Database::get_lastdocid", "");
+    LOGCALL(API, Xapian::docid, "Database::get_lastdocid", NO_ARGS);
     Xapian::docid did = 0;
 
     unsigned int multiplier = internal.size();
@@ -256,7 +273,7 @@ Database::get_lastdocid() const
 Xapian::doclength
 Database::get_avlength() const
 {
-    DEBUGAPICALL(Xapian::doclength, "Database::get_avlength", "");
+    LOGCALL(API, Xapian::doclength, "Database::get_avlength", NO_ARGS);
     Xapian::doccount docs = 0;
     Xapian::doclength totlen = 0;
 
@@ -266,8 +283,8 @@ Database::get_avlength() const
 	docs += db_doccount;
 	totlen += (*i)->get_avlength() * db_doccount;
     }
-    DEBUGLINE(UNKNOWN, "get_avlength() = " << totlen << " / " << docs <<
-	      " (from " << internal.size() << " dbs)");
+    LOGLINE(UNKNOWN, "get_avlength() = " << totlen << " / " << docs <<
+	    " (from " << internal.size() << " dbs)");
 
     if (docs == 0) RETURN(0.0);
     RETURN(totlen / docs);
@@ -276,7 +293,7 @@ Database::get_avlength() const
 Xapian::doccount
 Database::get_termfreq(const string & tname) const
 {
-    DEBUGAPICALL(Xapian::doccount, "Database::get_termfreq", tname);
+    LOGCALL(API, Xapian::doccount, "Database::get_termfreq", tname);
     if (tname.empty()) RETURN(get_doccount());
 
     Xapian::doccount tf = 0;
@@ -290,7 +307,7 @@ Database::get_termfreq(const string & tname) const
 Xapian::termcount
 Database::get_collection_freq(const string & tname) const
 {
-    DEBUGAPICALL(Xapian::termcount, "Database::get_collection_freq", tname);
+    LOGCALL(API, Xapian::termcount, "Database::get_collection_freq", tname);
     if (tname.empty()) RETURN(get_doccount());
 
     Xapian::termcount cf = 0;
@@ -301,11 +318,113 @@ Database::get_collection_freq(const string & tname) const
     RETURN(cf);
 }
 
-Xapian::doclength
+Xapian::doccount
+Database::get_value_freq(Xapian::valueno valno) const
+{
+    LOGCALL(API, Xapian::doccount, "Database::get_value_freq", valno);
+
+    Xapian::doccount vf = 0;
+    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    for (i = internal.begin(); i != internal.end(); i++) {
+	vf += (*i)->get_value_freq(valno);
+    }
+    RETURN(vf);
+}
+
+string
+Database::get_value_lower_bound(Xapian::valueno valno) const
+{
+    LOGCALL(API, string, "Database::get_value_lower_bound", valno);
+
+    if (rare(internal.empty())) RETURN(string());
+
+    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    i = internal.begin();
+    string full_lb = (*i)->get_value_lower_bound(valno);
+    while (++i != internal.end()) {
+	string lb = (*i)->get_value_lower_bound(valno);
+	if (lb < full_lb) full_lb = lb;
+    }
+    RETURN(full_lb);
+}
+
+std::string
+Database::get_value_upper_bound(Xapian::valueno valno) const
+{
+    LOGCALL(API, std::string, "Database::get_value_upper_bound", valno);
+
+    std::string full_ub;
+    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    for (i = internal.begin(); i != internal.end(); i++) {
+	std::string ub = (*i)->get_value_upper_bound(valno);
+	if (ub > full_ub)
+	    full_ub = ub;
+    }
+    RETURN(full_ub);
+}
+
+Xapian::termcount
+Database::get_doclength_lower_bound() const
+{
+    LOGCALL(API, Xapian::termcount, "Database::get_doclength_lower_bound", NO_ARGS);
+
+    if (rare(internal.empty())) RETURN(0);
+
+    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    i = internal.begin();
+    Xapian::termcount full_lb = (*i)->get_doclength_lower_bound();
+    while (++i != internal.end()) {
+	Xapian::termcount lb = (*i)->get_doclength_lower_bound();
+	if (lb < full_lb) full_lb = lb;
+    }
+    RETURN(full_lb);
+}
+
+Xapian::termcount
+Database::get_doclength_upper_bound() const
+{
+    LOGCALL(API, Xapian::termcount, "Database::get_doclength_upper_bound", NO_ARGS);
+
+    Xapian::termcount full_ub = 0;
+    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    for (i = internal.begin(); i != internal.end(); i++) {
+	Xapian::termcount ub = (*i)->get_doclength_upper_bound();
+	if (ub > full_ub) full_ub = ub;
+    }
+    RETURN(full_ub);
+}
+
+Xapian::termcount
+Database::get_wdf_upper_bound(const string & term) const
+{
+    LOGCALL(API, Xapian::termcount, "Database::get_wdf_upper_bound", term);
+
+    Xapian::termcount full_ub = 0;
+    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    for (i = internal.begin(); i != internal.end(); i++) {
+	Xapian::termcount ub = (*i)->get_wdf_upper_bound(term);
+	if (ub > full_ub) full_ub = ub;
+    }
+    RETURN(full_ub);
+}
+
+ValueIterator
+Database::valuestream_begin(Xapian::valueno slot) const
+{
+    LOGCALL(API, ValueIterator, "Database::valuestream_begin", slot);
+    if (internal.size() == 0)
+       	RETURN(ValueIterator());
+    if (internal.size() != 1)
+	RETURN(ValueIterator(new MultiValueList(internal, slot)));
+    RETURN(ValueIterator(internal[0]->open_value_list(slot)));
+}
+
+Xapian::termcount
 Database::get_doclength(Xapian::docid did) const
 {
-    DEBUGAPICALL(Xapian::doclength, "Database::get_doclength", did);
-    if (did == 0) throw InvalidArgumentError("Document ID 0 is invalid");
+    LOGCALL(API, Xapian::termcount, "Database::get_doclength", did);
+    if (did == 0)
+	docid_zero_invalid();
 
     unsigned int multiplier = internal.size();
     if (rare(multiplier == 0))
@@ -318,8 +437,9 @@ Database::get_doclength(Xapian::docid did) const
 Document
 Database::get_document(Xapian::docid did) const
 {
-    DEBUGAPICALL(Document, "Database::get_document", did);
-    if (did == 0) throw InvalidArgumentError("Document ID 0 is invalid");
+    LOGCALL(API, Document, "Database::get_document", did);
+    if (did == 0)
+	docid_zero_invalid();
 
     unsigned int multiplier = internal.size();
     if (rare(multiplier == 0))
@@ -331,10 +451,25 @@ Database::get_document(Xapian::docid did) const
     RETURN(Document(internal[n]->open_document(m, false)));
 }
 
+Document::Internal *
+Database::get_document_lazily(Xapian::docid did) const
+{
+    LOGCALL(DB, Document::Internal *, "Database::get_document_lazily", did);
+    if (did == 0)
+	docid_zero_invalid();
+
+    unsigned int multiplier = internal.size();
+    Assert(multiplier != 0);
+    Xapian::doccount n = (did - 1) % multiplier; // which actual database
+    Xapian::docid m = (did - 1) / multiplier + 1; // real docid in that database
+
+    RETURN(internal[n]->open_document(m, true));
+}
+
 bool
 Database::term_exists(const string & tname) const
 {
-    DEBUGAPICALL(bool, "Database::term_exists", tname);
+    LOGCALL(API, bool, "Database::term_exists", tname);
     if (tname.empty()) {
 	RETURN(get_doccount() != 0);
     }
@@ -348,7 +483,7 @@ Database::term_exists(const string & tname) const
 void
 Database::keep_alive()
 {
-    DEBUGAPICALL(void, "Database::keep_alive", "");
+    LOGCALL_VOID(API, "Database::keep_alive", NO_ARGS);
     vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	(*i)->keep_alive();
@@ -405,18 +540,17 @@ string
 Database::get_spelling_suggestion(const string &word,
 				  unsigned max_edit_distance) const
 {
-    DEBUGAPICALL(string, "Database::get_spelling_suggestion",
-		 word << ", " << max_edit_distance);
+    LOGCALL(API, string, "Database::get_spelling_suggestion", word | max_edit_distance);
     if (word.size() <= 1) return string();
     AutoPtr<TermList> merger;
     for (size_t i = 0; i < internal.size(); ++i) {
 	TermList * tl = internal[i]->open_spelling_termlist(word);
-	DEBUGLINE(SPELLING, "Sub db " << i << " tl = " << (void*)tl);
+	LOGLINE(SPELLING, "Sub db " << i << " tl = " << (void*)tl);
 	if (tl) {
 	    if (merger.get()) {
-		merger = new OrTermList(merger.release(), tl);
+		merger.reset(new OrTermList(merger.release(), tl));
 	    } else {
-		merger = tl;
+		merger.reset(tl);
 	    }
 	}
     }
@@ -442,14 +576,14 @@ Database::get_spelling_suggestion(const string &word,
     Xapian::doccount freq_best = 0;
     while (true) {
 	TermList *ret = merger->next();
-	if (ret) merger = ret;
+	if (ret) merger.reset(ret);
 
 	if (merger->at_end()) break;
 
 	string term = merger->get_termname();
 	Xapian::termcount score = merger->get_wdf();
 
-	DEBUGLINE(SPELLING, "Term \"" << term << "\" ngram score " << score);
+	LOGLINE(SPELLING, "Term \"" << term << "\" ngram score " << score);
 	if (score + TRIGRAM_SCORE_THRESHOLD >= best) {
 	    if (score > best) best = score;
 
@@ -459,8 +593,8 @@ Database::get_spelling_suggestion(const string &word,
 
 	    // First check the length of the encoded UTF-8 version of term.
 	    // Each UTF-32 character is 1-4 bytes in UTF-8.
-	    if (abs((long)term.size() - (long)word.size()) > edist_best * 4) {
-		DEBUGLINE(SPELLING, "Lengths much too different");
+	    if (abs(long(term.size()) - long(word.size())) > edist_best * 4) {
+		LOGLINE(SPELLING, "Lengths much too different");
 		continue;
 	    }
 
@@ -470,12 +604,12 @@ Database::get_spelling_suggestion(const string &word,
 
 	    if (abs(long(utf32_term.size()) - long(utf32_word.size()))
 		    > edist_best) {
-		DEBUGLINE(SPELLING, "Lengths too different");
+		LOGLINE(SPELLING, "Lengths too different");
 		continue;
 	    }
 
 	    if (freq_edit_lower_bound(utf32_term, utf32_word) > edist_best) {
-		DEBUGLINE(SPELLING, "Rejected by character frequency test");
+		LOGLINE(SPELLING, "Rejected by character frequency test");
 		continue;
 	    }
 
@@ -484,7 +618,7 @@ Database::get_spelling_suggestion(const string &word,
 					       &utf32_word[0],
 					       int(utf32_word.size()),
 					       edist_best);
-	    DEBUGLINE(SPELLING, "Edit distance " << edist);
+	    LOGLINE(SPELLING, "Edit distance " << edist);
 	    // If we have an exact match, return an empty string since there's
 	    // no correction required.
 	    if (edist == 0) RETURN(string());
@@ -494,11 +628,10 @@ Database::get_spelling_suggestion(const string &word,
 		for (size_t j = 0; j < internal.size(); ++j)
 		    freq += internal[j]->get_spelling_frequency(term);
 
-		DEBUGLINE(SPELLING, "Freq " << freq << " best " << freq_best);
+		LOGLINE(SPELLING, "Freq " << freq << " best " << freq_best);
 		if (edist < edist_best || freq > freq_best) {
-		    DEBUGLINE(SPELLING, "Best so far: \"" << term <<
-					"\" edist " << edist << " freq " <<
-					freq);
+		    LOGLINE(SPELLING, "Best so far: \"" << term <<
+				      "\" edist " << edist << " freq " << freq);
 		    result = term;
 		    edist_best = edist;
 		    freq_best = freq;
@@ -512,15 +645,15 @@ Database::get_spelling_suggestion(const string &word,
 TermIterator
 Database::spellings_begin() const
 {
-    DEBUGAPICALL(TermIterator, "Database::spellings_begin", "");
+    LOGCALL(API, TermIterator, "Database::spellings_begin", NO_ARGS);
     AutoPtr<TermList> merger;
     for (size_t i = 0; i < internal.size(); ++i) {
 	TermList * tl = internal[i]->open_spelling_wordlist();
 	if (tl) {
 	    if (merger.get()) {
-		merger = new FreqAdderOrTermList(merger.release(), tl);
+		merger.reset(new FreqAdderOrTermList(merger.release(), tl));
 	    } else {
-		merger = tl;
+		merger.reset(tl);
 	    }
 	}
     }
@@ -530,15 +663,15 @@ Database::spellings_begin() const
 TermIterator
 Database::synonyms_begin(const std::string &term) const
 {
-    DEBUGAPICALL(TermIterator, "Database::synonyms_begin", term);
+    LOGCALL(API, TermIterator, "Database::synonyms_begin", term);
     AutoPtr<TermList> merger;
     for (size_t i = 0; i < internal.size(); ++i) {
 	TermList * tl = internal[i]->open_synonym_termlist(term);
 	if (tl) {
 	    if (merger.get()) {
-		merger = new OrTermList(merger.release(), tl);
+		merger.reset(new OrTermList(merger.release(), tl));
 	    } else {
-		merger = tl;
+		merger.reset(tl);
 	    }
 	}
     }
@@ -548,15 +681,15 @@ Database::synonyms_begin(const std::string &term) const
 TermIterator
 Database::synonym_keys_begin(const std::string &prefix) const
 {
-    DEBUGAPICALL(TermIterator, "Database::synonyms_keys_begin", prefix);
+    LOGCALL(API, TermIterator, "Database::synonyms_keys_begin", prefix);
     AutoPtr<TermList> merger;
     for (size_t i = 0; i < internal.size(); ++i) {
 	TermList * tl = internal[i]->open_synonym_keylist(prefix);
 	if (tl) {
 	    if (merger.get()) {
-		merger = new OrTermList(merger.release(), tl);
+		merger.reset(new OrTermList(merger.release(), tl));
 	    } else {
-		merger = tl;
+		merger.reset(tl);
 	    }
 	}
     }
@@ -566,7 +699,7 @@ Database::synonym_keys_begin(const std::string &prefix) const
 string
 Database::get_metadata(const string & key) const
 {
-    DEBUGAPICALL(string, "Database::get_metadata", key);
+    LOGCALL(API, string, "Database::get_metadata", key);
     if (key.empty())
 	throw InvalidArgumentError("Empty metadata keys are invalid");
     if (internal.empty()) RETURN(std::string());
@@ -576,41 +709,57 @@ Database::get_metadata(const string & key) const
 Xapian::TermIterator
 Database::metadata_keys_begin(const std::string &prefix) const
 {
-    DEBUGAPICALL(Xapian::TermIterator, "Database::metadata_keys_begin", "");
+    LOGCALL(API, Xapian::TermIterator, "Database::metadata_keys_begin", NO_ARGS);
     if (internal.empty()) RETURN(TermIterator(NULL));
     RETURN(TermIterator(internal[0]->open_metadata_keylist(prefix)));
 }
 
+std::string
+Database::get_uuid() const
+{
+    LOGCALL(API, std::string, "Database::get_uuid", NO_ARGS);
+    string uuid;
+    for (size_t i = 0; i < internal.size(); ++i) {
+	string sub_uuid = internal[i]->get_uuid();
+	// If any of the sub-databases have no uuid, we can't make a uuid for
+	// the combined database.
+	if (sub_uuid.empty())
+	    RETURN(sub_uuid);
+	if (!uuid.empty()) uuid += ':';
+	uuid += sub_uuid;
+    }
+    RETURN(uuid);
+}
+
 ///////////////////////////////////////////////////////////////////////////
 
 WritableDatabase::WritableDatabase() : Database()
 {
-    DEBUGAPICALL(void, "WritableDatabase::WritableDatabase", "");
+    LOGCALL_CTOR(API, "WritableDatabase", NO_ARGS);
 }
 
 WritableDatabase::WritableDatabase(Database::Internal *internal_)
 	: Database(internal_)
 {
-    DEBUGAPICALL(void, "WritableDatabase::WritableDatabase",
-		 "Database::Internal");
+    LOGCALL_CTOR(API, "WritableDatabase", internal_);
 }
 
 WritableDatabase::WritableDatabase(const WritableDatabase &other)
 	: Database(other)
 {
-    DEBUGAPICALL(void, "WritableDatabase::WritableDatabase", "WritableDatabase");
+    LOGCALL_CTOR(API, "WritableDatabase", other);
 }
 
 void
 WritableDatabase::operator=(const WritableDatabase &other)
 {
-    DEBUGAPICALL(void, "WritableDatabase::operator=", "WritableDatabase");
+    LOGCALL_VOID(API, "WritableDatabase::operator=", other);
     Database::operator=(other);
 }
 
 WritableDatabase::~WritableDatabase()
 {
-    DEBUGAPICALL(void, "WritableDatabase::~WritableDatabase", "");
+    LOGCALL_DTOR(API, "WritableDatabase");
 }
 
 XAPIAN_NORETURN(static void only_one_subdatabase_allowed());
@@ -620,17 +769,17 @@ static void only_one_subdatabase_allowed()
 }
 
 void
-WritableDatabase::flush()
+WritableDatabase::commit()
 {
-    DEBUGAPICALL(void, "WritableDatabase::flush", "");
+    LOGCALL_VOID(API, "WritableDatabase::commit", NO_ARGS);
     if (internal.size() != 1) only_one_subdatabase_allowed();
-    internal[0]->flush();
+    internal[0]->commit();
 }
 
 void
 WritableDatabase::begin_transaction(bool flushed)
 {
-    DEBUGAPICALL(void, "WritableDatabase::begin_transaction", "");
+    LOGCALL_VOID(API, "WritableDatabase::begin_transaction", NO_ARGS);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->begin_transaction(flushed);
 }
@@ -638,7 +787,7 @@ WritableDatabase::begin_transaction(bool flushed)
 void
 WritableDatabase::commit_transaction()
 {
-    DEBUGAPICALL(void, "WritableDatabase::commit_transaction", "");
+    LOGCALL_VOID(API, "WritableDatabase::commit_transaction", NO_ARGS);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->commit_transaction();
 }
@@ -646,7 +795,7 @@ WritableDatabase::commit_transaction()
 void
 WritableDatabase::cancel_transaction()
 {
-    DEBUGAPICALL(void, "WritableDatabase::cancel_transaction", "");
+    LOGCALL_VOID(API, "WritableDatabase::cancel_transaction", NO_ARGS);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->cancel_transaction();
 }
@@ -655,7 +804,7 @@ WritableDatabase::cancel_transaction()
 Xapian::docid
 WritableDatabase::add_document(const Document & document)
 {
-    DEBUGAPICALL(Xapian::docid, "WritableDatabase::add_document", document);
+    LOGCALL(API, Xapian::docid, "WritableDatabase::add_document", document);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     RETURN(internal[0]->add_document(document));
 }
@@ -663,16 +812,17 @@ WritableDatabase::add_document(const Document & document)
 void
 WritableDatabase::delete_document(Xapian::docid did)
 {
-    DEBUGAPICALL(void, "WritableDatabase::delete_document", did);
+    LOGCALL_VOID(API, "WritableDatabase::delete_document", did);
     if (internal.size() != 1) only_one_subdatabase_allowed();
-    if (did == 0) throw InvalidArgumentError("Document ID 0 is invalid");
+    if (did == 0)
+	docid_zero_invalid();
     internal[0]->delete_document(did);
 }
 
 void
 WritableDatabase::delete_document(const std::string & unique_term)
 {
-    DEBUGAPICALL(void, "WritableDatabase::delete_document", unique_term);
+    LOGCALL_VOID(API, "WritableDatabase::delete_document", unique_term);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     if (unique_term.empty())
 	throw InvalidArgumentError("Empty termnames are invalid");
@@ -682,10 +832,10 @@ WritableDatabase::delete_document(const std::string & unique_term)
 void
 WritableDatabase::replace_document(Xapian::docid did, const Document & document)
 {
-    DEBUGAPICALL(void, "WritableDatabase::replace_document",
-		 did << ", " << document);
+    LOGCALL_VOID(API, "WritableDatabase::replace_document", did | document);
     if (internal.size() != 1) only_one_subdatabase_allowed();
-    if (did == 0) throw Xapian::InvalidArgumentError("Document ID 0 is invalid");
+    if (did == 0)
+	docid_zero_invalid();
     internal[0]->replace_document(did, document);
 }
 
@@ -693,8 +843,7 @@ Xapian::docid
 WritableDatabase::replace_document(const std::string & unique_term,
 				   const Document & document)
 {
-    DEBUGAPICALL(Xapian::docid, "WritableDatabase::replace_document",
-		 unique_term << ", " << document);
+    LOGCALL(API, Xapian::docid, "WritableDatabase::replace_document", unique_term | document);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     if (unique_term.empty())
 	throw InvalidArgumentError("Empty termnames are invalid");
@@ -705,8 +854,7 @@ void
 WritableDatabase::add_spelling(const std::string & word,
 			       Xapian::termcount freqinc) const
 {
-    DEBUGAPICALL(void, "WritableDatabase::add_spelling",
-		 word << ", " << freqinc);
+    LOGCALL_VOID(API, "WritableDatabase::add_spelling", word | freqinc);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->add_spelling(word, freqinc);
 }
@@ -715,8 +863,7 @@ void
 WritableDatabase::remove_spelling(const std::string & word,
 				  Xapian::termcount freqdec) const
 {
-    DEBUGAPICALL(void, "WritableDatabase::remove_spelling",
-		 word << ", " << freqdec);
+    LOGCALL_VOID(API, "WritableDatabase::remove_spelling", word | freqdec);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->remove_spelling(word, freqdec);
 }
@@ -725,8 +872,7 @@ void
 WritableDatabase::add_synonym(const std::string & term,
 			      const std::string & synonym) const
 {
-    DEBUGAPICALL(void, "WritableDatabase::add_synonym",
-		 term << ", " << synonym);
+    LOGCALL_VOID(API, "WritableDatabase::add_synonym", term | synonym);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->add_synonym(term, synonym);
 }
@@ -735,8 +881,7 @@ void
 WritableDatabase::remove_synonym(const std::string & term,
 				 const std::string & synonym) const
 {
-    DEBUGAPICALL(void, "WritableDatabase::remove_synonym",
-		 term << ", " << synonym);
+    LOGCALL_VOID(API, "WritableDatabase::remove_synonym", term | synonym);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->remove_synonym(term, synonym);
 }
@@ -744,7 +889,7 @@ WritableDatabase::remove_synonym(const std::string & term,
 void
 WritableDatabase::clear_synonyms(const std::string & term) const
 {
-    DEBUGAPICALL(void, "WritableDatabase::clear_synonyms", term);
+    LOGCALL_VOID(API, "WritableDatabase::clear_synonyms", term);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->clear_synonyms(term);
 }
@@ -752,7 +897,7 @@ WritableDatabase::clear_synonyms(const std::string & term) const
 void
 WritableDatabase::set_metadata(const string & key, const string & value)
 {
-    DEBUGAPICALL(void, "WritableDatabase::set_metadata", key << ", " << value);
+    LOGCALL_VOID(API, "WritableDatabase::set_metadata", key | value);
     if (internal.size() != 1) only_one_subdatabase_allowed();
     if (key.empty())
 	throw InvalidArgumentError("Empty metadata keys are invalid");
