@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
  * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
- * Copyright 2007 Lemur Consulting Ltd
+ * Copyright 2007,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,40 +22,43 @@
  */
 
 #include <config.h>
-#include "omdebug.h"
+#include "xapian/enquire.h"
 
-#include <xapian/document.h>
-#include <xapian/enquire.h>
-#include <xapian/error.h>
-#include <xapian/errorhandler.h>
-#include <xapian/expanddecider.h>
-#include <xapian/termiterator.h>
+#include "xapian/document.h"
+#include "xapian/error.h"
+#include "xapian/errorhandler.h"
+#include "xapian/expanddecider.h"
+#include "xapian/termiterator.h"
+#include "xapian/weight.h"
 
 #include "vectortermlist.h"
 
-#include "rset.h"
-#include "multimatch.h"
-#include "expand.h"
 #include "database.h"
+#include "debuglog.h"
+#include "esetinternal.h"
+#include "expandweight.h"
+#include "multimatch.h"
 #include "omenquireinternal.h"
-#include "stats.h"
-#include "utils.h"
+#include "str.h"
+#include "weightinternal.h"
 
-#include <vector>
-#include "autoptr.h"
 #include <algorithm>
+#include "autoptr.h"
 #include <cfloat>
-#include <math.h>
+#include <cmath>
+#include <vector>
 
 using namespace std;
 
+using Xapian::Internal::ExpandWeight;
+
 namespace Xapian {
 
 MatchDecider::~MatchDecider() { }
 
 // Methods for Xapian::RSet
 
-RSet::RSet() : internal(new RSet::Internal())
+RSet::RSet() : internal(new RSet::Internal)
 {
 }
 
@@ -88,6 +91,7 @@ RSet::empty() const
 void
 RSet::add_document(Xapian::docid did)
 {
+    if (did == 0) throw Xapian::InvalidArgumentError("Docid 0 not valid");
     internal->items.insert(did);
 }
 
@@ -117,7 +121,7 @@ RSet::Internal::get_description() const
     set<Xapian::docid>::const_iterator i;
     for (i = items.begin(); i != items.end(); ++i) {
 	if (i != items.begin()) description += ", ";
-	description += om_tostring(*i);
+	description += str(*i);
     }
 
     description += ')';
@@ -134,7 +138,7 @@ MSetItem::get_description() const
 {
     string description;
 
-    description = om_tostring(did) + ", " + om_tostring(wt) + ", " +
+    description = str(did) + ", " + str(wt) + ", " +
 	    collapse_key;
 
     description = "Xapian::MSetItem(" + description + ")";
@@ -146,7 +150,7 @@ MSetItem::get_description() const
 
 // Methods for Xapian::MSet
 
-MSet::MSet() : internal(new MSet::Internal())
+MSet::MSet() : internal(new MSet::Internal)
 {
 }
 
@@ -171,7 +175,7 @@ MSet::operator=(const MSet &other)
 void
 MSet::fetch(const MSetIterator & beginiter, const MSetIterator & enditer) const
 {
-    DEBUGAPICALL(void, "Xapian::MSet::fetch", beginiter << ", " << enditer);
+    LOGCALL_VOID(API, "Xapian::MSet::fetch", beginiter | enditer);
     Assert(internal.get() != 0);
     if (beginiter != enditer)
 	internal->fetch_items(beginiter.index, enditer.index - 1);
@@ -180,7 +184,7 @@ MSet::fetch(const MSetIterator & beginiter, const MSetIterator & enditer) const
 void
 MSet::fetch(const MSetIterator & beginiter) const
 {
-    DEBUGAPICALL(void, "Xapian::MSet::fetch", beginiter);
+    LOGCALL_VOID(API, "Xapian::MSet::fetch", beginiter);
     Assert(internal.get() != 0);
     internal->fetch_items(beginiter.index, beginiter.index);
 }
@@ -188,7 +192,7 @@ MSet::fetch(const MSetIterator & beginiter) const
 void
 MSet::fetch() const
 {
-    DEBUGAPICALL(void, "Xapian::MSet::fetch", "");
+    LOGCALL_VOID(API, "Xapian::MSet::fetch", NO_ARGS);
     Assert(internal.get() != 0);
     if (!internal->items.empty())
 	internal->fetch_items(0, internal->items.size() - 1);
@@ -197,7 +201,7 @@ MSet::fetch() const
 percent
 MSet::convert_to_percent(Xapian::weight wt) const
 {
-    DEBUGAPICALL(Xapian::percent, "Xapian::MSet::convert_to_percent", wt);
+    LOGCALL(API, Xapian::percent, "Xapian::MSet::convert_to_percent", wt);
     Assert(internal.get() != 0);
     RETURN(internal->convert_to_percent_internal(wt));
 }
@@ -205,7 +209,7 @@ MSet::convert_to_percent(Xapian::weight wt) const
 percent
 MSet::convert_to_percent(const MSetIterator & it) const
 {
-    DEBUGAPICALL(Xapian::percent, "Xapian::MSet::convert_to_percent", it);
+    LOGCALL(API, Xapian::percent, "Xapian::MSet::convert_to_percent", it);
     Assert(internal.get() != 0);
     RETURN(internal->convert_to_percent_internal(it.get_weight()));
 }
@@ -213,21 +217,23 @@ MSet::convert_to_percent(const MSetIterator & it) const
 Xapian::doccount
 MSet::get_termfreq(const string &tname) const
 {
-    DEBUGAPICALL(Xapian::doccount, "Xapian::MSet::get_termfreq", tname);
+    LOGCALL(API, Xapian::doccount, "Xapian::MSet::get_termfreq", tname);
     map<string, Internal::TermFreqAndWeight>::const_iterator i;
     Assert(internal.get() != 0);
     i = internal->termfreqandwts.find(tname);
-    if (i == internal->termfreqandwts.end()) {
-	throw InvalidArgumentError("Term frequency of `" + tname +
-				     "' not available.");
+    if (i != internal->termfreqandwts.end()) {
+	RETURN(i->second.termfreq);
     }
-    RETURN(i->second.termfreq);
+    if (internal->enquire.get() == 0) {
+	throw InvalidOperationError("Can't get termfreq from an MSet which is not derived from a query.");
+    }
+    RETURN(internal->enquire->get_termfreq(tname));
 }
 
 Xapian::weight
 MSet::get_termweight(const string &tname) const
 {
-    DEBUGAPICALL(Xapian::weight, "Xapian::MSet::get_termweight", tname);
+    LOGCALL(API, Xapian::weight, "Xapian::MSet::get_termweight", tname);
     map<string, Internal::TermFreqAndWeight>::const_iterator i;
     Assert(internal.get() != 0);
     i = internal->termfreqandwts.find(tname);
@@ -266,6 +272,27 @@ MSet::get_matches_upper_bound() const
     return internal->matches_upper_bound;
 }
 
+Xapian::doccount
+MSet::get_uncollapsed_matches_lower_bound() const
+{
+    Assert(internal.get() != 0);
+    return internal->uncollapsed_lower_bound;
+}
+
+Xapian::doccount
+MSet::get_uncollapsed_matches_estimated() const
+{
+    Assert(internal.get() != 0);
+    return internal->uncollapsed_estimated;
+}
+
+Xapian::doccount
+MSet::get_uncollapsed_matches_upper_bound() const
+{
+    Assert(internal.get() != 0);
+    return internal->uncollapsed_upper_bound;
+}
+
 Xapian::weight
 MSet::get_max_possible() const
 {
@@ -339,14 +366,14 @@ MSet::get_description() const
 percent
 MSet::Internal::convert_to_percent_internal(Xapian::weight wt) const
 {
-    DEBUGAPICALL(Xapian::percent, "Xapian::MSet::Internal::convert_to_percent_internal", wt);
+    LOGCALL(MATCH, Xapian::percent, "Xapian::MSet::Internal::convert_to_percent_internal", wt);
     if (percent_factor == 0) RETURN(100);
 
     // Excess precision on x86 can result in a difference here.
     double v = wt * percent_factor + 100.0 * DBL_EPSILON;
     Xapian::percent pcent = static_cast<Xapian::percent>(v);
-    DEBUGLINE(API, "wt = " << wt << ", max_possible = "
-	      << max_possible << " =>  pcent = " << pcent);
+    LOGLINE(MATCH, "wt = " << wt << ", max_possible = " << max_possible <<
+		   " =>  pcent = " << pcent);
     if (pcent > 100) pcent = 100;
     if (pcent < 0) pcent = 0;
     if (pcent == 0 && wt > 0) pcent = 1;
@@ -357,8 +384,7 @@ MSet::Internal::convert_to_percent_internal(Xapian::weight wt) const
 Document
 MSet::Internal::get_doc_by_index(Xapian::doccount index) const
 {
-    DEBUGCALL(API, Document, "Xapian::MSet::Internal::Data::get_doc_by_index",
-	      index);
+    LOGCALL(MATCH, Document, "Xapian::MSet::Internal::get_doc_by_index", index);
     index += firstitem; 
     map<Xapian::doccount, Document>::const_iterator doc;
     doc = indexeddocs.find(index);
@@ -366,7 +392,7 @@ MSet::Internal::get_doc_by_index(Xapian::doccount index) const
 	RETURN(doc->second);
     }
     if (index < firstitem || index >= firstitem + items.size()) {
-	throw RangeError("The mset returned from the match does not contain the document at index " + om_tostring(index));
+	throw RangeError("The mset returned from the match does not contain the document at index " + str(index));
     }
     fetch_items(index, index); // FIXME: this checks indexeddocs AGAIN!
     /* Actually read the fetched documents */
@@ -379,8 +405,7 @@ MSet::Internal::get_doc_by_index(Xapian::doccount index) const
 void
 MSet::Internal::fetch_items(Xapian::doccount first, Xapian::doccount last) const
 {
-    DEBUGAPICALL(void, "Xapian::MSet::Internal::Data::fetch_items",
-		 first << ", " << last);
+    LOGCALL_VOID(MATCH, "Xapian::MSet::Internal::fetch_items", first | last);
     if (enquire.get() == 0) {
 	throw InvalidOperationError("Can't fetch documents from an MSet which is not derived from a query.");
     }
@@ -405,12 +430,12 @@ MSet::Internal::get_description() const
 {
     string description = "Xapian::MSet::Internal(";
 
-    description += "firstitem=" + om_tostring(firstitem) + ", " +
-	    "matches_lower_bound=" + om_tostring(matches_lower_bound) + ", " +
-	    "matches_estimated=" + om_tostring(matches_estimated) + ", " +
-	    "matches_upper_bound=" + om_tostring(matches_upper_bound) + ", " +
-	    "max_possible=" + om_tostring(max_possible) + ", " +
-	    "max_attained=" + om_tostring(max_attained);
+    description += "firstitem=" + str(firstitem) + ", " +
+	    "matches_lower_bound=" + str(matches_lower_bound) + ", " +
+	    "matches_estimated=" + str(matches_estimated) + ", " +
+	    "matches_upper_bound=" + str(matches_upper_bound) + ", " +
+	    "max_possible=" + str(max_possible) + ", " +
+	    "max_attained=" + str(max_attained);
 
     for (vector<Xapian::Internal::MSetItem>::const_iterator i = items.begin();
 	 i != items.end(); ++i) {
@@ -429,23 +454,15 @@ MSet::Internal::read_docs() const
     set<Xapian::doccount>::const_iterator i;
     for (i = requested_docs.begin(); i != requested_docs.end(); ++i) {
 	indexeddocs[*i] = enquire->read_doc(items[*i - firstitem]);
-	DEBUGLINE(API, "stored doc at index " << *i << " is " << indexeddocs[*i]);
+	LOGLINE(MATCH, "stored doc at index " << *i << " is " << indexeddocs[*i]);
     }
     /* Clear list of requested but not fetched documents. */
     requested_docs.clear();
 }
 
-// Methods for Xapian::Internal::ESetItem
-
-string
-Xapian::Internal::ESetItem::get_description() const
-{
-    return "Xapian::Internal::ESetItem(" + tname + ", " + om_tostring(wt) + ")";
-}
-
 // Methods for Xapian::ESet
 
-ESet::ESet() : internal(new Internal()) { }
+ESet::ESet() : internal(new Internal) { }
 
 ESet::~ESet()
 {
@@ -521,30 +538,12 @@ ESet::get_description() const
     return "Xapian::ESet(" + internal->get_description() + ")";
 }
 
-//////////////////////////////////
-// Methods for Xapian::ESet::Internal //
-//////////////////////////////////
-
-string
-Xapian::ESet::Internal::get_description() const
-{
-    string description = "ebound=" + om_tostring(ebound);
-
-    for (vector<Xapian::Internal::ESetItem>::const_iterator i = items.begin();
-	 i != items.end();
-	 i++) {
-	description += ", " + i->get_description();
-    }
-
-    return "Xapian::ESet::Internal(" + description + ")";
-}
-
 // Xapian::ESetIterator
 
 const string &
 ESetIterator::operator *() const
 {
-    return eset.internal->items[index].tname;
+    return eset.internal->items[index].term;
 }
 
 Xapian::weight
@@ -556,7 +555,7 @@ ESetIterator::get_weight() const
 string
 ESetIterator::get_description() const
 {
-    return "Xapian::ESetIterator(" + om_tostring(index) + ")";
+    return "Xapian::ESetIterator(" + str(index) + ")";
 }
 
 // MSetIterator
@@ -594,20 +593,20 @@ MSetIterator::get_collapse_count() const
 Xapian::percent
 MSetIterator::get_percent() const
 {
-    DEBUGAPICALL(Xapian::percent, "MSetIterator::get_percent", "");
+    LOGCALL(API, Xapian::percent, "MSetIterator::get_percent", NO_ARGS);
     RETURN(mset.internal->convert_to_percent_internal(get_weight()));
 }
 
 string
 MSetIterator::get_description() const
 {
-    return "Xapian::MSetIterator(" + om_tostring(index) + ")";
+    return "Xapian::MSetIterator(" + str(index) + ")";
 }
 
 // Methods for Xapian::Enquire::Internal
 
 Enquire::Internal::Internal(const Database &db_, ErrorHandler * errorhandler_)
-  : db(db_), query(), collapse_key(Xapian::BAD_VALUENO),
+  : db(db_), query(), collapse_key(Xapian::BAD_VALUENO), collapse_max(0),
     order(Enquire::ASCENDING), percent_cutoff(0), weight_cutoff(0),
     sort_key(Xapian::BAD_VALUENO), sort_by(REL), sort_value_forward(true),
     sorter(0), errorhandler(errorhandler_), weight(0)
@@ -640,11 +639,9 @@ MSet
 Enquire::Internal::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 			    Xapian::doccount check_at_least, const RSet *rset,
 			    const MatchDecider *mdecider,
-			    const MatchDecider *matchspy) const
+			    const MatchDecider *matchspy_legacy) const
 {
-    DEBUGCALL(API, MSet, "Enquire::Internal::get_mset", first << ", " <<
-	      maxitems << ", " << check_at_least << ", " << rset << ", " <<
-	      mdecider << ", " << matchspy);
+    LOGCALL(MATCH, MSet, "Enquire::Internal::get_mset", first | maxitems | check_at_least | rset | mdecider | matchspy_legacy);
 
     if (percent_cutoff && (sort_by == VAL || sort_by == VAL_REL)) {
 	throw Xapian::UnimplementedError("Use of a percentage cutoff while sorting primary by value isn't currently supported");
@@ -661,17 +658,18 @@ Enquire::Internal::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	check_at_least = max(check_at_least, maxitems);
     }
 
-    Stats stats;
-    ::MultiMatch match(db, query.internal.get(), qlen, rset, collapse_key,
+    Xapian::Weight::Internal stats;
+    ::MultiMatch match(db, query.internal.get(), qlen, rset,
+		       collapse_max, collapse_key,
 		       percent_cutoff, weight_cutoff,
 		       order, sort_key, sort_by, sort_value_forward,
-		       errorhandler, stats, weight,
+		       errorhandler, stats, weight, spies,
 		       (sorter != NULL),
-		       (mdecider != NULL || matchspy != NULL));
+		       (mdecider != NULL || matchspy_legacy != NULL));
     // Run query and put results into supplied Xapian::MSet object.
     MSet retval;
     match.get_mset(first, maxitems, check_at_least, retval,
-		   stats, mdecider, matchspy, sorter);
+		   stats, mdecider, matchspy_legacy, sorter);
 
     Assert(weight->name() != "bool" || retval.get_max_possible() == 0);
 
@@ -689,12 +687,15 @@ Enquire::Internal::get_eset(Xapian::termcount maxitems,
                     const RSet & rset, int flags, double k,
 		    const ExpandDecider * edecider) const
 {
-    ESet retval;
+    LOGCALL(MATCH, ESet, "Enquire::Internal::get_eset", maxitems | rset | flags | k | edecider);
 
-    OmExpand expand(db);
-    RSetI rseti(db, rset);
+    if (maxitems == 0 || rset.empty()) {
+	// Either we were asked for no results, or wouldn't produce any
+	// because no documents were marked as relevant.
+	RETURN(ESet());
+    }
 
-    DEBUGLINE(API, "rset size is " << rset.size());
+    LOGVALUE(MATCH, rset.size());
 
     /* The AutoPtrs will clean up any dynamically allocated
      * expand deciders automatically.
@@ -718,10 +719,12 @@ Enquire::Internal::get_eset(Xapian::termcount maxitems,
 	}
     }
 
-    expand.expand(maxitems, retval, &rseti, edecider,
-		  bool(flags & Enquire::USE_EXACT_TERMFREQ), k);
+    bool use_exact_termfreq(flags & Enquire::USE_EXACT_TERMFREQ);
+    ExpandWeight eweight(db, rset.size(), use_exact_termfreq, k);
 
-    return retval;
+    Xapian::ESet eset;
+    eset.internal->expand(maxitems, db, rset, edecider, eweight);
+    RETURN(eset);
 }
 
 class ByQueryIndexCmp {
@@ -757,7 +760,7 @@ Enquire::Internal::get_matching_terms(Xapian::docid did) const
     // a problem.
     map<string, unsigned int> tmap;
     unsigned int index = 1;
-    for ( ; qt != qt_end; qt++) {
+    for ( ; qt != qt_end; ++qt) {
 	if (tmap.find(*qt) == tmap.end())
 	    tmap[*qt] = index++;
     }
@@ -788,6 +791,12 @@ Enquire::Internal::get_matching_terms(const MSetIterator &it) const
     return get_matching_terms(*it);
 }
 
+Xapian::doccount
+Enquire::Internal::get_termfreq(const string &tname) const
+{
+    return db.get_termfreq(tname);
+}
+
 string
 Enquire::Internal::get_description() const
 {
@@ -843,38 +852,38 @@ Enquire::Internal::register_match_decider(const string &,
 
 Enquire::Enquire(const Enquire & other) : internal(other.internal)
 {
-    DEBUGAPICALL(void, "Xapian::Enquire::Enquire", other);
+    LOGCALL_VOID(API, "Xapian::Enquire::Enquire", other);
 }
 
 void
 Enquire::operator=(const Enquire & other)
 {
-    DEBUGAPICALL(void, "Xapian::Enquire::operator=", other);
+    LOGCALL_VOID(API, "Xapian::Enquire::operator=", other);
     internal = other.internal;
 }
 
 Enquire::Enquire(const Database &databases, ErrorHandler * errorhandler)
     : internal(new Internal(databases, errorhandler))
 {
-    DEBUGAPICALL(void, "Xapian::Enquire::Enquire", databases);
+    LOGCALL_VOID(API, "Xapian::Enquire::Enquire", databases);
 }
 
 Enquire::~Enquire()
 {
-    DEBUGAPICALL(void, "Xapian::Enquire::~Enquire", "");
+    LOGCALL_VOID(API, "Xapian::Enquire::~Enquire", NO_ARGS);
 }
 
 void
 Enquire::set_query(const Query & query, termcount len)
 {
-    DEBUGAPICALL(void, "Xapian::Enquire::set_query", query << ", " << len);
+    LOGCALL_VOID(API, "Xapian::Enquire::set_query", query | len);
     internal->set_query(query, len);
 }
 
 const Query &
 Enquire::get_query() const
 {
-    DEBUGAPICALL(const Xapian::Query &, "Xapian::Enquire::get_query", "");
+    LOGCALL(API, const Xapian::Query &, "Xapian::Enquire::get_query", NO_ARGS);
     try {
 	RETURN(internal->get_query());
     } catch (Error & e) {
@@ -884,17 +893,33 @@ Enquire::get_query() const
 }
 
 void
-Enquire::set_weighting_scheme(const Weight &weight_)
-{
-    DEBUGAPICALL(void, "Xapian::Enquire::set_weighting_scheme", "[Weight]");
-    delete internal->weight;
-    internal->weight = weight_.clone();
+Enquire::add_matchspy(MatchSpy * spy) {
+    LOGCALL_VOID(API, "Xapian::Enquire::add_matchspy", spy);
+    internal->spies.push_back(spy);
 }
 
 void
-Enquire::set_collapse_key(Xapian::valueno collapse_key)
+Enquire::clear_matchspies() {
+    LOGCALL(API, const Xapian::Query &, "Xapian::Enquire::clear_matchspies", NO_ARGS);
+    internal->spies.clear();
+}
+
+void
+Enquire::set_weighting_scheme(const Weight &weight_)
 {
+    LOGCALL_VOID(API, "Xapian::Enquire::set_weighting_scheme", weight_);
+    // Clone first in case doing so throws an exception.
+    Weight * wt = weight_.clone();
+    swap(wt, internal->weight);
+    delete wt;
+}
+
+void
+Enquire::set_collapse_key(Xapian::valueno collapse_key, Xapian::doccount collapse_max)
+{
+    if (collapse_key == Xapian::BAD_VALUENO) collapse_max = 0;
     internal->collapse_key = collapse_key;
+    internal->collapse_max = collapse_max;
 }
 
 void
@@ -917,7 +942,7 @@ Enquire::set_sort_by_relevance()
 }
 
 void
-Enquire::set_sort_by_value(Xapian::valueno sort_key, bool ascending)
+Enquire::set_sort_by_value(valueno sort_key, bool ascending)
 {
     internal->sorter = NULL;
     internal->sort_key = sort_key;
@@ -926,8 +951,7 @@ Enquire::set_sort_by_value(Xapian::valueno sort_key, bool ascending)
 }
 
 void
-Enquire::set_sort_by_value_then_relevance(Xapian::valueno sort_key,
-					  bool ascending)
+Enquire::set_sort_by_value_then_relevance(valueno sort_key, bool ascending)
 {
     internal->sorter = NULL;
     internal->sort_key = sort_key;
@@ -936,8 +960,7 @@ Enquire::set_sort_by_value_then_relevance(Xapian::valueno sort_key,
 }
 
 void
-Enquire::set_sort_by_relevance_then_value(Xapian::valueno sort_key,
-					  bool ascending)
+Enquire::set_sort_by_relevance_then_value(valueno sort_key, bool ascending)
 {
     internal->sorter = NULL;
     internal->sort_key = sort_key;
@@ -946,27 +969,27 @@ Enquire::set_sort_by_relevance_then_value(Xapian::valueno sort_key,
 }
 
 void
-Enquire::set_sort_by_key(Xapian::Sorter * sorter, bool ascending)
+Enquire::set_sort_by_key(KeyMaker * sorter, bool ascending)
 {
     if (sorter == NULL)
-	throw Xapian::InvalidArgumentError("sorter can't be NULL");
+	throw InvalidArgumentError("sorter can't be NULL");
     internal->sorter = sorter;
     internal->sort_by = Internal::VAL;
     internal->sort_value_forward = ascending;
 }
 
 void
-Enquire::set_sort_by_key_then_relevance(Xapian::Sorter * sorter, bool ascending)
+Enquire::set_sort_by_key_then_relevance(KeyMaker * sorter, bool ascending)
 {
     if (sorter == NULL)
-	throw Xapian::InvalidArgumentError("sorter can't be NULL");
+	throw InvalidArgumentError("sorter can't be NULL");
     internal->sorter = sorter;
     internal->sort_by = Internal::VAL_REL;
     internal->sort_value_forward = ascending;
 }
 
 void
-Enquire::set_sort_by_relevance_then_key(Xapian::Sorter * sorter, bool ascending)
+Enquire::set_sort_by_relevance_then_key(KeyMaker * sorter, bool ascending)
 {
     if (sorter == NULL)
 	throw Xapian::InvalidArgumentError("sorter can't be NULL");
@@ -975,14 +998,6 @@ Enquire::set_sort_by_relevance_then_key(Xapian::Sorter * sorter, bool ascending)
     internal->sort_value_forward = ascending;
 }
 
-MSet
-Enquire::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
-		  Xapian::doccount check_at_least, const RSet *rset,
-		  const MatchDecider *mdecider) const
-{
-    return get_mset(first, maxitems, check_at_least, rset, mdecider, NULL);
-}
-
 MSet
 Enquire::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		  Xapian::doccount check_at_least, const RSet *rset,
@@ -990,9 +1005,7 @@ Enquire::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		  const MatchDecider *matchspy) const
 {
     // FIXME: display contents of pointer params, if they're not null.
-    DEBUGAPICALL(Xapian::MSet, "Xapian::Enquire::get_mset", first << ", " <<
-		 maxitems << ", " << check_at_least << ", " << rset << ", " <<
-		 mdecider << ", " << matchspy);
+    LOGCALL(API, Xapian::MSet, "Xapian::Enquire::get_mset", first | maxitems | check_at_least | rset | mdecider | matchspy);
 
     try {
 	RETURN(internal->get_mset(first, maxitems, check_at_least, rset,
@@ -1003,14 +1016,30 @@ Enquire::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     }
 }
 
+MSet
+Enquire::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
+		  Xapian::doccount check_at_least, const RSet *rset,
+		  const MatchDecider *mdecider) const
+{
+    // FIXME: display contents of pointer params, if they're not null.
+    LOGCALL(API, Xapian::MSet, "Xapian::Enquire::get_mset", first | maxitems | check_at_least | rset | mdecider);
+
+    try {
+	RETURN(internal->get_mset(first, maxitems, check_at_least, rset,
+				  mdecider, NULL));
+    } catch (Error & e) {
+	if (internal->errorhandler) (*internal->errorhandler)(e);
+	throw;
+    }
+}
+
 ESet
 Enquire::get_eset(Xapian::termcount maxitems, const RSet & rset, int flags,
-		    double k, const ExpandDecider * edecider) const
+		  double k, const ExpandDecider * edecider) const
 {
     // FIXME: display contents of pointer params and rset, if they're not
     // null.
-    DEBUGAPICALL(Xapian::ESet, "Xapian::Enquire::get_eset", maxitems << ", " <<
-		 rset << ", " << flags << ", " << k << ", " << edecider);
+    LOGCALL(API, Xapian::ESet, "Xapian::Enquire::get_eset", maxitems | rset | flags | k | edecider);
 
     try {
 	// FIXME: this copies the eset too much: pass it in by reference?
@@ -1024,7 +1053,7 @@ Enquire::get_eset(Xapian::termcount maxitems, const RSet & rset, int flags,
 TermIterator
 Enquire::get_matching_terms_begin(const MSetIterator &it) const
 {
-    DEBUGAPICALL(Xapian::TermIterator, "Xapian::Enquire::get_matching_terms", it);
+    LOGCALL(API, Xapian::TermIterator, "Xapian::Enquire::get_matching_terms", it);
     try {
 	RETURN(internal->get_matching_terms(it));
     } catch (Error & e) {
@@ -1036,7 +1065,7 @@ Enquire::get_matching_terms_begin(const MSetIterator &it) const
 TermIterator
 Enquire::get_matching_terms_begin(Xapian::docid did) const
 {
-    DEBUGAPICALL(Xapian::TermIterator, "Xapian::Enquire::get_matching_terms", did);
+    LOGCALL(API, Xapian::TermIterator, "Xapian::Enquire::get_matching_terms", did);
     try {
 	RETURN(internal->get_matching_terms(did));
     } catch (Error & e) {
@@ -1045,13 +1074,6 @@ Enquire::get_matching_terms_begin(Xapian::docid did) const
     }
 }
 
-void
-Enquire::register_match_decider(const string &name,
-				  const MatchDecider *mdecider)
-{
-    internal->register_match_decider(name, mdecider);
-}
-
 string
 Enquire::get_description() const
 {
