@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
  * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
- * Copyright 2006,2007,2008 Lemur Consulting Ltd
+ * Copyright 2006,2007,2008,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,24 +23,27 @@
 
 #include <config.h>
 
-#include "autoptr.h"
-#include "omdebug.h"
 #include "omqueryinternal.h"
-#include "utils.h"
+
+#include "debuglog.h"
+#include "registryinternal.h"
 #include "serialise.h"
 #include "serialise-double.h"
+#include "str.h"
 
 #include <xapian/error.h>
+#include <xapian/postingsource.h>
 #include <xapian/termiterator.h>
 #include <xapian/version.h>
 #include "vectortermlist.h"
 
-#include <vector>
-#include <set>
 #include <algorithm>
-#include <math.h>
-#include <limits.h>
+#include "autoptr.h"
 #include <cfloat>
+#include <climits>
+#include <cmath>
+#include <set>
+#include <vector>
 
 using namespace std;
 
@@ -50,6 +53,7 @@ static unsigned int
 get_min_subqs(Xapian::Query::Internal::op_t op)
 {
     switch (op) {
+	case Xapian::Query::Internal::OP_EXTERNAL_SOURCE:
 	case Xapian::Query::Internal::OP_LEAF:
 	case Xapian::Query::OP_AND:
 	case Xapian::Query::OP_OR:
@@ -60,6 +64,7 @@ get_min_subqs(Xapian::Query::Internal::op_t op)
 	case Xapian::Query::OP_VALUE_RANGE:
 	case Xapian::Query::OP_VALUE_GE:
 	case Xapian::Query::OP_VALUE_LE:
+	case Xapian::Query::OP_SYNONYM:
 	    return 0;
 	case Xapian::Query::OP_SCALE_WEIGHT:
 	    return 1;
@@ -77,6 +82,7 @@ static unsigned int
 get_max_subqs(Xapian::Query::Internal::op_t op)
 {
     switch (op) {
+	case Xapian::Query::Internal::OP_EXTERNAL_SOURCE:
 	case Xapian::Query::Internal::OP_LEAF:
 	case Xapian::Query::OP_VALUE_RANGE:
 	case Xapian::Query::OP_VALUE_GE:
@@ -94,6 +100,7 @@ get_max_subqs(Xapian::Query::Internal::op_t op)
 	case Xapian::Query::OP_NEAR:
 	case Xapian::Query::OP_PHRASE:
 	case Xapian::Query::OP_ELITE_SET:
+	case Xapian::Query::OP_SYNONYM:
 	    return UINT_MAX;
 	default:
 	    Assert(false);
@@ -107,6 +114,20 @@ is_leaf(Xapian::Query::Internal::op_t op)
     return (op == Xapian::Query::Internal::OP_LEAF);
 }
 
+inline bool
+is_distributable(Xapian::Query::Internal::op_t op)
+{
+    switch (op) {
+	case Xapian::Query::OP_AND:
+	case Xapian::Query::OP_OR:
+	case Xapian::Query::OP_XOR:
+	case Xapian::Query::OP_SYNONYM:
+	    return true;
+	default:
+	    return false;
+    }
+}
+
 // Methods for Xapian::Query::Internal
 
 /** serialising method, for network matches.
@@ -137,9 +158,20 @@ Xapian::Query::Internal::serialise(Xapian::termpos & curpos) const
 	result += '[';
 	result += encode_length(tname.length());
 	result += tname;
-	if (term_pos != curpos) result += '@' + om_tostring(term_pos);
-	if (wqf != 1) result += '#' + om_tostring(wqf);
+	if (term_pos != curpos) result += '@' + encode_length(term_pos);
+	// parameter is wqf.
+	if (parameter != 1) result += '#' + encode_length(parameter);
 	++curpos;
+    } else if (op == Xapian::Query::Internal::OP_EXTERNAL_SOURCE) {
+	string sourcename = external_source->name();
+	if (sourcename.empty())
+	    throw Xapian::UnimplementedError("This PostingSource doesn't support remote use.");
+	result += '!';
+	result += encode_length(sourcename.length());
+	result += sourcename;
+	string sourcedata = external_source->serialise();
+	result += encode_length(sourcedata.length());
+	result += sourcedata;
     } else {
 	result += "(";
 	for (subquery_list::const_iterator i = subqs.begin();
@@ -151,6 +183,9 @@ Xapian::Query::Internal::serialise(Xapian::termpos & curpos) const
 	    case Xapian::Query::Internal::OP_LEAF:
 		Assert(false);
 		break;
+	    case Xapian::Query::Internal::OP_EXTERNAL_SOURCE:
+		Assert(false);
+		break;
 	    case Xapian::Query::OP_AND:
 		result += "&";
 		break;
@@ -170,13 +205,13 @@ Xapian::Query::Internal::serialise(Xapian::termpos & curpos) const
 		result += "^";
 		break;
 	    case Xapian::Query::OP_NEAR:
-		result += "~" + om_tostring(parameter);
+		result += "~" + encode_length(parameter);
 		break;
 	    case Xapian::Query::OP_PHRASE:
-		result += "\"" + om_tostring(parameter);
+		result += "\"" + encode_length(parameter);
 		break;
 	    case Xapian::Query::OP_ELITE_SET:
-		result += "*" + om_tostring(parameter);
+		result += "*" + encode_length(parameter);
 		break;
 	    case Xapian::Query::OP_VALUE_RANGE:
 		result += "]";
@@ -184,24 +219,27 @@ Xapian::Query::Internal::serialise(Xapian::termpos & curpos) const
 		result += tname;
 		result += encode_length(str_parameter.length());
 		result += str_parameter;
-		result += om_tostring(parameter);
+		result += encode_length(parameter);
 		break;
 	    case Xapian::Query::OP_VALUE_GE:
 		result += "}";
 		result += encode_length(tname.length());
 		result += tname;
-		result += om_tostring(parameter);
+		result += encode_length(parameter);
 		break;
 	    case Xapian::Query::OP_VALUE_LE:
 		result += "{";
 		result += encode_length(tname.length());
 		result += tname;
-		result += om_tostring(parameter);
+		result += encode_length(parameter);
 		break;
 	    case Xapian::Query::OP_SCALE_WEIGHT:
 		result += ".";
 		result += str_parameter; // serialise_double(get_dbl_parameter());
 		break;
+	    case Xapian::Query::OP_SYNONYM:
+		result += "=";
+		break;
 	}
     }
     return result;
@@ -216,6 +254,8 @@ Xapian::Query::Internal::get_op_name(Xapian::Query::Internal::op_t op)
 {
     string name;
     switch (op) {
+	case Xapian::Query::Internal::OP_EXTERNAL_SOURCE:
+	    name = "EXTERNAL_SOURCE"; break;
 	case Xapian::Query::Internal::OP_LEAF:  name = "LEAF"; break;
 	case Xapian::Query::OP_AND:             name = "AND"; break;
 	case Xapian::Query::OP_OR:              name = "OR"; break;
@@ -230,6 +270,7 @@ Xapian::Query::Internal::get_op_name(Xapian::Query::Internal::op_t op)
 	case Xapian::Query::OP_VALUE_GE:        name = "VALUE_GE"; break;
 	case Xapian::Query::OP_VALUE_LE:        name = "VALUE_LE"; break;
 	case Xapian::Query::OP_SCALE_WEIGHT:    name = "SCALE_WEIGHT"; break;
+	case Xapian::Query::OP_SYNONYM:         name = "SYNONYM"; break;
     }
     return name;
 }
@@ -241,11 +282,12 @@ Xapian::Query::Internal::get_description() const
 
     if (is_leaf(op)) {
 	if (term_pos != 0) {
-	    opstr += "pos=" + om_tostring(term_pos);
+	    opstr += "pos=" + str(term_pos);
 	}
-	if (wqf != 1) {
+	// parameter is wqf.
+	if (parameter != 1) {
 	    if (!opstr.empty()) opstr += ",";
-	    opstr += "wqf=" + om_tostring(wqf);
+	    opstr += "wqf=" + str(parameter);
 	}
 	if (!opstr.empty()) opstr = ":(" + opstr + ")";
 	if (tname.empty()) return "<alldocuments>" + opstr;
@@ -253,38 +295,40 @@ Xapian::Query::Internal::get_description() const
     }
 
     switch (op) {
-	case Xapian::Query::OP_VALUE_RANGE: {
+	case Xapian::Query::OP_VALUE_RANGE:
 	    opstr = get_op_name(op);
 	    opstr += ' ';
-	    opstr += om_tostring(parameter);
+	    opstr += str(parameter);
 	    opstr += ' ';
 	    opstr += tname;
 	    opstr += ' ';
 	    opstr += str_parameter;
 	    return opstr;
-	}
 	case Xapian::Query::OP_VALUE_GE:
-	case Xapian::Query::OP_VALUE_LE: {
+	case Xapian::Query::OP_VALUE_LE:
 	    opstr = get_op_name(op);
 	    opstr += ' ';
-	    opstr += om_tostring(parameter);
+	    opstr += str(parameter);
 	    opstr += ' ';
 	    opstr += tname;
 	    return opstr;
-	}
-	case Xapian::Query::OP_SCALE_WEIGHT: {
-	    opstr += om_tostring(get_dbl_parameter());
+	case Xapian::Query::OP_SCALE_WEIGHT:
+	    opstr += str(get_dbl_parameter());
 	    opstr += " * ";
 	    opstr += subqs[0]->get_description();
 	    return opstr;
-	}
+	case Xapian::Query::Internal::OP_EXTERNAL_SOURCE:
+	    opstr = "PostingSource(";
+	    opstr += external_source->get_description();
+	    opstr += ')';
+	    return opstr;
     }
 
     opstr = " " + get_op_name(op) + " ";
     if (op == Xapian::Query::OP_NEAR ||
 	op == Xapian::Query::OP_PHRASE ||
 	op == Xapian::Query::OP_ELITE_SET)
-	opstr += om_tostring(parameter) + " ";
+	opstr += str(parameter) + " ";
 
     string description;
     subquery_list::const_iterator i;
@@ -299,7 +343,10 @@ Xapian::Query::Internal::get_description() const
 Xapian::termcount
 Xapian::Query::Internal::get_length() const
 {
-    if (is_leaf(op)) return wqf;
+    if (is_leaf(op)) {
+	// parameter is wqf.
+	return parameter;
+    }
     Xapian::termcount len = 0;
     subquery_list::const_iterator i;
     for (i = subqs.begin(); i != subqs.end(); ++i) {
@@ -371,18 +418,21 @@ class QUnserial {
     const char *p;
     const char *end;
     Xapian::termpos curpos;
+    const Xapian::Registry & reg;
 
     Xapian::Query::Internal * readquery();
+    Xapian::Query::Internal * readexternal();
     Xapian::Query::Internal * readcompound();
 
   public:
-    QUnserial(const string & s) : p(s.c_str()), end(p + s.size()), curpos(1) { }
+    QUnserial(const string & s, const Xapian::Registry & reg_)
+	    : p(s.c_str()), end(p + s.size()), curpos(1), reg(reg_) { }
     Xapian::Query::Internal * decode();
 };
 
 Xapian::Query::Internal *
 QUnserial::decode() {
-    DEBUGLINE(UNKNOWN, "QUnserial::decode(" << p << ")");
+    LOGLINE(UNKNOWN, "QUnserial::decode(" << p << ")");
     AutoPtr<Xapian::Query::Internal> qint(readquery());
     if (p != end)
         throw Xapian::InvalidArgumentError("Bad serialised query");
@@ -402,27 +452,49 @@ QUnserial::readquery() {
 	    Xapian::termcount wqf = 1;
 	    if (p != end) {
 		if (*p == '@') {
-		    char *tmp; // avoid compiler warning
-		    term_pos = strtol(p + 1, &tmp, 10);
-		    p = tmp;
+		    ++p;
+		    term_pos = decode_length(&p, end, false);
 		}
 		if (*p == '#') {
-		    char *tmp; // avoid compiler warning
-		    wqf = strtol(p + 1, &tmp, 10);
-		    p = tmp;
+		    ++p;
+		    wqf = decode_length(&p, end, false);
 		}
 	    }
 	    ++curpos;
 	    return new Xapian::Query::Internal(tname, wqf, term_pos);
 	}
+	case '!':
+	    return readexternal();
 	case '(':
 	    return readcompound();
 	default:
-	    DEBUGLINE(UNKNOWN, "Can't parse remainder `" << p - 1 << "'");
+	    LOGLINE(UNKNOWN, "Can't parse remainder `" << p - 1 << "'");
 	    throw Xapian::InvalidArgumentError("Invalid query string");
     }
 }
 
+Xapian::Query::Internal *
+QUnserial::readexternal()
+{
+    if (p == end)
+	throw Xapian::InvalidArgumentError("Bad serialised query");
+
+    size_t length = decode_length(&p, end, true);
+    string sourcename(p, length);
+    const Xapian::PostingSource * source = reg.get_posting_source(sourcename);
+    if (source == NULL) {
+	throw Xapian::InvalidArgumentError("PostingSource " + sourcename +
+					   " not registered");
+    }
+
+    p += length;
+    length = decode_length(&p, end, true);
+    string sourcedata(p, length);
+    p += length;
+
+    return new Xapian::Query::Internal(source->unserialise(sourcedata), true);
+}
+
 static Xapian::Query::Internal *
 qint_from_vector(Xapian::Query::op op,
 		 const vector<Xapian::Query::Internal *> & vec,
@@ -471,6 +543,9 @@ QUnserial::readcompound() {
 		    --p;
 		    subqs.push_back(readquery());
 		    break;
+		case '!':
+		    subqs.push_back(readexternal());
+		    break;
 	        case '(': {
 		    subqs.push_back(readcompound());
 		    break;
@@ -488,21 +563,15 @@ QUnserial::readcompound() {
 	        case '-':
 		    return qint_from_vector(Xapian::Query::OP_AND_NOT, subqs);
 	        case '~': {
-		    char *tmp; // avoid compiler warning
-		    Xapian::termcount window(strtol(p, &tmp, 10));
-		    p = tmp;
+		    Xapian::termcount window(decode_length(&p, end, false));
 		    return qint_from_vector(Xapian::Query::OP_NEAR, subqs, window);
 	        }
 	        case '"': {
-		    char *tmp; // avoid compiler warning
-		    Xapian::termcount window(strtol(p, &tmp, 10));
-		    p = tmp;
+		    Xapian::termcount window(decode_length(&p, end, false));
 		    return qint_from_vector(Xapian::Query::OP_PHRASE, subqs, window);
 	        }
 	        case '*': {
-		    char *tmp; // avoid compiler warning
-		    Xapian::termcount elite_set_size(strtol(p, &tmp, 10));
-		    p = tmp;
+		    Xapian::termcount elite_set_size(decode_length(&p, end, false));
 		    return qint_from_vector(Xapian::Query::OP_ELITE_SET, subqs,
 					    elite_set_size);
 		}
@@ -513,9 +582,7 @@ QUnserial::readcompound() {
 		    len = decode_length(&p, end, true);
 		    string stop(p, len);
 		    p += len;
-		    char *tmp; // avoid compiler warning
-		    Xapian::valueno valno = strtoul(p, &tmp, 10);
-		    p = tmp;
+		    Xapian::valueno valno(decode_length(&p, end, false));
 		    return new Xapian::Query::Internal(Xapian::Query::OP_VALUE_RANGE, valno,
 						       start, stop);
 	        }
@@ -523,9 +590,7 @@ QUnserial::readcompound() {
 		    size_t len = decode_length(&p, end, true);
 		    string start(p, len);
 		    p += len;
-		    char *tmp; // avoid compiler warning
-		    Xapian::valueno valno = strtoul(p, &tmp, 10);
-		    p = tmp;
+		    Xapian::valueno valno(decode_length(&p, end, false));
 		    return new Xapian::Query::Internal(Xapian::Query::OP_VALUE_GE, valno,
 						       start);
 	        }
@@ -533,9 +598,7 @@ QUnserial::readcompound() {
 		    size_t len = decode_length(&p, end, true);
 		    string start(p, len);
 		    p += len;
-		    char *tmp; // avoid compiler warning
-		    Xapian::valueno valno = strtoul(p, &tmp, 10);
-		    p = tmp;
+		    Xapian::valueno valno(decode_length(&p, end, false));
 		    return new Xapian::Query::Internal(Xapian::Query::OP_VALUE_LE, valno,
 						       start);
 	        }
@@ -544,8 +607,11 @@ QUnserial::readcompound() {
 		    return qint_from_vector(Xapian::Query::OP_SCALE_WEIGHT,
 					    subqs, 0, param);
 		}
-	        default:
-		    DEBUGLINE(UNKNOWN, "Can't parse remainder `" << p - 1 << "'");
+		case '=': {
+		    return qint_from_vector(Xapian::Query::OP_SYNONYM, subqs);
+		}
+		default:
+		    LOGLINE(UNKNOWN, "Can't parse remainder `" << p - 1 << "'");
 		    throw Xapian::InvalidArgumentError("Invalid query string");
 	    }
         }
@@ -558,17 +624,18 @@ QUnserial::readcompound() {
 }
 
 Xapian::Query::Internal *
-Xapian::Query::Internal::unserialise(const string &s)
+Xapian::Query::Internal::unserialise(const string &s,
+				     const Xapian::Registry & reg)
 {
     Assert(s.length() > 1);
-    QUnserial u(s);
+    QUnserial u(s, reg);
     Xapian::Query::Internal * qint = u.decode();
     AssertEq(s, qint->serialise());
     return qint;
 }
 #else
 Xapian::Query::Internal *
-Xapian::Query::Internal::unserialise(const string &)
+Xapian::Query::Internal::unserialise(const string &, const Xapian::Registry &)
 {
     throw Xapian::InternalError("query serialisation not compiled in");
 }
@@ -582,13 +649,23 @@ Xapian::Query::Internal::Internal(const Xapian::Query::Internal &copyme)
 	  tname(copyme.tname),
 	  str_parameter(copyme.str_parameter),
 	  term_pos(copyme.term_pos),
-	  wqf(copyme.wqf)
+	  external_source(NULL),
+	  external_source_owned(false)
 {
     for (subquery_list::const_iterator i = copyme.subqs.begin();
 	 i != copyme.subqs.end();
 	 ++i) {
 	subqs.push_back(new Xapian::Query::Internal(**i));
     }
+    if (copyme.external_source) {
+	external_source = copyme.external_source->clone();
+	if (external_source == NULL) {
+	    external_source = copyme.external_source;
+	    external_source_owned = false;
+	} else {
+	    external_source_owned = true;
+	}
+    }
 }
 
 //////////////////////////////////////////
@@ -598,10 +675,11 @@ Xapian::Query::Internal::Internal(const string & tname_, Xapian::termcount wqf_,
 		 Xapian::termpos term_pos_)
 	: op(Xapian::Query::Internal::OP_LEAF),
 	  subqs(),
-	  parameter(0),
+	  parameter(wqf_),
 	  tname(tname_),
 	  term_pos(term_pos_),
-	  wqf(wqf_)
+	  external_source(NULL),
+	  external_source_owned(false)
 {
     validate_query();
 }
@@ -612,7 +690,8 @@ Xapian::Query::Internal::Internal(op_t op_, Xapian::termcount parameter_)
 	  parameter(parameter_),
 	  tname(),
 	  term_pos(0),
-	  wqf(0)
+	  external_source(NULL),
+	  external_source_owned(false)
 {
     if (parameter != 0 && op != OP_PHRASE && op != OP_NEAR && op != OP_ELITE_SET)
 	throw Xapian::InvalidArgumentError("parameter is only meaningful for OP_NEAR, OP_PHRASE, or OP_ELITE_SET");
@@ -623,7 +702,9 @@ Xapian::Query::Internal::Internal(op_t op_, Xapian::valueno valno,
 	: op(op_),
 	  parameter(Xapian::termcount(valno)),
 	  tname(begin),
-	  str_parameter(end)
+	  str_parameter(end),
+	  external_source(NULL),
+	  external_source_owned(false)
 {
     if (op != OP_VALUE_RANGE)
 	throw Xapian::InvalidArgumentError("This constructor is only meaningful for OP_VALUE_RANGE");
@@ -634,32 +715,43 @@ Xapian::Query::Internal::Internal(op_t op_, Xapian::valueno valno,
 				  const std::string &value)
 	: op(op_),
 	  parameter(Xapian::termcount(valno)),
-	  tname(value)
+	  tname(value),
+	  external_source(NULL),
+	  external_source_owned(false)
 {
     if (op != OP_VALUE_GE && op != OP_VALUE_LE)
 	throw Xapian::InvalidArgumentError("This constructor is only meaningful for OP_VALUE_GE or OP_VALUE_LE");
     if (op == OP_VALUE_GE && value.empty()) {
 	// Map '<value> >= ""' to MatchAll.
 	op = OP_LEAF;
-	parameter = 0;
+	parameter = 1; // wqf
 	term_pos = 0;
-	wqf = 1;
     }
     validate_query();
 }
 
+Xapian::Query::Internal::Internal(PostingSource * external_source_, bool owned)
+	: op(OP_EXTERNAL_SOURCE), external_source(external_source_),
+	  external_source_owned(owned)
+{
+    Assert(external_source);
+}
+
 Xapian::Query::Internal::~Internal()
 {
     subquery_list::iterator i;
     for (i = subqs.begin(); i != subqs.end(); i++) {
         delete *i;
     }
+    if (external_source_owned) {
+	delete external_source;
+    }
 }
 
 Xapian::Query::Internal *
 Xapian::Query::Internal::end_construction()
 {
-    DEBUGCALL(API, void, "Xapian::Query::Internal::end_construction", "");
+    LOGCALL_VOID(MATCH, "Xapian::Query::Internal::end_construction", NO_ARGS);
     validate_query();
     Xapian::Query::Internal * qint = simplify_query();
     if (qint) qint->validate_query();
@@ -669,16 +761,16 @@ Xapian::Query::Internal::end_construction()
 void
 Xapian::Query::Internal::validate_query() const
 {
-    DEBUGCALL(API, void, "Xapian::Query::Internal::validate_query", "");
+    LOGCALL_VOID(MATCH, "Xapian::Query::Internal::validate_query", NO_ARGS);
 
     // Check that the number of subqueries is in acceptable limits for this op
     if (subqs.size() < get_min_subqs(op) ||
 	subqs.size() > get_max_subqs(op)) {
 	throw Xapian::InvalidArgumentError("Xapian::Query: " + get_op_name(op) +
-		" requires a minimum of " + om_tostring(get_min_subqs(op)) +
-		" and a maximum of " + om_tostring(get_max_subqs(op)) +
+		" requires a minimum of " + str(get_min_subqs(op)) +
+		" and a maximum of " + str(get_max_subqs(op)) +
 		" sub queries, had " +
-		om_tostring(subqs.size()) + ".");
+		str(subqs.size()) + ".");
     }
 
     if (op == OP_SCALE_WEIGHT && get_dbl_parameter() < 0) {
@@ -717,6 +809,7 @@ Xapian::Query::Internal::simplify_matchnothing()
         case OP_ELITE_SET:
         case OP_OR:
         case OP_XOR:
+	case OP_SYNONYM:
             // Doing an "OR" type operation - if we've got any MatchNothing
             // subnodes, drop them; except that we mustn't become an empty
             // node due to this, so we never drop a MatchNothing subnode
@@ -762,7 +855,7 @@ Xapian::Query::Internal::simplify_matchnothing()
 Xapian::Query::Internal *
 Xapian::Query::Internal::simplify_query()
 {
-    DEBUGCALL(API, Xapian::Query::Internal *, "Xapian::Query::Internal::simplify_query", "");
+    LOGCALL(MATCH, Xapian::Query::Internal *, "Xapian::Query::Internal::simplify_query", NO_ARGS);
 
     // Simplify any MatchNothing nodes.
     if (simplify_matchnothing()) {
@@ -808,7 +901,7 @@ Xapian::Query::Internal::simplify_query()
 		}
 	    }
 	    break;
-	case OP_OR: case OP_AND: case OP_XOR:
+	case OP_OR: case OP_AND: case OP_XOR: case OP_SYNONYM:
 	    // Remove duplicates if we can.
 	    if (subqs.size() > 1) collapse_subqs();
 	    break;
@@ -852,7 +945,7 @@ struct SortPosName {
 void
 Xapian::Query::Internal::collapse_subqs()
 {
-    Assert(op == OP_OR || op == OP_AND || op == OP_XOR);
+    Assert(op == OP_OR || op == OP_AND || op == OP_XOR || op == OP_SYNONYM);
     typedef set<Xapian::Query::Internal *, SortPosName> subqtable;
     subqtable sqtab;
 
@@ -868,7 +961,8 @@ Xapian::Query::Internal::collapse_subqs()
 	    } else {
 		AssertEq((*s)->tname, (*sq)->tname);
 		AssertEq((*s)->term_pos, (*sq)->term_pos);
-		(*s)->wqf += (*sq)->wqf;
+		// parameter is wqf.
+		(*s)->parameter += (*sq)->parameter;
 		// Rather than incrementing sq, delete the current
 		// element, as it has been merged into the other
 		// equivalent term.
@@ -910,7 +1004,7 @@ Xapian::Query::Internal::flatten_subqs()
 	AutoPtr<Xapian::Query::Internal> newq(new Xapian::Query::Internal(*this));
 	delete *sq;
 	*sq = 0;
-	newq = newq->flatten_subqs();
+	newq.reset(newq->flatten_subqs());
 	*j = newq.release();
     }
 
@@ -946,7 +1040,7 @@ Xapian::Query::Internal::add_subquery(const Xapian::Query::Internal * subq)
     Assert(!is_leaf(op));
     if (subq == 0) {
 	subqs.push_back(0);
-    } else if (op == subq->op && (op == OP_AND || op == OP_OR || op == OP_XOR)) {
+    } else if (op == subq->op && is_distributable(op)) {
 	// Distribute the subquery.
 	for (subquery_list::const_iterator i = subq->subqs.begin();
 	     i != subq->subqs.end(); i++) {
@@ -963,7 +1057,7 @@ Xapian::Query::Internal::add_subquery_nocopy(Xapian::Query::Internal * subq)
     Assert(!is_leaf(op));
     if (subq == 0) {
 	subqs.push_back(0);
-    } else if (op == subq->op && (op == OP_AND || op == OP_OR || op == OP_XOR)) {
+    } else if (op == subq->op && is_distributable(op)) {
 	// Distribute the subquery.
 	for (subquery_list::const_iterator i = subq->subqs.begin();
 	     i != subq->subqs.end(); i++) {
@@ -981,7 +1075,7 @@ Xapian::Query::Internal::set_dbl_parameter(double dbl_parameter_)
     // We store the double parameter encoded as a string because
     // Xapian::Query::Internal is defined in an external API header and we want
     // to avoid any risk of ABI breakage (we suspect it would be OK, but it's
-    // not risking).  FIXME: rework for 1.1.0
+    // not risking).  FIXME: rework in 1.1.x series - see ticket #280
     str_parameter = serialise_double(dbl_parameter_);
 }
 
@@ -991,7 +1085,7 @@ Xapian::Query::Internal::get_dbl_parameter() const
     // We store the double parameter encoded as a string because
     // Xapian::Query::Internal is defined in an external API header and we want
     // to avoid any risk of ABI breakage (we suspect it would be OK, but it's
-    // not risking).  FIXME: rework for 1.1.0
+    // not risking).  FIXME: rework in 1.1.x series - see ticket #280
     const char * p = str_parameter.data();
     const char * end = p + str_parameter.size();
     return unserialise_double(&p, end);
