@@ -1,6 +1,6 @@
 /* dbfactory_remote.cc: Database factories for remote databases.
  *
- * Copyright (C) 2006,2007 Olly Betts
+ * Copyright (C) 2006,2007,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,8 +21,9 @@
 
 #include <xapian/dbfactory.h>
 
+#include "debuglog.h"
 #include "progclient.h"
-#include "tcpclient.h"
+#include "remotetcpclient.h"
 
 #include <string>
 
@@ -34,25 +35,22 @@ Database
 Remote::open(const string &host, unsigned int port, Xapian::timeout timeout,
 	     Xapian::timeout connect_timeout)
 {
-    DEBUGAPICALL_STATIC(Database, "Remote::open",
-	host << ", " << port << ", " << timeout << ", " << connect_timeout);
-    return Database(new TcpClient(host, port, timeout, connect_timeout, false));
+    LOGCALL_STATIC(API, Database, "Remote::open", host | port | timeout | connect_timeout);
+    return Database(new RemoteTcpClient(host, port, timeout, connect_timeout, false));
 }
 
 WritableDatabase
 Remote::open_writable(const string &host, unsigned int port,
 		      Xapian::timeout timeout, Xapian::timeout connect_timeout)
 {
-    DEBUGAPICALL_STATIC(WritableDatabase, "Remote::open_writable",
-	host << ", " << port << ", " << timeout << ", " << connect_timeout);
-    return WritableDatabase(new TcpClient(host, port, timeout, connect_timeout, true));
+    LOGCALL_STATIC(API, WritableDatabase, "Remote::open_writable", host | port | timeout | connect_timeout);
+    return WritableDatabase(new RemoteTcpClient(host, port, timeout, connect_timeout, true));
 }
 
 Database
 Remote::open(const string &program, const string &args, Xapian::timeout timeout)
 {
-    DEBUGAPICALL_STATIC(Database, "Remote::open",
-	program << ", " << args << ", " << timeout);
+    LOGCALL_STATIC(API, Database, "Remote::open", program | args | timeout);
     return Database(new ProgClient(program, args, timeout, false));
 }
 
@@ -60,8 +58,7 @@ WritableDatabase
 Remote::open_writable(const string &program, const string &args,
 		      Xapian::timeout timeout)
 {
-    DEBUGAPICALL_STATIC(WritableDatabase, "Remote::open_writable",
-	program << ", " << args << ", " << timeout);
+    LOGCALL_STATIC(API, WritableDatabase, "Remote::open_writable", program | args | timeout);
     return WritableDatabase(new ProgClient(program, args, timeout, true));
 }
 
