@@ -1,7 +1,7 @@
 /** @file flint_alldocspostlist.cc
  * @brief A PostList which iterates over all documents in a FlintDatabase.
  */
-/* Copyright (C) 2006,2007 Olly Betts
+/* Copyright (C) 2006,2007,2008,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,20 +19,16 @@
  */
 
 #include <config.h>
+#include "flint_alldocspostlist.h"
 
 #include <string>
 
+#include "debuglog.h"
 #include "flint_database.h"
-#include "flint_alldocspostlist.h"
-
-#include "utils.h"
+#include "str.h"
 
 using namespace std;
 
-FlintAllDocsPostList::~FlintAllDocsPostList()
-{
-}
-
 Xapian::doccount
 FlintAllDocsPostList::get_termfreq() const
 {
@@ -45,10 +41,10 @@ FlintAllDocsPostList::get_docid() const
     return current_did;
 }
 
-Xapian::doclength
+Xapian::termcount
 FlintAllDocsPostList::get_doclength() const
 {
-    DEBUGCALL(DB, Xapian::doclength, "FlintAllDocsPostList::get_doclength", "");
+    LOGCALL(DB, Xapian::termcount, "FlintAllDocsPostList::get_doclength", NO_ARGS);
     Assert(current_did);
 
     cursor->read_tag();
@@ -59,7 +55,7 @@ FlintAllDocsPostList::get_doclength() const
     const char * end = pos + cursor->current_tag.size();
 
     flint_doclen_t doclen;
-    if (!unpack_uint(&pos, end, &doclen)) {
+    if (!F_unpack_uint(&pos, end, &doclen)) {
 	const char *msg;
 	if (pos == 0) {
 	    msg = "Too little data for doclen in termlist";
@@ -75,7 +71,7 @@ FlintAllDocsPostList::get_doclength() const
 Xapian::termcount
 FlintAllDocsPostList::get_wdf() const
 {
-    DEBUGCALL(DB, Xapian::termcount, "FlintAllDocsPostList::get_wdf", "");
+    LOGCALL(DB, Xapian::termcount, "FlintAllDocsPostList::get_wdf", NO_ARGS);
     Assert(current_did);
     RETURN(1);
 }
@@ -83,12 +79,11 @@ FlintAllDocsPostList::get_wdf() const
 PostList *
 FlintAllDocsPostList::read_did_from_current_key()
 {
-    DEBUGCALL(DB, PostList *, "FlintAllDocsPostList::read_did_from_current_key",
-	      "");
+    LOGCALL(DB, PostList *, "FlintAllDocsPostList::read_did_from_current_key", NO_ARGS);
     const string & key = cursor->current_key;
     const char * pos = key.data();
     const char * end = pos + key.size();
-    if (!unpack_uint_preserving_sort(&pos, end, &current_did)) {
+    if (!F_unpack_uint_preserving_sort(&pos, end, &current_did)) {
 	const char *msg;
 	if (pos == 0) {
 	    msg = "Too little data in termlist key";
@@ -105,7 +100,7 @@ FlintAllDocsPostList::read_did_from_current_key()
 PostList *
 FlintAllDocsPostList::next(Xapian::weight /*w_min*/)
 {
-    DEBUGCALL(DB, PostList *, "FlintAllDocsPostList::next", "/*w_min*/");
+    LOGCALL(DB, PostList *, "FlintAllDocsPostList::next", "/*w_min*/");
     Assert(!at_end());
     if (!cursor->next()) RETURN(NULL);
     RETURN(read_did_from_current_key());
@@ -114,12 +109,11 @@ FlintAllDocsPostList::next(Xapian::weight /*w_min*/)
 PostList *
 FlintAllDocsPostList::skip_to(Xapian::docid did, Xapian::weight /*w_min*/)
 {
-    DEBUGCALL(DB, PostList *, "FlintAllDocsPostList::skip_to",
-	      did << ", /*w_min*/");
+    LOGCALL(DB, PostList *, "FlintAllDocsPostList::skip_to", did | "/*w_min*/");
 
     if (did <= current_did || at_end()) RETURN(NULL);
 
-    if (cursor->find_entry_ge(pack_uint_preserving_sort(did))) {
+    if (cursor->find_entry_ge(F_pack_uint_preserving_sort(did))) {
 	// The exact docid that was asked for exists.
 	current_did = did;
 	RETURN(NULL);
@@ -131,7 +125,7 @@ FlintAllDocsPostList::skip_to(Xapian::docid did, Xapian::weight /*w_min*/)
 
 bool
 FlintAllDocsPostList::at_end() const {
-    DEBUGCALL(DB, bool, "FlintAllDocsPostList::at_end", "");
+    LOGCALL(DB, bool, "FlintAllDocsPostList::at_end", NO_ARGS);
     RETURN(cursor->after_end());
 }
 
@@ -139,9 +133,9 @@ string
 FlintAllDocsPostList::get_description() const
 {
     string desc = "FlintAllDocsPostList(did=";
-    desc += om_tostring(current_did);
+    desc += str(current_did);
     desc += ",doccount=";
-    desc += om_tostring(doccount);
+    desc += str(doccount);
     desc += ')';
     return desc;
 }
