@@ -3,9 +3,9 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Hein Ragas
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2010 Olly Betts
- * Copyright 2006 Lemur Consulting Ltd
- * Copyright 2009 Richard Boulton
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2006,2008 Lemur Consulting Ltd
+ * Copyright 2009,2010 Richard Boulton
  * Copyright 2009 Kan-Ru Chen
  *
  * This program is free software; you can redistribute it and/or
@@ -31,29 +31,43 @@
 #include <xapian/error.h>
 #include <xapian/valueiterator.h>
 
-#include "autoptr.h"
 #include "contiguousalldocspostlist.h"
 #include "flint_alldocspostlist.h"
 #include "flint_alltermslist.h"
 #include "flint_document.h"
-#include "flint_lock.h"
+#include "../flint_lock.h"
 #include "flint_metadata.h"
 #include "flint_modifiedpostlist.h"
 #include "flint_positionlist.h"
 #include "flint_postlist.h"
 #include "flint_record.h"
+#include "flint_replicate_internal.h"
 #include "flint_spellingwordslist.h"
 #include "flint_termlist.h"
 #include "flint_utils.h"
 #include "flint_values.h"
-#include "omdebug.h"
-#include "safeerrno.h"
-#include "safesysstat.h"
+#include "debuglog.h"
+#include "io_utils.h"
+#include "omtime.h"
+#include "remoteconnection.h"
+#include "replicate_utils.h"
+#include "replication.h"
+#include "replicationprotocol.h"
+#include "serialise.h"
+#include "str.h"
 #include "stringutils.h"
 #include "utils.h"
 
+#ifdef __WIN32__
+# include "msvc_posix_wrapper.h"
+#endif
+
+#include "safeerrno.h"
+#include "safesysstat.h"
 #include <sys/types.h>
 
+#include "autoptr.h"
+#include <cstdio> // For rename().
 #include <list>
 #include <string>
 
@@ -70,7 +84,24 @@ using namespace Xapian;
 
 // Magic key in the postlist table (which corresponds to an invalid docid) is
 // used to store the next free docid and total length of all documents.
-static const string METAINFO_KEY("", 1);
+static const string METAINFO_KEY(1, '\0');
+
+/** Delete file, throwing an error if we can't delete it (but not if it
+ *  doesn't exist).
+ */
+static void
+sys_unlink_if_exists(const string & filename)
+{
+#ifdef __WIN32__
+    if (msvc_posix_unlink(filename.c_str()) == -1) {
+#else
+    if (unlink(filename) == -1) {
+#endif
+	if (errno == ENOENT) return;
+	throw Xapian::DatabaseError("Can't delete file: `" + filename + "'",
+				    errno);
+    }
+}
 
 /* This finds the tables, opens them at consistent revisions, manages
  * determining the current and next revision numbers, and stores handles
@@ -88,18 +119,22 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
 	  synonym_table(db_dir, readonly),
 	  spelling_table(db_dir, readonly),
 	  record_table(db_dir, readonly),
-	  lock(db_dir + "/flintlock"),
+	  lock(db_dir),
 	  total_length(0),
-	  lastdocid(0)
+	  lastdocid(0),
+	  max_changesets(0)
 {
-    DEBUGCALL(DB, void, "FlintDatabase", flint_dir << ", " << action <<
-	      ", " << block_size);
+    LOGCALL_CTOR(DB, "FlintDatabase", flint_dir | action | block_size);
 
     if (action == XAPIAN_DB_READONLY) {
 	open_tables_consistent();
 	return;
     }
 
+    const char *p = getenv("XAPIAN_MAX_CHANGESETS");
+    if (p)
+	max_changesets = atoi(p);
+
     if (action != Xapian::DB_OPEN && !database_exists()) {
 	// FIXME: if we allow Xapian::DB_OVERWRITE, check it here
 
@@ -116,7 +151,7 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
 	    throw Xapian::DatabaseCreateError("Cannot create directory `" +
 					      db_dir + "'", errno);
 	}
-	get_database_write_lock();
+	get_database_write_lock(true);
 
 	create_and_open_tables(block_size);
 	return;
@@ -128,7 +163,7 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
 					  "not to overwrite it");
     }
 
-    get_database_write_lock();
+    get_database_write_lock(false);
     // if we're overwriting, pretend the db doesn't exist
     // FIXME: if we allow Xapian::DB_OVERWRITE, check it here
     if (action == Xapian::DB_CREATE_OR_OVERWRITE) {
@@ -152,13 +187,13 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
 
 FlintDatabase::~FlintDatabase()
 {
-    DEBUGCALL(DB, void, "~FlintDatabase", "");
+    LOGCALL_DTOR(DB, "~FlintDatabase");
 }
 
 void
 FlintDatabase::read_metainfo()
 {
-    DEBUGCALL(DB, void, "FlintDatabase::read_metainfo", "");
+    LOGCALL_VOID(DB, "FlintDatabase::read_metainfo", NO_ARGS);
 
     string tag;
     if (!postlist_table.get_exact_entry(METAINFO_KEY, tag)) {
@@ -169,15 +204,15 @@ FlintDatabase::read_metainfo()
 
     const char * data = tag.data();
     const char * end = data + tag.size();
-    if (!unpack_uint(&data, end, &lastdocid) ||
-	!unpack_uint_last(&data, end, &total_length)) {
+    if (!F_unpack_uint(&data, end, &lastdocid) ||
+	!F_unpack_uint_last(&data, end, &total_length)) {
 	throw Xapian::DatabaseCorruptError("Meta information is corrupt.");
     }
 }
 
 bool
 FlintDatabase::database_exists() {
-    DEBUGCALL(DB, bool, "FlintDatabase::database_exists", "");
+    LOGCALL(DB, bool, "FlintDatabase::database_exists", NO_ARGS);
     RETURN(record_table.exists() &&
 	   postlist_table.exists() &&
 	   termlist_table.exists());
@@ -186,7 +221,7 @@ FlintDatabase::database_exists() {
 void
 FlintDatabase::create_and_open_tables(unsigned int block_size)
 {
-    DEBUGCALL(DB, void, "FlintDatabase::create_and_open_tables", "");
+    LOGCALL_VOID(DB, "FlintDatabase::create_and_open_tables", NO_ARGS);
     // The caller is expected to create the database directory if it doesn't
     // already exist.
 
@@ -225,7 +260,7 @@ FlintDatabase::create_and_open_tables(unsigned int block_size)
 void
 FlintDatabase::open_tables_consistent()
 {
-    DEBUGCALL(DB, void, "FlintDatabase::open_tables_consistent", "");
+    LOGCALL_VOID(DB, "FlintDatabase::open_tables_consistent", NO_ARGS);
     // Open record_table first, since it's the last to be written to,
     // and hence if a revision is available in it, it should be available
     // in all the other tables (unless they've moved on already).
@@ -302,7 +337,7 @@ FlintDatabase::open_tables_consistent()
 void
 FlintDatabase::open_tables(flint_revision_number_t revision)
 {
-    DEBUGCALL(DB, void, "FlintDatabase::open_tables", revision);
+    LOGCALL_VOID(DB, "FlintDatabase::open_tables", revision);
     version_file.read_and_check(readonly);
     record_table.open(revision);
 
@@ -325,7 +360,7 @@ FlintDatabase::open_tables(flint_revision_number_t revision)
 flint_revision_number_t
 FlintDatabase::get_revision_number() const
 {
-    DEBUGCALL(DB, flint_revision_number_t, "FlintDatabase::get_revision_number", "");
+    LOGCALL(DB, flint_revision_number_t, "FlintDatabase::get_revision_number", NO_ARGS);
     // We could use any table here, theoretically.
     RETURN(postlist_table.get_open_revision_number());
 }
@@ -333,7 +368,7 @@ FlintDatabase::get_revision_number() const
 flint_revision_number_t
 FlintDatabase::get_next_revision_number() const
 {
-    DEBUGCALL(DB, flint_revision_number_t, "FlintDatabase::get_next_revision_number", "");
+    LOGCALL(DB, flint_revision_number_t, "FlintDatabase::get_next_revision_number", NO_ARGS);
     /* We _must_ use postlist_table here, since it is always the first
      * to be written, and hence will have the greatest available revision
      * number.
@@ -344,51 +379,341 @@ FlintDatabase::get_next_revision_number() const
     RETURN(new_revision);
 }
 
+void
+FlintDatabase::get_changeset_revisions(const string & path,
+				       flint_revision_number_t * startrev,
+				       flint_revision_number_t * endrev) const
+{
+    int changes_fd = -1;
+#ifdef __WIN32__
+    changes_fd = msvc_posix_open(path.c_str(), O_RDONLY);
+#else
+    changes_fd = open(path.c_str(), O_RDONLY);
+#endif
+    fdcloser closer(changes_fd);
+
+    if (changes_fd < 0) {
+	string message = string("Couldn't open changeset ")
+		+ path + " to read";
+	throw Xapian::DatabaseError(message, errno);
+    }
+
+    char buf[REASONABLE_CHANGESET_SIZE];
+    const char *start = buf;
+    const char *end = buf + io_read(changes_fd, buf,
+				    REASONABLE_CHANGESET_SIZE, 0);
+    if (strncmp(start, CHANGES_MAGIC_STRING,
+		CONST_STRLEN(CHANGES_MAGIC_STRING)) != 0) {
+	string message = string("Changeset at ")
+		+ path + " does not contain valid magic string";
+	throw Xapian::DatabaseError(message);
+    }
+    start += CONST_STRLEN(CHANGES_MAGIC_STRING);
+    if (start >= end)
+	throw Xapian::DatabaseError("Changeset too short at " + path);
+
+    unsigned int changes_version;
+    if (!F_unpack_uint(&start, end, &changes_version))
+	throw Xapian::DatabaseError("Couldn't read a valid version number for "
+				    "changeset at " + path);
+    if (changes_version != CHANGES_VERSION)
+	throw Xapian::DatabaseError("Don't support version of changeset at "
+				    + path);
+
+    if (!F_unpack_uint(&start, end, startrev))
+	throw Xapian::DatabaseError("Couldn't read a valid start revision from "
+				    "changeset at " + path);
+
+    if (!F_unpack_uint(&start, end, endrev))
+	throw Xapian::DatabaseError("Couldn't read a valid end revision for "
+				    "changeset at " + path);
+}
+
 void
 FlintDatabase::set_revision_number(flint_revision_number_t new_revision)
 {
-    DEBUGCALL(DB, void, "FlintDatabase::set_revision_number", new_revision);
-    postlist_table.commit(new_revision);
-    position_table.commit(new_revision);
-    termlist_table.commit(new_revision);
-    value_table.commit(new_revision);
-    synonym_table.commit(new_revision);
-    spelling_table.commit(new_revision);
-    record_table.commit(new_revision);
+    LOGCALL_VOID(DB, "FlintDatabase::set_revision_number", new_revision);
+
+    postlist_table.flush_db();
+    position_table.flush_db();
+    termlist_table.flush_db();
+    value_table.flush_db();
+    synonym_table.flush_db();
+    spelling_table.flush_db();
+    record_table.flush_db();
+
+    int changes_fd = -1;
+    string changes_name;
+
+    if (max_changesets > 0) {
+	flint_revision_number_t old_revision = get_revision_number();
+	if (old_revision) {
+	    // Don't generate a changeset for the first revision.
+	    changes_fd = create_changeset_file(db_dir,
+					       "/changes" + str(old_revision),
+					       changes_name);
+	}
+    }
+
+    try {
+	fdcloser closefd(changes_fd);
+	if (changes_fd >= 0) {
+	    string buf;
+	    flint_revision_number_t old_revision = get_revision_number();
+	    buf += CHANGES_MAGIC_STRING;
+	    buf += F_pack_uint(CHANGES_VERSION);
+	    buf += F_pack_uint(old_revision);
+	    buf += F_pack_uint(new_revision);
+
+	    // FIXME - if DANGEROUS mode is in use, this should contain F_pack_uint(1u)
+	    buf += F_pack_uint(0u); // Changes can be applied to a live database.
+
+	    io_write(changes_fd, buf.data(), buf.size());
+
+	    // Write the changes to the blocks in the tables.  Do the postlist
+	    // table last, so that ends up cached the most, if the cache
+	    // available is limited.  Do the position and value tables just
+	    // before that, because they're also critical to search speed.
+	    termlist_table.write_changed_blocks(changes_fd);
+	    synonym_table.write_changed_blocks(changes_fd);
+	    spelling_table.write_changed_blocks(changes_fd);
+	    record_table.write_changed_blocks(changes_fd);
+	    position_table.write_changed_blocks(changes_fd);
+	    value_table.write_changed_blocks(changes_fd);
+	    postlist_table.write_changed_blocks(changes_fd);
+	}
+
+	postlist_table.commit(new_revision, changes_fd);
+	position_table.commit(new_revision, changes_fd);
+	termlist_table.commit(new_revision, changes_fd);
+	value_table.commit(new_revision, changes_fd);
+	synonym_table.commit(new_revision, changes_fd);
+	spelling_table.commit(new_revision, changes_fd);
+
+	string changes_tail; // Data to be appended to the changes file
+	if (changes_fd >= 0) {
+	    changes_tail += '\0';
+	    changes_tail += F_pack_uint(new_revision);
+	}
+	record_table.commit(new_revision, changes_fd, &changes_tail);
+
+    } catch (...) {
+	// Remove the changeset, if there was one.
+	if (changes_fd >= 0) {
+	    sys_unlink_if_exists(changes_name);
+	}
+
+	throw;
+    }
 }
 
 void
 FlintDatabase::reopen()
 {
-    DEBUGCALL(DB, void, "FlintDatabase::reopen", "");
+    LOGCALL_VOID(DB, "FlintDatabase::reopen", NO_ARGS);
     if (readonly) {
 	open_tables_consistent();
     }
 }
 
 void
-FlintDatabase::get_database_write_lock()
+FlintDatabase::close()
 {
-    DEBUGCALL(DB, void, "FlintDatabase::get_database_write_lock", "");
-    FlintLock::reason why = lock.lock(true);
+    LOGCALL_VOID(DB, "FlintDatabase::close", NO_ARGS);
+    postlist_table.close(true);
+    position_table.close(true);
+    termlist_table.close(true);
+    value_table.close(true);
+    synonym_table.close(true);
+    spelling_table.close(true);
+    record_table.close(true);
+    lock.release();
+}
+
+void
+FlintDatabase::get_database_write_lock(bool creating)
+{
+    LOGCALL_VOID(DB, "FlintDatabase::get_database_write_lock", creating);
+    string explanation;
+    FlintLock::reason why = lock.lock(true, explanation);
     if (why != FlintLock::SUCCESS) {
-	if (why == FlintLock::UNKNOWN && !database_exists()) {
+	if (why == FlintLock::UNKNOWN && !creating && !database_exists()) {
 	    string msg("No flint database found at path `");
 	    msg += db_dir;
 	    msg += '\'';
 	    throw Xapian::DatabaseOpeningError(msg);
 	}
-	string msg("Unable to acquire database write lock on ");
-	msg += db_dir;
-	if (why == FlintLock::INUSE) {
-	    msg += ": already locked";
-	} else if (why == FlintLock::UNSUPPORTED) {
-	    msg += ": locking probably not supported by this FS";
-	}
-	throw Xapian::DatabaseLockError(msg);
+	lock.throw_databaselockerror(why, db_dir, explanation);
     }
 }
 
+void
+FlintDatabase::send_whole_database(RemoteConnection & conn,
+				   const OmTime & end_time)
+{
+    LOGCALL_VOID(DB, "FlintDatabase::send_whole_database", conn | end_time);
+
+    // Send the current revision number in the header.
+    string buf;
+    string uuid = get_uuid();
+    buf += encode_length(uuid.size());
+    buf += uuid;
+    buf += F_pack_uint(get_revision_number());
+    conn.send_message(REPL_REPLY_DB_HEADER, buf, end_time);
+
+    // Send all the tables.  The tables which we want to be cached best after
+    // the copy finished are sent last.
+    static const char filenames[] =
+	"\x0b""termlist.DB""\x0e""termlist.baseA\x0e""termlist.baseB"
+	"\x0a""synonym.DB""\x0d""synonym.baseA\x0d""synonym.baseB"
+	"\x0b""spelling.DB""\x0e""spelling.baseA\x0e""spelling.baseB"
+	"\x09""record.DB""\x0c""record.baseA\x0c""record.baseB"
+	"\x0b""position.DB""\x0e""position.baseA\x0e""position.baseB"
+	"\x08""value.DB""\x0b""value.baseA\x0b""value.baseB"
+	"\x0b""postlist.DB""\x0e""postlist.baseA\x0e""postlist.baseB"
+	"\x08""iamflint"
+	"\x04""uuid";
+    string filepath = db_dir;
+    filepath += '/';
+    for (const char * p = filenames; *p; p += *p + 1) {
+	string leaf(p + 1, size_t(static_cast<unsigned char>(*p)));
+        filepath.replace(db_dir.size() + 1, string::npos, leaf);
+	if (file_exists(filepath)) {
+	    // FIXME - there is a race condition here - the file might get
+	    // deleted between the file_exists() test and the access to send it.
+	    conn.send_message(REPL_REPLY_DB_FILENAME, leaf, end_time);
+	    conn.send_file(REPL_REPLY_DB_FILEDATA, filepath, end_time);
+	}
+    }
+}
+
+void
+FlintDatabase::write_changesets_to_fd(int fd,
+				      const string & revision,
+				      bool need_whole_db,
+				      ReplicationInfo * info)
+{
+    LOGCALL_VOID(DB, "FlintDatabase::write_changesets_to_fd", fd | revision | need_whole_db | info);
+
+    int whole_db_copies_left = MAX_DB_COPIES_PER_CONVERSATION;
+    flint_revision_number_t start_rev_num = 0;
+    string start_uuid = get_uuid();
+
+    flint_revision_number_t needed_rev_num = 0;
+
+    const char * rev_ptr = revision.data();
+    const char * rev_end = rev_ptr + revision.size();
+    if (!F_unpack_uint(&rev_ptr, rev_end, &start_rev_num)) {
+	need_whole_db = true;
+    }
+
+    RemoteConnection conn(-1, fd, "");
+    OmTime end_time;
+
+    // While the starting revision number is less than the latest revision
+    // number, look for a changeset, and write it.
+    //
+    // FIXME - perhaps we should make hardlinks for all the changesets we're
+    // likely to need, first, and then start sending them, so that there's no
+    // risk of them disappearing while we're sending earlier ones.
+    while (true) {
+	if (need_whole_db) {
+	    // Decrease the counter of copies left to be sent, and fail
+	    // if we've already copied the database enough.  This ensures that
+	    // synchronisation attempts always terminate eventually.
+	    if (whole_db_copies_left == 0) {
+		conn.send_message(REPL_REPLY_FAIL,
+				  "Database changing too fast",
+				  end_time);
+		return;
+	    }
+	    whole_db_copies_left--;
+
+	    // Send the whole database across.
+	    start_rev_num = get_revision_number();
+	    start_uuid = get_uuid();
+
+	    send_whole_database(conn, end_time);
+	    if (info != NULL)
+		++(info->fullcopy_count);
+
+	    need_whole_db = false;
+
+	    reopen();
+	    if (start_uuid == get_uuid()) {
+		// Send the latest revision number after sending the tables.
+		// The update must proceed to that revision number before the
+		// copy is safe to make live.
+
+		string buf;
+		needed_rev_num = get_revision_number();
+		buf += F_pack_uint(needed_rev_num);
+		conn.send_message(REPL_REPLY_DB_FOOTER, buf, end_time);
+		if (info != NULL && start_rev_num == needed_rev_num)
+		    info->changed = true;
+	    } else {
+		// Database has been replaced since we did the copy.  Send a
+		// higher revision number than the revision we've just copied,
+		// so that the client doesn't make the copy we've just done
+		// live, and then mark that we need to do a copy again.
+		// The client will never actually get the required revision,
+		// because the next message is going to be the start of a new
+		// database transfer.
+
+		string buf;
+		buf += F_pack_uint(start_rev_num + 1);
+		conn.send_message(REPL_REPLY_DB_FOOTER, buf, end_time);
+		need_whole_db = true;
+	    }
+	} else {
+	    // Check if we've sent all the updates.
+	    if (start_rev_num >= get_revision_number()) {
+		reopen();
+		if (start_uuid != get_uuid()) {
+		    need_whole_db = true;
+		    continue;
+		}
+		if (start_rev_num >= get_revision_number()) {
+		    break;
+		}
+	    }
+
+	    // Look for the changeset for revision start_rev_num.
+	    string changes_name = db_dir + "/changes" + str(start_rev_num);
+	    if (file_exists(changes_name)) {
+		// Send it, and also update start_rev_num to the new value
+		// specified in the changeset.
+		flint_revision_number_t changeset_start_rev_num;
+		flint_revision_number_t changeset_end_rev_num;
+		get_changeset_revisions(changes_name,
+					&changeset_start_rev_num,
+					&changeset_end_rev_num);
+		if (changeset_start_rev_num != start_rev_num) {
+		    throw Xapian::DatabaseError("Changeset start revision does not match changeset filename");
+		}
+		if (changeset_start_rev_num >= changeset_end_rev_num) {
+		    throw Xapian::DatabaseError("Changeset start revision is not less than end revision");
+		}
+		// FIXME - there is a race condition here - the file might get
+		// deleted between the file_exists() test and the access to
+		// send it.
+		conn.send_file(REPL_REPLY_CHANGESET, changes_name, end_time);
+		start_rev_num = changeset_end_rev_num;
+		if (info != NULL) {
+		    ++(info->changeset_count);
+		    if (start_rev_num >= needed_rev_num)
+			info->changed = true;
+		}
+	    } else {
+		// The changeset doesn't exist: leave the revision number as it
+		// is, and mark for doing a full database copy.
+		need_whole_db = true;
+	    }
+	}
+    }
+    conn.send_message(REPL_REPLY_END_OF_CHANGES, string(), end_time);
+}
+
 void
 FlintDatabase::modifications_failed(flint_revision_number_t old_revision,
 				    flint_revision_number_t new_revision,
@@ -408,16 +733,9 @@ FlintDatabase::modifications_failed(flint_revision_number_t old_revision,
 	++new_revision;
 	set_revision_number(new_revision);
     } catch (const Xapian::Error &e) {
-	// Permanently close the table, since we can't get it into a
-	// consistent state, to avoid risk of database corruption.
-	postlist_table.close(true);
-	position_table.close(true);
-	termlist_table.close(true);
-	value_table.close(true);
-	synonym_table.close(true);
-	spelling_table.close(true);
-	record_table.close(true);
-	lock.release();
+	// We can't get the database into a consistent state, so close
+	// it to avoid the risk of database corruption.
+	FlintDatabase::close();
 	throw Xapian::DatabaseError("Modifications failed (" + msg +
 				    "), and cannot set consistent table "
 				    "revision numbers: " + e.get_msg());
@@ -427,7 +745,7 @@ FlintDatabase::modifications_failed(flint_revision_number_t old_revision,
 void
 FlintDatabase::apply()
 {
-    DEBUGCALL(DB, void, "FlintDatabase::apply", "");
+    LOGCALL_VOID(DB, "FlintDatabase::apply", NO_ARGS);
     if (!postlist_table.is_modified() &&
 	!position_table.is_modified() &&
 	!termlist_table.is_modified() &&
@@ -455,7 +773,7 @@ FlintDatabase::apply()
 void
 FlintDatabase::cancel()
 {
-    DEBUGCALL(DB, void, "FlintDatabase::cancel", "");
+    LOGCALL_VOID(DB, "FlintDatabase::cancel", NO_ARGS);
     postlist_table.cancel();
     position_table.cancel();
     termlist_table.cancel();
@@ -468,21 +786,28 @@ FlintDatabase::cancel()
 Xapian::doccount
 FlintDatabase::get_doccount() const
 {
-    DEBUGCALL(DB, Xapian::doccount, "FlintDatabase::get_doccount", "");
+    LOGCALL(DB, Xapian::doccount, "FlintDatabase::get_doccount", NO_ARGS);
     RETURN(record_table.get_doccount());
 }
 
 Xapian::docid
 FlintDatabase::get_lastdocid() const
 {
-    DEBUGCALL(DB, Xapian::docid, "FlintDatabase::get_lastdocid", "");
+    LOGCALL(DB, Xapian::docid, "FlintDatabase::get_lastdocid", NO_ARGS);
     RETURN(lastdocid);
 }
 
+totlen_t
+FlintDatabase::get_total_length() const
+{
+    LOGCALL(DB, totlen_t, "FlintDatabase::get_total_length", NO_ARGS);
+    RETURN(total_length);
+}
+
 Xapian::doclength
 FlintDatabase::get_avlength() const
 {
-    DEBUGCALL(DB, Xapian::doclength, "FlintDatabase::get_avlength", "");
+    LOGCALL(DB, Xapian::doclength, "FlintDatabase::get_avlength", NO_ARGS);
     Xapian::doccount doccount = record_table.get_doccount();
     if (doccount == 0) {
 	// Avoid dividing by zero when there are no documents.
@@ -491,10 +816,10 @@ FlintDatabase::get_avlength() const
     RETURN(double(total_length) / doccount);
 }
 
-Xapian::doclength
+Xapian::termcount
 FlintDatabase::get_doclength(Xapian::docid did) const
 {
-    DEBUGCALL(DB, Xapian::doclength, "FlintDatabase::get_doclength", did);
+    LOGCALL(DB, Xapian::termcount, "FlintDatabase::get_doclength", did);
     Assert(did != 0);
     RETURN(termlist_table.get_doclength(did));
 }
@@ -502,7 +827,7 @@ FlintDatabase::get_doclength(Xapian::docid did) const
 Xapian::doccount
 FlintDatabase::get_termfreq(const string & term) const
 {
-    DEBUGCALL(DB, Xapian::doccount, "FlintDatabase::get_termfreq", term);
+    LOGCALL(DB, Xapian::doccount, "FlintDatabase::get_termfreq", term);
     Assert(!term.empty());
     RETURN(postlist_table.get_termfreq(term));
 }
@@ -510,7 +835,7 @@ FlintDatabase::get_termfreq(const string & term) const
 Xapian::termcount
 FlintDatabase::get_collection_freq(const string & term) const
 {
-    DEBUGCALL(DB, Xapian::termcount, "FlintDatabase::get_collection_freq", term);
+    LOGCALL(DB, Xapian::termcount, "FlintDatabase::get_collection_freq", term);
     Assert(!term.empty());
     RETURN(postlist_table.get_collection_freq(term));
 }
@@ -518,7 +843,7 @@ FlintDatabase::get_collection_freq(const string & term) const
 bool
 FlintDatabase::term_exists(const string & term) const
 {
-    DEBUGCALL(DB, bool, "FlintDatabase::term_exists", term);
+    LOGCALL(DB, bool, "FlintDatabase::term_exists", term);
     Assert(!term.empty());
     return postlist_table.term_exists(term);
 }
@@ -532,7 +857,7 @@ FlintDatabase::has_positions() const
 LeafPostList *
 FlintDatabase::open_post_list(const string& term) const
 {
-    DEBUGCALL(DB, LeafPostList *, "FlintDatabase::open_post_list", term);
+    LOGCALL(DB, LeafPostList *, "FlintDatabase::open_post_list", term);
     Xapian::Internal::RefCntPtr<const FlintDatabase> ptrtothis(this);
 
     if (term.empty()) {
@@ -549,7 +874,7 @@ FlintDatabase::open_post_list(const string& term) const
 TermList *
 FlintDatabase::open_term_list(Xapian::docid did) const
 {
-    DEBUGCALL(DB, TermList *, "FlintDatabase::open_term_list", did);
+    LOGCALL(DB, TermList *, "FlintDatabase::open_term_list", did);
     Assert(did != 0);
 
     Xapian::Internal::RefCntPtr<const FlintDatabase> ptrtothis(this);
@@ -559,8 +884,7 @@ FlintDatabase::open_term_list(Xapian::docid did) const
 Xapian::Document::Internal *
 FlintDatabase::open_document(Xapian::docid did, bool lazy) const
 {
-    DEBUGCALL(DB, Xapian::Document::Internal *, "FlintDatabase::open_document",
-	      did << ", " << lazy);
+    LOGCALL(DB, Xapian::Document::Internal *, "FlintDatabase::open_document", did | lazy);
     Assert(did != 0);
 
     Xapian::Internal::RefCntPtr<const FlintDatabase> ptrtothis(this);
@@ -575,16 +899,11 @@ FlintDatabase::open_position_list(Xapian::docid did, const string & term) const
 {
     Assert(did != 0);
 
-    AutoPtr<FlintPositionList> poslist(new FlintPositionList());
+    AutoPtr<FlintPositionList> poslist(new FlintPositionList);
     if (!poslist->read_data(&position_table, did, term)) {
-	// Check that term / document combination exists.
-	// If the doc doesn't exist, this will throw Xapian::DocNotFoundError:
-	AutoPtr<TermList> tl(open_term_list(did));
-	tl->skip_to(term);
-	if (tl->at_end() || tl->get_termname() != term)
-	    throw Xapian::RangeError("Can't open position list: requested term is not present in document.");
-	// FIXME: For 1.2.0, change this to just return an empty termlist.
-	// If the user really needs to know, they can check themselves.
+	// As of 1.1.0, we don't check if the did and term exist - we just
+	// return an empty positionlist.  If the user really needs to know,
+	// they can check for themselves.
     }
 
     return poslist.release();
@@ -593,7 +912,7 @@ FlintDatabase::open_position_list(Xapian::docid did, const string & term) const
 TermList *
 FlintDatabase::open_allterms(const string & prefix) const
 {
-    DEBUGCALL(DB, TermList *, "FlintDatabase::open_allterms", "");
+    LOGCALL(DB, TermList *, "FlintDatabase::open_allterms", NO_ARGS);
     RETURN(new FlintAllTermsList(Xapian::Internal::RefCntPtr<const FlintDatabase>(this),
 				 prefix));
 }
@@ -634,10 +953,21 @@ FlintDatabase::open_synonym_keylist(const string & prefix) const
 				    cursor, prefix);
 }
 
+string
+FlintDatabase::get_metadata(const string & key) const
+{
+    LOGCALL(DB, string, "FlintDatabase::get_metadata", key);
+    string btree_key("\x00\xc0", 2);
+    btree_key += key;
+    string tag;
+    (void)postlist_table.get_exact_entry(btree_key, tag);
+    RETURN(tag);
+}
+
 TermList *
 FlintDatabase::open_metadata_keylist(const std::string &prefix) const
 {
-    DEBUGCALL(DB, string, "FlintDatabase::open_metadata_keylist", "");
+    LOGCALL(DB, string, "FlintDatabase::open_metadata_keylist", NO_ARGS);
     FlintCursor * cursor = postlist_table.cursor_get();
     if (!cursor) return NULL;
     return new FlintMetadataTermList(Xapian::Internal::RefCntPtr<const FlintDatabase>(this),
@@ -645,14 +975,19 @@ FlintDatabase::open_metadata_keylist(const std::string &prefix) const
 }
 
 string
-FlintDatabase::get_metadata(const string & key) const
+FlintDatabase::get_revision_info() const
 {
-    DEBUGCALL(DB, string, "FlintDatabase::get_metadata", key);
-    string btree_key("\x00\xc0", 2);
-    btree_key += key;
-    string tag;
-    (void)postlist_table.get_exact_entry(btree_key, tag);
-    RETURN(tag);
+    LOGCALL(DB, string, "FlintDatabase::get_revision_info", NO_ARGS);
+    string buf;
+    buf += F_pack_uint(get_revision_number());
+    RETURN(buf);
+}
+
+string
+FlintDatabase::get_uuid() const
+{
+    LOGCALL(DB, string, "FlintDatabase::get_uuid", NO_ARGS);
+    RETURN(version_file.get_uuid_string());
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -668,8 +1003,7 @@ FlintWritableDatabase::FlintWritableDatabase(const string &dir, int action,
 	  modify_shortcut_document(NULL),
 	  modify_shortcut_docid(0)
 {
-    DEBUGCALL(DB, void, "FlintWritableDatabase", dir << ", " << action << ", "
-	      << block_size);
+    LOGCALL_CTOR(DB, "FlintWritableDatabase", dir | action | block_size);
 
     const char *p = getenv("XAPIAN_FLUSH_THRESHOLD");
     if (p)
@@ -680,15 +1014,15 @@ FlintWritableDatabase::FlintWritableDatabase(const string &dir, int action,
 
 FlintWritableDatabase::~FlintWritableDatabase()
 {
-    DEBUGCALL(DB, void, "~FlintWritableDatabase", "");
+    LOGCALL_DTOR(DB, "~FlintWritableDatabase");
     dtor_called();
 }
 
 void
-FlintWritableDatabase::flush()
+FlintWritableDatabase::commit()
 {
     if (transaction_active())
-	throw Xapian::InvalidOperationError("Can't flush during a transaction");
+	throw Xapian::InvalidOperationError("Can't commit during a transaction");
     if (change_count) flush_postlist_changes();
     apply();
 }
@@ -699,8 +1033,8 @@ FlintWritableDatabase::flush_postlist_changes() const
     postlist_table.merge_changes(mod_plists, doclens, freq_deltas);
 
     // Update the total document length and last used docid.
-    string tag = pack_uint(lastdocid);
-    tag += pack_uint_last(total_length);
+    string tag = F_pack_uint(lastdocid);
+    tag += F_pack_uint_last(total_length);
     postlist_table.add(METAINFO_KEY, tag);
 
     freq_deltas.clear();
@@ -709,6 +1043,17 @@ FlintWritableDatabase::flush_postlist_changes() const
     change_count = 0;
 }
 
+void
+FlintWritableDatabase::close()
+{
+    LOGCALL_VOID(DB, "FlintWritableDatabase::close", NO_ARGS);
+    if (!transaction_active()) {
+	commit();
+	// FIXME: if commit() throws, should we still close?
+    }
+    FlintDatabase::close();
+}
+
 void
 FlintWritableDatabase::add_freq_delta(const string & tname,
 				      Xapian::termcount_diff tf_delta,
@@ -770,8 +1115,7 @@ FlintWritableDatabase::update_mod_plist(Xapian::docid did,
 Xapian::docid
 FlintWritableDatabase::add_document(const Xapian::Document & document)
 {
-    DEBUGCALL(DB, Xapian::docid,
-	      "FlintWritableDatabase::add_document", document);
+    LOGCALL(DB, Xapian::docid, "FlintWritableDatabase::add_document", document);
     // Make sure the docid counter doesn't overflow.
     if (lastdocid == Xapian::docid(-1))
 	throw Xapian::DatabaseError("Run out of docids - you'll have to use copydatabase to eliminate any gaps before you can add more documents");
@@ -783,8 +1127,7 @@ Xapian::docid
 FlintWritableDatabase::add_document_(Xapian::docid did,
 				     const Xapian::Document & document)
 {
-    DEBUGCALL(DB, Xapian::docid,
-	      "FlintWritableDatabase::add_document_", did << ", " << document);
+    LOGCALL(DB, Xapian::docid, "FlintWritableDatabase::add_document_", did | document);
     Assert(did != 0);
     try {
 	// Add the record using that document ID.
@@ -822,7 +1165,7 @@ FlintWritableDatabase::add_document_(Xapian::docid did,
 		}
 	    }
 	}
-	DEBUGLINE(DB, "Calculated doclen for new document " << did << " as " << new_doclen);
+	LOGLINE(DB, "Calculated doclen for new document " << did << " as " << new_doclen);
 
 	// Set the termlist
 	termlist_table.set_termlist(did, document, new_doclen);
@@ -861,7 +1204,7 @@ FlintWritableDatabase::add_document_(Xapian::docid did,
 void
 FlintWritableDatabase::delete_document(Xapian::docid did)
 {
-    DEBUGCALL(DB, void, "FlintWritableDatabase::delete_document", did);
+    LOGCALL_VOID(DB, "FlintWritableDatabase::delete_document", did);
     Assert(did != 0);
 
     if (rare(modify_shortcut_docid == did)) {
@@ -922,7 +1265,7 @@ void
 FlintWritableDatabase::replace_document(Xapian::docid did,
 					const Xapian::Document & document)
 {
-    DEBUGCALL(DB, void, "FlintWritableDatabase::replace_document", did << ", " << document);
+    LOGCALL_VOID(DB, "FlintWritableDatabase::replace_document", did | document);
     Assert(did != 0);
 
     try {
@@ -948,7 +1291,7 @@ FlintWritableDatabase::replace_document(Xapian::docid did,
 		    return;
 		}
 		modifying = true;
-		DEBUGLINE(DB, "Detected potential document modification shortcut.");
+		LOGLINE(DB, "Detected potential document modification shortcut.");
 	    } else {
 		// The modify_shortcut document can't be used for a
 		// modification shortcut now, because it's about to be
@@ -1037,7 +1380,7 @@ FlintWritableDatabase::replace_document(Xapian::docid did,
 		    ++term;
 		}
 	    }
-	    DEBUGLINE(DB, "Calculated doclen for replacement document " << did << " as " << new_doclen);
+	    LOGLINE(DB, "Calculated doclen for replacement document " << did << " as " << new_doclen);
 
 	    // Set the termlist
 	    termlist_table.set_termlist(did, document, new_doclen);
@@ -1086,8 +1429,7 @@ FlintWritableDatabase::replace_document(Xapian::docid did,
 Xapian::Document::Internal *
 FlintWritableDatabase::open_document(Xapian::docid did, bool lazy) const
 {
-    DEBUGCALL(DB, Xapian::Document::Internal *, "FlintWritableDatabase::open_document",
-	      did << ", " << lazy);
+    LOGCALL(DB, Xapian::Document::Internal *, "FlintWritableDatabase::open_document", did | lazy);
     modify_shortcut_document = FlintDatabase::open_document(did, lazy);
     // Store the docid only after open_document() successfully returns, so an
     // attempt to open a missing document doesn't overwrite this.
@@ -1095,10 +1437,10 @@ FlintWritableDatabase::open_document(Xapian::docid did, bool lazy) const
     RETURN(modify_shortcut_document);
 }
 
-Xapian::doclength
+Xapian::termcount
 FlintWritableDatabase::get_doclength(Xapian::docid did) const
 {
-    DEBUGCALL(DB, Xapian::doclength, "FlintWritableDatabase::get_doclength", did);
+    LOGCALL(DB, Xapian::termcount, "FlintWritableDatabase::get_doclength", did);
     map<docid, termcount>::const_iterator i = doclens.find(did);
     if (i != doclens.end()) RETURN(i->second);
 
@@ -1108,7 +1450,7 @@ FlintWritableDatabase::get_doclength(Xapian::docid did) const
 Xapian::doccount
 FlintWritableDatabase::get_termfreq(const string & tname) const
 {
-    DEBUGCALL(DB, Xapian::doccount, "FlintWritableDatabase::get_termfreq", tname);
+    LOGCALL(DB, Xapian::doccount, "FlintWritableDatabase::get_termfreq", tname);
     Xapian::doccount termfreq = FlintDatabase::get_termfreq(tname);
     map<string, pair<termcount_diff, termcount_diff> >::const_iterator i;
     i = freq_deltas.find(tname);
@@ -1119,7 +1461,7 @@ FlintWritableDatabase::get_termfreq(const string & tname) const
 Xapian::termcount
 FlintWritableDatabase::get_collection_freq(const string & tname) const
 {
-    DEBUGCALL(DB, Xapian::termcount, "FlintWritableDatabase::get_collection_freq", tname);
+    LOGCALL(DB, Xapian::termcount, "FlintWritableDatabase::get_collection_freq", tname);
     Xapian::termcount collfreq = FlintDatabase::get_collection_freq(tname);
 
     map<string, pair<termcount_diff, termcount_diff> >::const_iterator i;
@@ -1132,14 +1474,14 @@ FlintWritableDatabase::get_collection_freq(const string & tname) const
 bool
 FlintWritableDatabase::term_exists(const string & tname) const
 {
-    DEBUGCALL(DB, bool, "FlintWritableDatabase::term_exists", tname);
+    LOGCALL(DB, bool, "FlintWritableDatabase::term_exists", tname);
     RETURN(get_termfreq(tname) != 0);
 }
 
 LeafPostList *
 FlintWritableDatabase::open_post_list(const string& tname) const
 {
-    DEBUGCALL(DB, LeafPostList *, "FlintWritableDatabase::open_post_list", tname);
+    LOGCALL(DB, LeafPostList *, "FlintWritableDatabase::open_post_list", tname);
     Xapian::Internal::RefCntPtr<const FlintWritableDatabase> ptrtothis(this);
 
     if (tname.empty()) {
@@ -1164,7 +1506,7 @@ FlintWritableDatabase::open_post_list(const string& tname) const
 TermList *
 FlintWritableDatabase::open_allterms(const string & prefix) const
 {
-    DEBUGCALL(DB, TermList *, "FlintWritableDatabase::open_allterms", "");
+    LOGCALL(DB, TermList *, "FlintWritableDatabase::open_allterms", NO_ARGS);
     // If there are changes, terms may have been added or removed, and so we
     // need to flush (but don't commit - there may be a transaction in progress.
     if (change_count) flush_postlist_changes();
@@ -1233,8 +1575,7 @@ FlintWritableDatabase::clear_synonyms(const string & term) const
 void
 FlintWritableDatabase::set_metadata(const string & key, const string & value)
 {
-    DEBUGCALL(DB, string, "FlintWritableDatabase::set_metadata",
-	      key << ", " << value);
+    LOGCALL(DB, string, "FlintWritableDatabase::set_metadata", key | value);
     string btree_key("\x00\xc0", 2);
     btree_key += key;
     if (value.empty()) {
