@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,7 +33,7 @@
 #include "flint_termlisttable.h"
 #include "flint_values.h"
 #include "flint_version.h"
-#include "flint_lock.h"
+#include "../flint_lock.h"
 
 #include "flint_types.h"
 
@@ -40,8 +41,8 @@
 
 class FlintTermList;
 class FlintAllDocsPostList;
-
-const int XAPIAN_DB_READONLY = 0;
+class RemoteConnection;
+class OmTime;
 
 /** A backend designed for efficient indexing and retrieval, using
  *  compressed posting lists and a btree storage scheme.
@@ -109,11 +110,15 @@ class FlintDatabase : public Xapian::Database::Internal {
 	FlintLock lock;
 
 	/** Total length of all documents including unflushed modifications. */
-	mutable flint_totlen_t total_length;
+	mutable totlen_t total_length;
 
 	/** Highest document ID ever allocated by this database. */
 	mutable Xapian::docid lastdocid;
 
+	/** The maximum number of changesets which should be kept in the
+	 *  database. */
+	unsigned int max_changesets;
+
 	/// Read lastdocid and total_length from the postlist table.
 	void read_metainfo();
 
@@ -136,8 +141,12 @@ class FlintDatabase : public Xapian::Database::Internal {
 
 	/** Get a write lock on the database, or throw an
 	 *  Xapian::DatabaseLockError if failure.
+	 *
+	 *  @param creating true if the database is in the process of being
+	 *  created - if false, will throw a DatabaseOpening error if the lock
+	 *  can't be acquired and the database doesn't exist.
 	 */
-	void get_database_write_lock();
+	void get_database_write_lock(bool creating);
 
 	/** Open tables at specified revision number.
 	 *
@@ -177,6 +186,10 @@ class FlintDatabase : public Xapian::Database::Internal {
 	 */
 	void reopen();
 
+	/** Close all the tables permanently.
+	 */
+	void close();
+
 	/** Called if a modifications fail.
 	 *
 	 *  @param msg is a string description of the exception that was
@@ -200,6 +213,16 @@ class FlintDatabase : public Xapian::Database::Internal {
 	 */
 	void cancel();
 
+	/** Send a set of messages which transfer the whole database.
+	 */
+	void send_whole_database(RemoteConnection & conn,
+				 const OmTime & end_time);
+
+	/** Get the revision stored in a changeset.
+	 */
+	void get_changeset_revisions(const string & path,
+				     flint_revision_number_t * startrev,
+				     flint_revision_number_t * endrev) const;
     public:
 	/** Create and open a flint database.
 	 *
@@ -229,8 +252,9 @@ class FlintDatabase : public Xapian::Database::Internal {
 	//@{
 	Xapian::doccount  get_doccount() const;
 	Xapian::docid get_lastdocid() const;
+	totlen_t get_total_length() const;
 	Xapian::doclength get_avlength() const;
-	Xapian::doclength get_doclength(Xapian::docid did) const;
+	Xapian::termcount get_doclength(Xapian::docid did) const;
 	Xapian::doccount get_termfreq(const string & tname) const;
 	Xapian::termcount get_collection_freq(const string & tname) const;
 	bool term_exists(const string & tname) const;
@@ -252,7 +276,14 @@ class FlintDatabase : public Xapian::Database::Internal {
 
 	string get_metadata(const string & key) const;
 	TermList * open_metadata_keylist(const std::string &prefix) const;
+	void write_changesets_to_fd(int fd,
+				    const string & start_revision,
+				    bool need_whole_db,
+				    Xapian::ReplicationInfo * info);
+	string get_revision_info() const;
+	string get_uuid() const;
 	//@}
+
 };
 
 /** A writable flint database.
@@ -292,6 +323,9 @@ class FlintWritableDatabase : public FlintDatabase {
 	/// Flush any unflushed postlist changes, but don't commit them.
 	void flush_postlist_changes() const;
 
+	/// Close all the tables permanently.
+	void close();
+
 	/** Add or modify an entry in freq_deltas.
 	 *
 	 *  @param tname The term to modify the entry for.
@@ -332,7 +366,7 @@ class FlintWritableDatabase : public FlintDatabase {
 	/** Implementation of virtual methods: see Database::Internal for
 	 *  details.
 	 */
-	void flush();
+	void commit();
 
 	/** Cancel pending modifications to the database. */
 	void cancel();
@@ -344,7 +378,7 @@ class FlintWritableDatabase : public FlintDatabase {
 	// a problem as we only try to call them through the base class
 	// (where they aren't hidden) but some compilers generate a warning
 	// about the hiding.
-#if (!defined __GNUC__ && !defined _MSC_VER) || __GNUC__ > 2
+#ifndef _MSC_VER
 	using Xapian::Database::Internal::delete_document;
 	using Xapian::Database::Internal::replace_document;
 #endif
@@ -374,7 +408,7 @@ class FlintWritableDatabase : public FlintDatabase {
 
 	/** Virtual methods of Database::Internal. */
 	//@{
-	Xapian::doclength get_doclength(Xapian::docid did) const;
+	Xapian::termcount get_doclength(Xapian::docid did) const;
 	Xapian::doccount get_termfreq(const string & tname) const;
 	Xapian::termcount get_collection_freq(const string & tname) const;
 	bool term_exists(const string & tname) const;
