@@ -1,8 +1,8 @@
 /** @file remote-database.cc
  *  @brief Remote backend database class
  */
-/* Copyright (C) 2006,2007 Olly Betts
- * Copyright (C) 2007 Lemur Consulting Ltd
+/* Copyright (C) 2006,2007,2008,2009 Olly Betts
+ * Copyright (C) 2007,2009,2010 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,27 +21,30 @@
 
 #include <config.h>
 
+#include "remote-database.h"
+
 #include "safeerrno.h"
 #include <signal.h>
 
-#include <xapian/error.h>
-
 #include "autoptr.h"
 #include "emptypostlist.h"
 #include "inmemory_positionlist.h"
 #include "net_postlist.h"
 #include "net_termlist.h"
-#include "net_document.h"
+#include "remote-document.h"
 #include "omassert.h"
 #include "serialise.h"
 #include "serialise-double.h"
-#include "stats.h"
+#include "str.h"
 #include "stringutils.h" // For STRINGIZE().
-#include "utils.h"
+#include "weightinternal.h"
 
 #include <string>
 #include <vector>
 
+#include "xapian/error.h"
+#include "xapian/matchspy.h"
+
 using namespace std;
 
 RemoteDatabase::RemoteDatabase(int fd, Xapian::timeout timeout_,
@@ -49,6 +52,8 @@ RemoteDatabase::RemoteDatabase(int fd, Xapian::timeout timeout_,
 	: link(fd, fd, context_),
 	  context(context_),
 	  cached_stats_valid(),
+	  mru_valstats(),
+	  mru_valno(Xapian::BAD_VALUENO),
 	  timeout(timeout_)
 {
 #ifndef __WIN32__
@@ -62,7 +67,7 @@ RemoteDatabase::RemoteDatabase(int fd, Xapian::timeout timeout_,
     if (!writable) {
 	// Transactions only make sense when writing, so flag them as
 	// "unimplemented" so that our destructor doesn't call dtor_called()
-	// since that might try to call flush() which will cause a message to
+	// since that might try to call commit() which will cause a message to
 	// be sent to the remote server and probably an InvalidOperationError
 	// exception message to be returned.
 	transaction_state = TRANSACTION_UNIMPLEMENTED;
@@ -91,23 +96,25 @@ RemoteDatabase::RemoteDatabase(int fd, Xapian::timeout timeout_,
     if (protocol_major != XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION ||
 	protocol_minor < XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION) {
 	string errmsg("Unknown protocol version ");
-	errmsg += om_tostring(protocol_major);
+	errmsg += str(protocol_major);
 	errmsg += '.';
-	errmsg += om_tostring(protocol_minor);
+	errmsg += str(protocol_minor);
 	errmsg += " ("STRINGIZE(XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION)"."STRINGIZE(XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION)" supported)";
 	throw Xapian::NetworkError(errmsg, context);
     }
 
     doccount = decode_length(&p, p_end, false);
     lastdocid = decode_length(&p, p_end, false);
+    doclen_lbound = decode_length(&p, p_end, false);
+    doclen_ubound = decode_length(&p, p_end, false);
     if (p == p_end) {
-	throw Xapian::NetworkError("Bad greeting message received (bool)", context);
+	throw Xapian::NetworkError("Bad greeting message received", context);
     }
     has_positional_info = (*p++ == '1');
-    avlength = unserialise_double(&p, p_end);
-    if (p != p_end || avlength < 0) {
-	throw Xapian::NetworkError("Bad greeting message received (double)", context);
-    }
+    total_length = decode_length(&p, p_end, false);
+    uuid.assign(p, p_end);
+
+    if (writable) update_stats(MSG_WRITEACCESS);
 }
 
 RemoteDatabase *
@@ -119,7 +126,7 @@ RemoteDatabase::as_remotedatabase()
 void
 RemoteDatabase::keep_alive()
 {
-    send_message(MSG_KEEPALIVE, "");
+    send_message(MSG_KEEPALIVE, string());
     string message;
     get_message(message, REPLY_DONE);
 }
@@ -127,9 +134,9 @@ RemoteDatabase::keep_alive()
 TermList *
 RemoteDatabase::open_term_list(Xapian::docid did) const
 {
-    if (did == 0) throw Xapian::InvalidArgumentError("Docid 0 invalid");
+    Assert(did);
 
-    // Ensure that avlength and doccount are up-to-date.
+    // Ensure that total_length and doccount are up-to-date.
     if (!cached_stats_valid) update_stats();
 
     send_message(MSG_TERMLIST, encode_length(did));
@@ -138,15 +145,15 @@ RemoteDatabase::open_term_list(Xapian::docid did) const
     get_message(message, REPLY_DOCLENGTH);
     const char * p = message.c_str();
     const char * p_end = p + message.size();
-    Xapian::doclength doclen = unserialise_double(&p, p_end);
-    if (p != p_end || doclen < 0) {
+    Xapian::termcount doclen = decode_length(&p, p_end, false);
+    if (p != p_end) {
 	throw Xapian::NetworkError("Bad REPLY_DOCLENGTH message received", context);
     }
 
-    AutoPtr<NetworkTermList> tlist;
-    tlist = new NetworkTermList(doclen, doccount,
-				Xapian::Internal::RefCntPtr<const RemoteDatabase>(this),
-				did);
+    AutoPtr<NetworkTermList> tlist(
+	new NetworkTermList(doclen, doccount,
+			    Xapian::Internal::RefCntPtr<const RemoteDatabase>(this),
+			    did));
     vector<NetworkTermListItem> & items = tlist->items;
 
     char type;
@@ -169,15 +176,15 @@ RemoteDatabase::open_term_list(Xapian::docid did) const
 
 TermList *
 RemoteDatabase::open_allterms(const string & prefix) const {
-    // Ensure that avlength and doccount are up-to-date.
+    // Ensure that total_length and doccount are up-to-date.
     if (!cached_stats_valid) update_stats();
 
     send_message(MSG_ALLTERMS, prefix);
 
-    AutoPtr<NetworkTermList> tlist;
-    tlist = new NetworkTermList(0.0, doccount,
-				Xapian::Internal::RefCntPtr<const RemoteDatabase>(this),
-				0);
+    AutoPtr<NetworkTermList> tlist(
+	new NetworkTermList(0, doccount,
+			    Xapian::Internal::RefCntPtr<const RemoteDatabase>(this),
+			    0));
     vector<NetworkTermListItem> & items = tlist->items;
 
     string message;
@@ -261,6 +268,13 @@ void
 RemoteDatabase::reopen()
 {
     update_stats(MSG_REOPEN);
+    mru_valno = Xapian::BAD_VALUENO;
+}
+
+void
+RemoteDatabase::close()
+{
+    do_close();
 }
 
 // Currently lazy is used when fetching documents from the MSet, and in three
@@ -273,7 +287,7 @@ RemoteDatabase::reopen()
 Xapian::Document::Internal *
 RemoteDatabase::open_document(Xapian::docid did, bool /*lazy*/) const
 {
-    if (did == 0) throw Xapian::InvalidArgumentError("Docid 0 invalid");
+    Assert(did);
 
     send_message(MSG_DOCUMENT, encode_length(did));
     string doc_data;
@@ -292,27 +306,27 @@ RemoteDatabase::open_document(Xapian::docid did, bool /*lazy*/) const
 	throw Xapian::NetworkError("Bad message received", context);
     }
 
-    return new NetworkDocument(this, did, doc_data, values);
+    return new RemoteDocument(this, did, doc_data, values);
 }
 
 void
 RemoteDatabase::update_stats(message_type msg_code) const
 {
-    send_message(msg_code, "");
+    send_message(msg_code, string());
     string message;
     get_message(message, REPLY_UPDATE);
     const char * p = message.c_str();
     const char * p_end = p + message.size();
     doccount = decode_length(&p, p_end, false);
     lastdocid = decode_length(&p, p_end, false);
+    doclen_lbound = decode_length(&p, p_end, false);
+    doclen_ubound = decode_length(&p, p_end, false);
     if (p == p_end) {
 	throw Xapian::NetworkError("Bad REPLY_UPDATE message received", context);
     }
     has_positional_info = (*p++ == '1');
-    avlength = unserialise_double(&p, p_end);
-    if (p != p_end || avlength < 0) {
-	throw Xapian::NetworkError("Bad REPLY_UPDATE message received", context);
-    }
+    total_length = decode_length(&p, p_end, false);
+    uuid.assign(p, p_end);
     cached_stats_valid = true;
 }
 
@@ -330,11 +344,18 @@ RemoteDatabase::get_lastdocid() const
     return lastdocid;
 }
 
+totlen_t
+RemoteDatabase::get_total_length() const
+{
+    if (!cached_stats_valid) update_stats();
+    return total_length;
+}
+
 Xapian::doclength
 RemoteDatabase::get_avlength() const
 {
     if (!cached_stats_valid) update_stats();
-    return avlength;
+    return Xapian::doclength(total_length) / doccount;
 }
 
 bool
@@ -374,7 +395,74 @@ RemoteDatabase::get_collection_freq(const string & tname) const
     return decode_length(&p, p_end, false);
 }
 
-Xapian::doclength
+
+void
+RemoteDatabase::read_value_stats(Xapian::valueno valno) const
+{
+    if (mru_valno != valno) {
+	send_message(MSG_VALUESTATS, encode_length(valno));
+	string message;
+	get_message(message, REPLY_VALUESTATS);
+	const char * p = message.data();
+	const char * p_end = p + message.size();
+	mru_valno = valno;
+	mru_valstats.freq = decode_length(&p, p_end, false);
+	size_t len = decode_length(&p, p_end, true);
+	mru_valstats.lower_bound.assign(p, len);
+	p += len;
+	len = decode_length(&p, p_end, true);
+	mru_valstats.upper_bound.assign(p, len);
+	p += len;
+	if (p != p_end) {
+	    throw Xapian::NetworkError("Bad REPLY_VALUESTATS message received", context);
+	}
+    }
+}
+
+Xapian::doccount
+RemoteDatabase::get_value_freq(Xapian::valueno valno) const
+{
+    read_value_stats(valno);
+    return mru_valstats.freq;
+}
+
+std::string
+RemoteDatabase::get_value_lower_bound(Xapian::valueno valno) const
+{
+    read_value_stats(valno);
+    return mru_valstats.lower_bound;
+}
+
+std::string
+RemoteDatabase::get_value_upper_bound(Xapian::valueno valno) const
+{
+    read_value_stats(valno);
+    return mru_valstats.upper_bound;
+}
+
+Xapian::termcount
+RemoteDatabase::get_doclength_lower_bound() const
+{
+    return doclen_lbound;
+}
+
+Xapian::termcount
+RemoteDatabase::get_doclength_upper_bound() const
+{
+    return doclen_ubound;
+}
+
+Xapian::termcount
+RemoteDatabase::get_wdf_upper_bound(const string &) const
+{
+    // The default implementation returns get_collection_freq(), but we
+    // don't want the overhead of a remote message and reply per query
+    // term, and we can get called in the middle of a remote exchange
+    // too.  FIXME: handle this bound in the stats local/remote code...
+    return doclen_ubound;
+}
+
+Xapian::termcount
 RemoteDatabase::get_doclength(Xapian::docid did) const
 {
     Assert(did != 0);
@@ -383,8 +471,8 @@ RemoteDatabase::get_doclength(Xapian::docid did) const
     get_message(message, REPLY_DOCLENGTH);
     const char * p = message.c_str();
     const char * p_end = p + message.size();
-    Xapian::doclength doclen = unserialise_double(&p, p_end);
-    if (p != p_end || doclen < 0) {
+    Xapian::termcount doclen = decode_length(&p, p_end, false);
+    if (p != p_end) {
 	throw Xapian::NetworkError("Bad REPLY_DOCLENGTH message received", context);
     }
     return doclen;
@@ -402,9 +490,9 @@ RemoteDatabase::get_message(string &result, reply_type required_type) const
     }
     if (required_type != REPLY_MAX && type != required_type) {
 	string errmsg("Expecting reply type ");
-	errmsg += om_tostring(int(required_type));
+	errmsg += str(int(required_type));
 	errmsg += ", got ";
-	errmsg += om_tostring(int(type));
+	errmsg += str(int(type));
 	throw Xapian::NetworkError(errmsg);
     }
 
@@ -441,6 +529,7 @@ RemoteDatabase::do_close()
 void
 RemoteDatabase::set_query(const Xapian::Query::Internal *query,
 			 Xapian::termcount qlen,
+			 Xapian::doccount collapse_max,
 			 Xapian::valueno collapse_key,
 			 Xapian::Enquire::docid_order order,
 			 Xapian::valueno sort_key,
@@ -448,7 +537,8 @@ RemoteDatabase::set_query(const Xapian::Query::Internal *query,
 			 bool sort_value_forward,
 			 int percent_cutoff, Xapian::weight weight_cutoff,
 			 const Xapian::Weight *wtscheme,
-			 const Xapian::RSet &omrset)
+			 const Xapian::RSet &omrset,
+			 const vector<Xapian::MatchSpy *> & matchspies)
 {
     string tmp = query->serialise();
     string message = encode_length(tmp.size());
@@ -456,7 +546,8 @@ RemoteDatabase::set_query(const Xapian::Query::Internal *query,
 
     // Serialise assorted Enquire settings.
     message += encode_length(qlen);
-    message += encode_length(collapse_key);
+    message += encode_length(collapse_max);
+    if (collapse_max) message += encode_length(collapse_key);
     message += char('0' + order);
     message += encode_length(sort_key);
     message += char('0' + sort_by);
@@ -472,13 +563,29 @@ RemoteDatabase::set_query(const Xapian::Query::Internal *query,
     message += encode_length(tmp.size());
     message += tmp;
 
-    message += serialise_rset(omrset);
+    tmp = serialise_rset(omrset);
+    message += encode_length(tmp.size());
+    message += tmp;
+
+    vector<Xapian::MatchSpy *>::const_iterator i;
+    for (i = matchspies.begin(); i != matchspies.end(); ++i) {
+	tmp = (*i)->name();
+	if (tmp.empty()) {
+	    throw Xapian::UnimplementedError("MatchSpy not suitable for use with remote searches - name() method returned empty string");
+	}
+	message += encode_length(tmp.size());
+	message += tmp;
+
+	tmp = (*i)->serialise();
+	message += encode_length(tmp.size());
+	message += tmp;
+    }
 
     send_message(MSG_QUERY, message);
 }
 
 bool
-RemoteDatabase::get_remote_stats(bool nowait, Stats &out)
+RemoteDatabase::get_remote_stats(bool nowait, Xapian::Weight::Internal &out)
 {
     if (nowait && !link.ready_to_read()) return false;
 
@@ -491,9 +598,9 @@ RemoteDatabase::get_remote_stats(bool nowait, Stats &out)
 
 void
 RemoteDatabase::send_global_stats(Xapian::doccount first,
-				Xapian::doccount maxitems,
-				Xapian::doccount check_at_least,
-				const Stats &stats)
+				  Xapian::doccount maxitems,
+				  Xapian::doccount check_at_least,
+				  const Xapian::Weight::Internal &stats)
 {
     string message = encode_length(first);
     message += encode_length(maxitems);
@@ -503,17 +610,30 @@ RemoteDatabase::send_global_stats(Xapian::doccount first,
 }
 
 void
-RemoteDatabase::get_mset(Xapian::MSet &mset)
+RemoteDatabase::get_mset(Xapian::MSet &mset,
+			 const vector<Xapian::MatchSpy *> & matchspies)
 {
     string message;
     get_message(message, REPLY_RESULTS);
-    mset = unserialise_mset(message);
+    const char * p = message.data();
+    const char * p_end = p + message.size();
+
+    vector<Xapian::MatchSpy *>::const_iterator i;
+    for (i = matchspies.begin(); i != matchspies.end(); ++i) {
+	if (p == p_end)
+	    throw Xapian::NetworkError("Expected serialised matchspy");
+	size_t len = decode_length(&p, p_end, true);
+	string spyresults(p, len);
+	p += len;
+	(*i)->merge_results(spyresults);
+    }
+    mset = unserialise_mset(p, p_end);
 }
 
 void
-RemoteDatabase::flush()
+RemoteDatabase::commit()
 {
-    send_message(MSG_FLUSH, "");
+    send_message(MSG_COMMIT, string());
 
     // We need to wait for a response to ensure documents have been committed.
     string message;
@@ -524,14 +644,16 @@ void
 RemoteDatabase::cancel()
 {
     cached_stats_valid = false;
+    mru_valno = Xapian::BAD_VALUENO;
 
-    send_message(MSG_CANCEL, "");
+    send_message(MSG_CANCEL, string());
 }
 
 Xapian::docid
 RemoteDatabase::add_document(const Xapian::Document & doc)
 {
     cached_stats_valid = false;
+    mru_valno = Xapian::BAD_VALUENO;
 
     send_message(MSG_ADDDOCUMENT, serialise_document(doc));
 
@@ -547,8 +669,8 @@ void
 RemoteDatabase::delete_document(Xapian::docid did)
 {
     cached_stats_valid = false;
+    mru_valno = Xapian::BAD_VALUENO;
 
-//    send_message(MSG_DELETEDOCUMENT_PRE_30_2, encode_length(did));
     send_message(MSG_DELETEDOCUMENT, encode_length(did));
     string dummy;
     get_message(dummy, REPLY_DONE);
@@ -558,6 +680,7 @@ void
 RemoteDatabase::delete_document(const std::string & unique_term)
 {
     cached_stats_valid = false;
+    mru_valno = Xapian::BAD_VALUENO;
 
     send_message(MSG_DELETEDOCUMENTTERM, unique_term);
 }
@@ -567,6 +690,7 @@ RemoteDatabase::replace_document(Xapian::docid did,
 				 const Xapian::Document & doc)
 {
     cached_stats_valid = false;
+    mru_valno = Xapian::BAD_VALUENO;
 
     string message = encode_length(did);
     message += serialise_document(doc);
@@ -579,6 +703,7 @@ RemoteDatabase::replace_document(const std::string & unique_term,
 				 const Xapian::Document & doc)
 {
     cached_stats_valid = false;
+    mru_valno = Xapian::BAD_VALUENO;
 
     string message = encode_length(unique_term.size());
     message += unique_term;
@@ -592,3 +717,45 @@ RemoteDatabase::replace_document(const std::string & unique_term,
     const char * p_end = p + message.size();
     return decode_length(&p, p_end, false);
 }
+
+string
+RemoteDatabase::get_uuid() const
+{
+    return uuid;
+}
+
+string
+RemoteDatabase::get_metadata(const string & key) const
+{
+    send_message(MSG_GETMETADATA, key);
+    string metadata;
+    get_message(metadata, REPLY_METADATA);
+    return metadata;
+}
+
+void
+RemoteDatabase::set_metadata(const string & key, const string & value)
+{
+    string data = encode_length(key.size());
+    data += key;
+    data += value;
+    send_message(MSG_SETMETADATA, data);
+}
+
+void
+RemoteDatabase::add_spelling(const string & word,
+			     Xapian::termcount freqinc) const
+{
+    string data = encode_length(freqinc);
+    data += word;
+    send_message(MSG_ADDSPELLING, data);
+}
+
+void
+RemoteDatabase::remove_spelling(const string & word,
+				Xapian::termcount freqdec) const
+{
+    string data = encode_length(freqdec);
+    data += word;
+    send_message(MSG_REMOVESPELLING, data);
+}
