@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2006,2007,2008,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,7 +22,7 @@
 
 #include <config.h>
 
-#include <stdlib.h>
+#include <cstdlib>
 
 #include "safeerrno.h"
 
@@ -31,8 +31,8 @@
 
 #include "gnu_getopt.h"
 
-#include <xapian/error.h>
-#include "tcpserver.h"
+#include "xapian/error.h"
+#include "remotetcpserver.h"
 
 using namespace std;
 
@@ -40,7 +40,9 @@ static void register_user_weighting_schemes(TcpServer &server) {
     (void)server; // Suppress "unused parameter" warning.
     // If you have defined your own weighting scheme, register it here
     // like so:
-    // server.register_weighting_scheme(FooWeight());
+    // Registry reg;
+    // reg.register_weighting_scheme(FooWeight());
+    // server.set_registry(reg);
 }
 
 const int MSECS_IDLE_TIMEOUT_DEFAULT = 60000;
@@ -173,8 +175,8 @@ int main(int argc, char **argv) {
 	    cout << " port " << port << endl;
 	}
 
-	TcpServer server(dbnames, host, port, msecs_active_timeout,
-			 msecs_idle_timeout, writable, verbose);
+	RemoteTcpServer server(dbnames, host, port, msecs_active_timeout,
+			       msecs_idle_timeout, writable, verbose);
 
 	if (verbose)
 	    cout << "Listening..." << endl;
