@@ -1,7 +1,8 @@
 /** @file leafpostlist.h
  * @brief Abstract base class for leaf postlists.
  */
-/* Copyright (C) 2007 Olly Betts
+/* Copyright (C) 2007,2009 Olly Betts
+ * Copyright (C) 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +24,8 @@
 
 #include "postlist.h"
 
+#include <string>
+
 namespace Xapian {
     class Weight;
 }
@@ -41,10 +44,15 @@ class LeafPostList : public PostList {
 
   protected:
     const Xapian::Weight * weight;
+
     bool need_doclength;
 
+    /// The term name for this postlist ("" for an alldocs postlist).
+    std::string term;
+
     /// Only constructable as a base class for derived classes.
-    LeafPostList() : weight(0), need_doclength(false) { }
+    LeafPostList(const std::string & term_)
+	: weight(0), need_doclength(false), term(term_) { }
 
   public:
     ~LeafPostList();
@@ -74,6 +82,11 @@ class LeafPostList : public PostList {
     Xapian::weight get_maxweight() const;
     Xapian::weight get_weight() const;
     Xapian::weight recalc_maxweight();
+
+    TermFreqs get_termfreq_est_using_stats(
+	const Xapian::Weight::Internal & stats) const;
+
+    Xapian::termcount count_matching_subqs() const;
 };
 
 #endif // XAPIAN_INCLUDED_LEAFPOSTLIST_H
