@@ -1,8 +1,8 @@
 /** @file remote-database.h
  *  @brief RemoteDatabase is the baseclass for remote database implementations.
  */
-/* Copyright (C) 2006,2007 Olly Betts
- * Copyright (C) 2007 Lemur Consulting Ltd
+/* Copyright (C) 2006,2007,2009,2010 Olly Betts
+ * Copyright (C) 2007,2009,2010 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,14 +27,14 @@
 #include "omqueryinternal.h"
 #include "omtime.h"
 #include "remoteconnection.h"
+#include "valuestats.h"
+#include "xapian/weight.h"
 
 namespace Xapian {
     class RSet;
-    class Weight;
 }
 
 class NetworkPostList;
-class Stats;
 
 /** RemoteDatabase is the baseclass for remote database implementations.
  *
@@ -58,17 +58,35 @@ class RemoteDatabase : public Xapian::Database::Internal {
     /// The remote last docid, given at open.
     mutable Xapian::docid lastdocid;
 
-    /// The remote document avlength, given at open.
-    mutable Xapian::doclength avlength;
+    /// A lower bound on the smallest document length in this database.
+    mutable Xapian::termcount doclen_lbound;
+
+    /// An upper bound on the greatest document length in this database.
+    mutable Xapian::termcount doclen_ubound;
+
+    /// The total length of all documents in this database.
+    mutable totlen_t total_length;
 
     /// Has positional information?
     mutable bool has_positional_info;
 
+    /// The UUID of the remote database.
+    mutable string uuid;
+
     /// The context to return with any error messages
     string context;
 
     mutable bool cached_stats_valid;
 
+    /** The most recently used value statistics. */
+    mutable ValueStats mru_valstats;
+
+    /** The value number for the most recently used value statistics.
+     *
+     *  Set to BAD_VALUENO if no value statistics have yet been looked up.
+     */
+    mutable Xapian::valueno mru_valno;
+
     void update_stats(message_type msg_code = MSG_UPDATE) const;
 
   protected:
@@ -113,6 +131,9 @@ class RemoteDatabase : public Xapian::Database::Internal {
      *
      * @param query			The query.
      * @param qlen			The query length.
+     * @param collapse_max		Max number of items with the same key
+     *					to leave after collapsing (0 for don't
+     *					collapse).
      * @param collapse_key		The value number to collapse matches on.
      * @param order			Sort order for docids.
      * @param sort_key			The value number to sort on.
@@ -122,9 +143,11 @@ class RemoteDatabase : public Xapian::Database::Internal {
      * @param weight_cutoff		Weight cutoff.
      * @param wtscheme			Weighting scheme.
      * @param omrset			The rset.
+     * @param matchspies                The matchspies to use.  NULL if none.
      */
     void set_query(const Xapian::Query::Internal *query,
 		   Xapian::termcount qlen,
+		   Xapian::doccount collapse_max,
 		   Xapian::valueno collapse_key,
 		   Xapian::Enquire::docid_order order,
 		   Xapian::valueno sort_key,
@@ -132,22 +155,24 @@ class RemoteDatabase : public Xapian::Database::Internal {
 		   bool sort_value_forward,
 		   int percent_cutoff, Xapian::weight weight_cutoff,
 		   const Xapian::Weight *wtscheme,
-		   const Xapian::RSet &omrset);
+		   const Xapian::RSet &omrset,
+		   const vector<Xapian::MatchSpy *> & matchspies);
 
-    /** Get the Stats from the remote server.
+    /** Get the stats from the remote server.
      *
      *  @return	true if we got the remote stats; false if we should try again.
      */
-    bool get_remote_stats(bool nowait, Stats &out);
+    bool get_remote_stats(bool nowait, Xapian::Weight::Internal &out);
 
-    /// Send the global Stats to the remote server.
+    /// Send the global stats to the remote server.
     void send_global_stats(Xapian::doccount first,
 			   Xapian::doccount maxitems,
 			   Xapian::doccount check_at_least,
-			   const Stats &stats);
+			   const Xapian::Weight::Internal &stats);
 
     /// Get the MSet from the remote server.
-    void get_mset(Xapian::MSet &mset);
+    void get_mset(Xapian::MSet &mset,
+		  const vector<Xapian::MatchSpy *> & matchspies);
 
     /// Get remote termlist.
     TermList * open_term_list(Xapian::docid did) const;
@@ -159,6 +184,8 @@ class RemoteDatabase : public Xapian::Database::Internal {
 
     void reopen();
 
+    void close();
+
     LeafPostList * open_post_list(const string & tname) const;
 
     Xapian::doccount read_post_list(const string &term, NetworkPostList & pl) const;
@@ -175,10 +202,12 @@ class RemoteDatabase : public Xapian::Database::Internal {
     /// Get the last used docid.
     Xapian::docid get_lastdocid() const;
 
+    totlen_t get_total_length() const;
+
     /// Find out the remote average document length.
     Xapian::doclength get_avlength() const;
 
-    Xapian::doclength get_doclength(Xapian::docid did) const;
+    Xapian::termcount get_doclength(Xapian::docid did) const;
 
     /// Check if term exists.
     bool term_exists(const string & tname) const;
@@ -188,7 +217,17 @@ class RemoteDatabase : public Xapian::Database::Internal {
 
     Xapian::termcount get_collection_freq(const string & tname) const;
 
-    void flush();
+    /// Read the value statistics for a value from a remote database.
+    void read_value_stats(Xapian::valueno valno) const;
+    Xapian::doccount get_value_freq(Xapian::valueno valno) const;
+    std::string get_value_lower_bound(Xapian::valueno valno) const;
+    std::string get_value_upper_bound(Xapian::valueno valno) const;
+
+    Xapian::termcount get_doclength_lower_bound() const;
+    Xapian::termcount get_doclength_upper_bound() const;
+    Xapian::termcount get_wdf_upper_bound(const string & term) const;
+
+    void commit();
 
     void cancel();
 
@@ -200,6 +239,16 @@ class RemoteDatabase : public Xapian::Database::Internal {
     void replace_document(Xapian::docid did, const Xapian::Document & doc);
     Xapian::docid replace_document(const std::string & unique_term,
 				   const Xapian::Document & document);
+
+    std::string get_uuid() const;
+
+    string get_metadata(const string & key) const;
+
+    void set_metadata(const string & key, const string & value);
+
+    void add_spelling(const std::string&, Xapian::termcount) const;
+
+    void remove_spelling(const std::string&,  Xapian::termcount freqdec) const;
 };
 
 #endif // XAPIAN_INCLUDED_REMOTE_DATABASE_H
