@@ -1,8 +1,8 @@
 /** @file remoteprotocol.h
  *  @brief Remote protocol version and message numbers
  */
-/* Copyright (C) 2006,2007 Olly Betts
- * Copyright (C) 2007 Lemur Consulting Ltd
+/* Copyright (C) 2006,2007,2008,2009,2010 Olly Betts
+ * Copyright (C) 2007,2010 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,24 +39,26 @@
 // 30.4: New query operator OP_SCALE_WEIGHT.
 // 30.5: New MSG_GETMSET which expects MSet's percent_factor to be returned.
 // 30.6: Support for OP_VALUE_GE and OP_VALUE_LE in query serialisation
-#define XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION 30
-#define XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION 6
+// 31: 1.1.0 Clean up for Xapian 1.1.0
+// 32: 1.1.1 Serialise termfreq and reltermfreqs together in serialise_stats.
+// 33: 1.1.3 Support for passing matchspies over the remote connection.
+// 34: 1.1.4 Support for metadata over with remote databases.
+// 35: 1.1.5 Support for add_spelling() and remove_spelling().
+#define XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION 35
+#define XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION 0
 
-/* When we move to version 31:
- * + Remove MSG_DELETEDOCUMENT_PRE_30_2
- * + Remove MSG_GETMSET_PRE_30_3
- * + Remove MSG_GETMSET_PRE_30_5
- * + Remove REPLY_RESULTS_PRE_30_5
- * Include MSet::Internal::percent_factor in the MSet serialisation.
+/** Message types (client -> server).
+ *
+ *  When modifying this list, you probably need to update the array of function
+ *  pointers in net/remoteserver.cc too.
  */
-
-/// Message types (client -> server).
 enum message_type {
     MSG_ALLTERMS,		// All Terms
     MSG_COLLFREQ,		// Get Collection Frequency
     MSG_DOCUMENT,		// Get Document
     MSG_TERMEXISTS,		// Term Exists?
     MSG_TERMFREQ,		// Get Term Frequency
+    MSG_VALUESTATS,		// Get value statistics
     MSG_KEEPALIVE,		// Keep-alive
     MSG_DOCLENGTH,		// Get Doc Length
     MSG_QUERY,			// Run Query
@@ -67,16 +69,18 @@ enum message_type {
     MSG_UPDATE,			// Get Updated DocCount and AvLength
     MSG_ADDDOCUMENT,		// Add Document
     MSG_CANCEL,			// Cancel
-    MSG_DELETEDOCUMENT_PRE_30_2,// Delete Document for < 30.2.
     MSG_DELETEDOCUMENTTERM,	// Delete Document by term
-    MSG_FLUSH,			// Flush
+    MSG_COMMIT,			// Commit
     MSG_REPLACEDOCUMENT,	// Replace Document
     MSG_REPLACEDOCUMENTTERM,	// Replace Document by term
-    MSG_GETMSET_PRE_30_3,	// Get MSet for < 30.3
-    MSG_SHUTDOWN,		// Shutdown
     MSG_DELETEDOCUMENT,		// Delete Document
-    MSG_GETMSET_PRE_30_5,	// Get MSet for < 30.5
+    MSG_WRITEACCESS,		// Upgrade to WritableDatabase
+    MSG_GETMETADATA,		// Get metadata
+    MSG_SETMETADATA,		// Set metadata
+    MSG_ADDSPELLING,		// Add a spelling
+    MSG_REMOVESPELLING,		// Remove a spelling
     MSG_GETMSET,		// Get MSet
+    MSG_SHUTDOWN,		// Shutdown
     MSG_MAX
 };
 
@@ -91,8 +95,8 @@ enum reply_type {
     REPLY_TERMDOESNTEXIST,	// Term Doesn't Exist
     REPLY_TERMEXISTS,		// Term Exists
     REPLY_TERMFREQ,		// Get Term Frequency
+    REPLY_VALUESTATS,		// Value statistics
     REPLY_DOCLENGTH,		// Get Doc Length
-    REPLY_RESULTS_PRE_30_5,	// Results (MSet) for < 30.5
     REPLY_STATS,		// Stats
     REPLY_TERMLIST,		// Get Termlist
     REPLY_POSITIONLIST,		// Get PositionList
@@ -102,6 +106,7 @@ enum reply_type {
     REPLY_VALUE,		// Document Value
     REPLY_ADDDOCUMENT,		// Add Document
     REPLY_RESULTS,		// Results (MSet)
+    REPLY_METADATA,		// Metadata
     REPLY_MAX
 };
 
