@@ -3,7 +3,8 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,7 +29,7 @@
 
 #include <xapian/base.h>
 #include <xapian/deprecated.h>
-#include <xapian/sorter.h>
+#include <xapian/keymaker.h>
 #include <xapian/types.h>
 #include <xapian/termiterator.h>
 #include <xapian/visibility.h>
@@ -39,6 +40,7 @@ class Database;
 class Document;
 class ErrorHandler;
 class ExpandDecider;
+class MatchSpy;
 class MSetIterator;
 class Query;
 class Weight;
@@ -105,8 +107,9 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 	 *
 	 *  @param tname The term to look for.
 	 *
-	 *  @exception Xapian::InvalidArgumentError is thrown if the term was
-	 *	       not in the query.
+	 *  This is sometimes more efficient than asking the database directly
+	 *  for the term frequency - in particular, if the term was in the
+	 *  query, its frequency will usually be cached in the MSet.
 	 */
 	Xapian::doccount get_termfreq(const std::string &tname) const;
 
@@ -164,6 +167,21 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 	 */
 	Xapian::doccount get_matches_upper_bound() const;
 
+	/** A lower bound on the number of documents in the database which
+	 *  would match the query if collapsing wasn't used.
+	 */
+	Xapian::doccount get_uncollapsed_matches_lower_bound() const;
+
+	/** A estimate of the number of documents in the database which
+	 *  would match the query if collapsing wasn't used.
+	 */
+	Xapian::doccount get_uncollapsed_matches_estimated() const;
+
+	/** A upper bound on the number of documents in the database which
+	 *  would match the query if collapsing wasn't used.
+	 */
+	Xapian::doccount get_uncollapsed_matches_upper_bound() const;
+
 	/** The maximum possible weight in the MSet.
 	 *
 	 *  This weight is likely not to be attained in the set of results,
@@ -199,7 +217,7 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 	/** Swap the MSet we point to with another */
 	void swap(MSet & other);
 
-	/** Iterator for the terms in this MSet */
+	/** Iterator for the items in this MSet */
 	MSetIterator begin() const;
 
 	/** End iterator corresponding to begin() */
@@ -256,8 +274,6 @@ class XAPIAN_VISIBILITY_DEFAULT MSetIterator {
 	 */
 	MSetIterator() : index(0), mset() { }
 
-	~MSetIterator() { }
-
 	/// Copying is allowed (and is cheap).
 	MSetIterator(const MSetIterator &other) {
 	    index = other.index;
@@ -471,8 +487,6 @@ class XAPIAN_VISIBILITY_DEFAULT ESetIterator {
 	 */
 	ESetIterator() : index(0), eset() { }
 
-	~ESetIterator() { }
-
 	/// Copying is allowed (and is cheap).
 	ESetIterator(const ESetIterator &other) {
 	    index = other.index;
@@ -677,6 +691,32 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 */
 	const Xapian::Query & get_query() const;
 
+	/** Add a matchspy.
+	 *
+	 *  This matchspy will be called with some of the documents which match
+	 *  the query, during the match process.  Exactly which of the matching
+	 *  documents are passed to it depends on exactly when certain
+	 *  optimisations occur during the match process, but it can be
+	 *  controlled to some extent by setting the @a checkatleast parameter
+	 *  to @a get_mset().
+	 *
+	 *  In particular, if there are enough matching documents, at least the
+	 *  number specified by @a checkatleast will be passed to the matchspy.
+	 *  This means that you can force the matchspy to be shown all matching
+	 *  documents by setting @a checkatleast to the number of documents in
+	 *  the database.
+	 *
+	 *  @param spy       The MatchSpy subclass to add.  The caller must
+	 *                   ensure that this remains valid while the Enquire
+	 *                   object remains active, or until @a
+	 *                   clear_matchspies() is called.
+	 */
+	void add_matchspy(MatchSpy * spy);
+
+	/** Remove all the matchspies.
+	 */
+	void clear_matchspies();
+
 	/** Set the weighting scheme to use for queries.
 	 *
 	 *  @param weight_  the new weighting scheme.  If no weighting scheme
@@ -688,17 +728,24 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	/** Set the collapse key to use for queries.
 	 *
 	 *  @param collapse_key  value number to collapse on - at most one MSet
-	 *	entry with each particular value will be returned.
+	 *	entry with each particular value will be returned
+	 *	(default is Xapian::BAD_VALUENO which means no collapsing).
 	 *
-	 *	The entry returned will be the best entry with that particular
-	 *	value (highest weight or highest sorting key).
+	 *  @param collapse_max	 Max number of items with the same key to leave
+	 *			 after collapsing (default 1).
+	 *
+	 *	The MSet returned by get_mset() will have only the "best"
+	 *	(at most) @a collapse_max entries with each particular
+	 *	value of @a collapse_key ("best" being highest ranked - i.e.
+	 *	highest weight or highest sorting key).
 	 *
 	 *	An example use might be to create a value for each document
 	 *	containing an MD5 hash of the document contents.  Then
 	 *	duplicate documents from different sources can be eliminated at
-	 *	search time (it's better to eliminate duplicates at index time,
-	 *	but this may not be always be possible - for example the search
-	 *	may be over more than one Xapian database).
+	 *	search time by collapsing with @a collapse_max = 1 (it's better
+	 *	to eliminate duplicates at index time, but this may not be
+	 *	always be possible - for example the search may be over more
+	 *	than one Xapian database).
 	 *
 	 *	Another use is to group matches in a particular category (e.g.
 	 *	you might collapse a mailing list search on the Subject: so
@@ -708,10 +755,9 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *	Subject: as a boolean term as well as putting it in a value,
 	 *	you can offer a link to a non-collapsed search restricted to
 	 *	that thread using a boolean filter.
-	 *
-	 *	(default is Xapian::BAD_VALUENO which means no collapsing).
 	 */
-	void set_collapse_key(Xapian::valueno collapse_key);
+	void set_collapse_key(Xapian::valueno collapse_key,
+			      Xapian::doccount collapse_max = 1);
 
 	typedef enum {
 	    ASCENDING = 1,
@@ -781,21 +827,21 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *
 	 * @param sort_key  value number to sort on.
 	 *
-	 * @param reverse   If true, reverses the sort order.  (default true, but
-	 *		    this default is confusing and deprecated in 1.1.0, so
-	 *		    we recommend specifying this parameter explicitly).
+	 * @param reverse   If true, reverses the sort order.
          */
-	void set_sort_by_value(Xapian::valueno sort_key, bool reverse = true);
+	void set_sort_by_value(Xapian::valueno sort_key, bool reverse);
+
+	XAPIAN_DEPRECATED(void set_sort_by_value(Xapian::valueno sort_key));
 
 	/** Set the sorting to be by key generated from values only.
 	 *
 	 * @param sorter    The functor to use for generating keys.
 	 *
-	 * @param reverse   If true, reverses the sort order.  (default true, but
-	 *		    this default is confusing and deprecated in 1.1.0, so
-	 *		    we recommend specifying this parameter explicitly).
+	 * @param reverse   If true, reverses the sort order.
          */
-	void set_sort_by_key(Xapian::Sorter * sorter, bool reverse = true);
+	void set_sort_by_key(Xapian::KeyMaker * sorter, bool reverse);
+
+	XAPIAN_DEPRECATED(void set_sort_by_key(Xapian::KeyMaker * sorter));
 
 	/** Set the sorting to be by value, then by relevance for documents
 	 *  with the same value.
@@ -809,24 +855,24 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *
 	 * @param sort_key  value number to sort on.
 	 *
-	 * @param reverse   If true, reverses the sort order.  (default true, but
-	 *		    this default is confusing and deprecated in 1.1.0, so
-	 *		    we recommend specifying this parameter explicitly).
+	 * @param reverse   If true, reverses the sort order.
 	 */
 	void set_sort_by_value_then_relevance(Xapian::valueno sort_key,
-					      bool reverse = true);
+					      bool reverse);
+
+	XAPIAN_DEPRECATED(void set_sort_by_value_then_relevance(Xapian::valueno sort_key));
 
 	/** Set the sorting to be by keys generated from values, then by
 	 *  relevance for documents with identical keys.
 	 *
 	 * @param sorter    The functor to use for generating keys.
 	 *
-	 * @param reverse   If true, reverses the sort order.  (default true, but
-	 *		    this default is confusing and deprecated in 1.1.0, so
-	 *		    we recommend specifying this parameter explicitly).
+	 * @param reverse   If true, reverses the sort order.
 	 */
-	void set_sort_by_key_then_relevance(Xapian::Sorter * sorter,
-					    bool reverse = true);
+	void set_sort_by_key_then_relevance(Xapian::KeyMaker * sorter,
+					    bool reverse);
+
+	XAPIAN_DEPRECATED(void set_sort_by_key_then_relevance(Xapian::KeyMaker * sorter));
 
 	/** Set the sorting to be by relevance then value.
 	 *
@@ -846,12 +892,12 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *
 	 * @param sort_key  value number to sort on.
 	 *
-	 * @param reverse   If true, reverses the sort order.  (default true, but
-	 *		    this default is confusing and deprecated in 1.1.0, so
-	 *		    we recommend specifying this parameter explicitly).
+	 * @param reverse   If true, reverses the sort order.
 	 */
 	void set_sort_by_relevance_then_value(Xapian::valueno sort_key,
-					      bool reverse = true);
+					      bool reverse);
+
+	XAPIAN_DEPRECATED(void set_sort_by_relevance_then_value(Xapian::valueno sort_key));
 
 	/** Set the sorting to be by relevance, then by keys generated from
 	 *  values.
@@ -865,12 +911,12 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *
 	 * @param sorter    The functor to use for generating keys.
 	 *
-	 * @param reverse   If true, reverses the sort order.  (default true, but
-	 *		    this default is confusing and deprecated in 1.1.0, so
-	 *		    we recommend specifying this parameter explicitly).
+	 * @param reverse   If true, reverses the sort order.
 	 */
-	void set_sort_by_relevance_then_key(Xapian::Sorter * sorter,
-					    bool reverse = true);
+	void set_sort_by_relevance_then_key(Xapian::KeyMaker * sorter,
+					    bool reverse);
+
+	XAPIAN_DEPRECATED(void set_sort_by_relevance_then_key(Xapian::KeyMaker * sorter));
 
 	/** Get (a portion of) the match set for the current query.
 	 *
@@ -911,6 +957,10 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *		     assumed to be a relatively expensive test so may
 	 *		     be applied in a lazier fashion.
 	 *
+	 *		     @deprecated this parameter is deprecated - use the
+	 *		     newer MatchSpy class and add_matchspy() method
+	 *		     instead.
+	 *
 	 *  @return	     A Xapian::MSet object containing the results of the
 	 *		     query.
 	 *
@@ -920,11 +970,12 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 		      Xapian::doccount checkatleast = 0,
 		      const RSet * omrset = 0,
 		      const MatchDecider * mdecider = 0) const;
+	XAPIAN_DEPRECATED(
 	MSet get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		      Xapian::doccount checkatleast,
 		      const RSet * omrset,
 		      const MatchDecider * mdecider,
-		      const MatchDecider * matchspy) const;
+		      const MatchDecider * matchspy) const);
 	MSet get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		      const RSet * omrset,
 		      const MatchDecider * mdecider = 0) const {
@@ -933,18 +984,6 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 
 	static const int INCLUDE_QUERY_TERMS = 1;
 	static const int USE_EXACT_TERMFREQ = 2;
-#ifndef _MSC_VER
-	/// Deprecated in Xapian 1.0.0, use INCLUDE_QUERY_TERMS instead.
-	XAPIAN_DEPRECATED(static const int include_query_terms) = 1;
-	/// Deprecated in Xapian 1.0.0, use USE_EXACT_TERMFREQ instead.
-	XAPIAN_DEPRECATED(static const int use_exact_termfreq) = 2;
-#else
-	// Work around MSVC stupidity (you get a warning for deprecating a
-	// declaration).
-	static const int include_query_terms = 1;
-	static const int use_exact_termfreq = 2;
-#pragma deprecated("Xapian::Enquire::include_query_terms", "Xapian::Enquire::use_exact_termfreq")
-#endif
 
 	/** Get the expand set for the given rset.
 	 *
@@ -1056,274 +1095,47 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	    return TermIterator(NULL);
 	}
 
-	/** Register a MatchDecider.
-	 *
-	 * This is used to associate a name with a matchdecider.
-	 *
-	 * @deprecated This method is deprecated.  It was added long ago with
-	 * the intention that it would allow the remote backend to support
-	 * use of MatchDecider objects, but there's a better approach.
-	 *
-	 * @param name		The name to register this matchdecider as.
-	 * @param mdecider	The matchdecider.  If omitted, then remove
-	 *			any matchdecider registered with this name.
-	 */
-	XAPIAN_DEPRECATED(
-	void register_match_decider(const std::string &name,
-				    const MatchDecider *mdecider = NULL));
-
 	/// Return a string describing this object.
 	std::string get_description() const;
 };
 
+// Deprecated forms:
+
+inline void
+Enquire::set_sort_by_value(Xapian::valueno sort_key)
+{
+    return set_sort_by_value(sort_key, true);
 }
 
-class RemoteServer;
-class ScaleWeight;
+inline void
+Enquire::set_sort_by_key(Xapian::KeyMaker * sorter)
+{
+    return set_sort_by_key(sorter, true);
+}
 
-namespace Xapian {
+inline void
+Enquire::set_sort_by_value_then_relevance(Xapian::valueno sort_key)
+{
+    return set_sort_by_value_then_relevance(sort_key, true);
+}
 
-/// Abstract base class for weighting schemes
-class XAPIAN_VISIBILITY_DEFAULT Weight {
-    friend class Enquire; // So Enquire can clone us
-    friend class ::RemoteServer; // So RemoteServer can clone us - FIXME
-    friend class ::ScaleWeight;
-    public:
-	class Internal;
-    protected:
-	Weight(const Weight &);
-    private:
-	void operator=(Weight &);
+inline void
+Enquire::set_sort_by_key_then_relevance(Xapian::KeyMaker * sorter)
+{
+    return set_sort_by_key_then_relevance(sorter, true);
+}
 
-	/** Return a new weight object of this type.
-	 *
-	 * A subclass called FooWeight taking parameters param1 and param2
-	 * should implement this as:
-	 *
-	 * virtual FooWeight * clone() const {
-	 *     return new FooWeight(param1, param2);
-	 * }
-	 */
-	virtual Weight * clone() const = 0;
+inline void
+Enquire::set_sort_by_relevance_then_value(Xapian::valueno sort_key)
+{
+    return set_sort_by_relevance_then_value(sort_key, true);
+}
 
-    protected:
-	const Internal * internal; // Weight::Internal == Stats
-	Xapian::doclength querysize;
-	Xapian::termcount wqf;
-	std::string tname;
-
-    public:
-	// FIXME:1.1: initialise internal to NULL here
-	Weight() { }
-	virtual ~Weight();
-
-	/** Create a new weight object of the same type as this and initialise
-	 *  it with the specified statistics.
-	 *
-	 *  You shouldn't call this method yourself - it's called by
-	 *  Enquire.
-	 *
-	 *  @param internal_  Object to ask for collection statistics.
-	 *  @param querysize_ Query size.
-	 *  @param wqf_       Within query frequency of term this object is
-	 *		      associated with.
-	 *  @param tname_     Term which this object is associated with.
-	 */
-	Weight * create(const Internal * internal_, Xapian::doclength querysize_,
-			Xapian::termcount wqf_, const std::string & tname_) const;
-
-	/** Name of the weighting scheme.
-	 *
-	 *  If the subclass is called FooWeight, this should return "Foo".
-	 */
-	virtual std::string name() const = 0;
-
-	/// Serialise object parameters into a string.
-	virtual std::string serialise() const = 0;
-
-	/// Create object given string serialisation returned by serialise().
-	virtual Weight * unserialise(const std::string &s) const = 0;
-
-	/** Get a weight which is part of the sum over terms being performed.
-	 *  This returns a weight for a given term and document.  These
-	 *  weights are summed to give a total weight for the document.
-	 *
-	 *  @param wdf the within document frequency of the term.
-	 *  @param len the (unnormalised) document length.
-	 */
-	virtual Xapian::weight get_sumpart(Xapian::termcount wdf,
-				      Xapian::doclength len) const = 0;
-
-	/** Gets the maximum value that get_sumpart() may return.  This
-	 *  is used in optimising searches, by having the postlist tree
-	 *  decay appropriately when parts of it can have limited, or no,
-	 *  further effect.
-	 */
-	virtual Xapian::weight get_maxpart() const = 0;
-
-	/** Get an extra weight for a document to add to the sum calculated
-	 *  over the query terms.
-	 *  This returns a weight for a given document, and is used by some
-	 *  weighting schemes to account for influence such as document
-	 *  length.
-	 *
-	 *  @param len the (unnormalised) document length.
-	 */
-	virtual Xapian::weight get_sumextra(Xapian::doclength len) const = 0;
-
-	/** Gets the maximum value that get_sumextra() may return.  This
-	 *  is used in optimising searches.
-	 */
-	virtual Xapian::weight get_maxextra() const = 0;
-
-	/// return false if the weight object doesn't need doclength
-	virtual bool get_sumpart_needs_doclength() const; /* { return true; } */
-};
-
-/// Boolean weighting scheme (everything gets 0)
-class XAPIAN_VISIBILITY_DEFAULT BoolWeight : public Weight {
-    public:
-	BoolWeight * clone() const;
-	BoolWeight() { }
-	~BoolWeight();
-	std::string name() const;
-	std::string serialise() const;
-	BoolWeight * unserialise(const std::string & s) const;
-	Xapian::weight get_sumpart(Xapian::termcount wdf, Xapian::doclength len) const;
-	Xapian::weight get_maxpart() const;
-
-	Xapian::weight get_sumextra(Xapian::doclength len) const;
-	Xapian::weight get_maxextra() const;
-
-	bool get_sumpart_needs_doclength() const;
-};
-
-/** BM25 weighting scheme
- *
- * BM25 weighting options : The BM25 formula is \f[
- *	\frac{k_{2}.n_{q}}{1+L_{d}}+\sum_{t}\frac{(k_{3}+1)q_{t}}{k_{3}+q_{t}}.\frac{(k_{1}+1)f_{t,d}}{k_{1}((1-b)+bL_{d})+f_{t,d}}.w_{t}
- * \f] where
- *   - \f$w_{t}\f$ is the termweight of term t
- *   - \f$f_{t,d}\f$ is the within document frequency of term t in document d
- *   - \f$q_{t}\f$ is the within query frequency of term t
- *   - \f$L_{d}\f$ is the normalised length of document d
- *   - \f$n_{q}\f$ is the size of the query
- *   - \f$k_{1}\f$, \f$k_{2}\f$, \f$k_{3}\f$ and \f$b\f$ are user specified parameters
- */
-class XAPIAN_VISIBILITY_DEFAULT BM25Weight : public Weight {
-    private:
-	mutable Xapian::weight termweight;
-	mutable Xapian::doclength lenpart;
-
-	double k1, k2, k3, b;
-	Xapian::doclength min_normlen;
-
-	mutable bool weight_calculated;
-
-	void calc_termweight() const;
-
-    public:
-	/** Construct a BM25 weight.
-	 *
-	 * @param k1 governs the importance of within document frequency.
-	 *		  Must be >= 0.  0 means ignore wdf.  Default is 1.
-	 * @param k2 compensation factor for the high wdf values in
-	 *		  large documents.  Must be >= 0.  0 means no
-	 *		  compensation.  Default is 0.
-	 * @param k3 governs the importance of within query frequency.
-	 *		  Must be >= 0.  0 means ignore wqf.  Default is 1.
-	 * @param b Relative importance of within document frequency and
-	 *		  document length.  Must be >= 0 and <= 1.  Default
-	 *		  is 0.5.
-	 * @param min_normlen specifies a cutoff on the minimum value that
-	 *		  can be used for a normalised document length -
-	 *		  smaller values will be forced up to this cutoff.
-	 *		  This prevents very small documents getting a huge
-	 *		  bonus weight.  Default is 0.5.
-	 */
-	BM25Weight(double k1_, double k2_, double k3_, double b_,
-		   double min_normlen_)
-		: k1(k1_), k2(k2_), k3(k3_), b(b_), min_normlen(min_normlen_),
-		  weight_calculated(false)
-	{
-	    if (k1 < 0) k1 = 0;
-	    if (k2 < 0) k2 = 0;
-	    if (k3 < 0) k3 = 0;
-	    if (b < 0) b = 0; else if (b > 1) b = 1;
-	}
-	BM25Weight() : k1(1), k2(0), k3(1), b(0.5), min_normlen(0.5),
-		       weight_calculated(false) { }
-
-	BM25Weight * clone() const;
-	~BM25Weight() { }
-	std::string name() const;
-	std::string serialise() const;
-	BM25Weight * unserialise(const std::string & s) const;
-	Xapian::weight get_sumpart(Xapian::termcount wdf, Xapian::doclength len) const;
-	Xapian::weight get_maxpart() const;
-
-	Xapian::weight get_sumextra(Xapian::doclength len) const;
-	Xapian::weight get_maxextra() const;
-
-	bool get_sumpart_needs_doclength() const;
-};
-
-/** Traditional probabilistic weighting scheme.
- *
- * This class implements the Traditional Probabilistic Weighting scheme, as
- * described by the early papers on Probabilistic Retrieval.  BM25 generally
- * gives better results.
- *
- * The Traditional weighting scheme formula is \f[
- *	\sum_{t}\frac{f_{t,d}}{k.L_{d}+f_{t,d}}.w_{t}
- * \f] where
- *   - \f$w_{t}\f$ is the termweight of term t
- *   - \f$f_{t,d}\f$ is the within document frequency of term t in document d
- *   - \f$L_{d}\f$ is the normalised length of document d
- *   - \f$k\f$ is a user specifiable parameter
- *
- * TradWeight(k) is equivalent to BM25Weight(k, 0, 0, 1, 0), except that
- * the latter returns weights (k+1) times larger.
- */
-class XAPIAN_VISIBILITY_DEFAULT TradWeight : public Weight {
-    private:
-	mutable Xapian::weight termweight;
-	mutable Xapian::doclength lenpart;
-
-	double param_k;
-
-	mutable bool weight_calculated;
-
-	void calc_termweight() const;
-
-    public:
-	/** Construct a TradWeight
-	 *
-	 * @param k  parameter governing the importance of within
-	 *	     document frequency and document length - any non-negative
-	 *	     number (0 meaning to ignore wdf and doc length when
-	 *	     calculating weights).  Default is 1.
-	 */
-	explicit TradWeight(double k) : param_k(k), weight_calculated(false) {
-	    if (param_k < 0) param_k = 0;
-	}
-
-	TradWeight() : param_k(1.0), weight_calculated(false) { }
-
-	TradWeight * clone() const;
-	~TradWeight() { }
-	std::string name() const;
-	std::string serialise() const;
-	TradWeight * unserialise(const std::string & s) const;
-
-	Xapian::weight get_sumpart(Xapian::termcount wdf, Xapian::doclength len) const;
-	Xapian::weight get_maxpart() const;
-
-	Xapian::weight get_sumextra(Xapian::doclength len) const;
-	Xapian::weight get_maxextra() const;
-
-	bool get_sumpart_needs_doclength() const;
-};
+inline void
+Enquire::set_sort_by_relevance_then_key(Xapian::KeyMaker * sorter)
+{
+    return set_sort_by_relevance_then_key(sorter, true);
+}
 
 }
 
