@@ -1,7 +1,7 @@
 /** @file remotesubmatch.cc
  *  @brief SubMatch class for a remote database.
  */
-/* Copyright (C) 2006,2007 Olly Betts
+/* Copyright (C) 2006,2007,2009,2010 Olly Betts
  * Copyright (C) 2007,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,25 +20,30 @@
  */
 
 #include <config.h>
+
 #include "remotesubmatch.h"
 
+#include "debuglog.h"
 #include "msetpostlist.h"
-#include "omdebug.h"
 #include "remote-database.h"
-#include "stats.h"
+#include "weightinternal.h"
 
-RemoteSubMatch::RemoteSubMatch(RemoteDatabase *db_, bool decreasing_relevance_)
-	: db(db_), decreasing_relevance(decreasing_relevance_)
+RemoteSubMatch::RemoteSubMatch(RemoteDatabase *db_,
+			       bool decreasing_relevance_,
+			       const vector<Xapian::MatchSpy *> & matchspies_)
+	: db(db_),
+	  decreasing_relevance(decreasing_relevance_),
+	  matchspies(matchspies_)
 {
-    DEBUGCALL(MATCH, void, "RemoteSubMatch",
-	      db_ << ", " << decreasing_relevance_);
+    LOGCALL_CTOR(MATCH, "RemoteSubMatch", db_ | decreasing_relevance_ | matchspies_);
 }
 
 bool
-RemoteSubMatch::prepare_match(bool nowait, Stats & total_stats)
+RemoteSubMatch::prepare_match(bool nowait,
+			      Xapian::Weight::Internal & total_stats)
 {
-    DEBUGCALL(MATCH, bool, "RemoteSubMatch::prepare_match", nowait);
-    Stats remote_stats;
+    LOGCALL(MATCH, bool, "RemoteSubMatch::prepare_match", nowait | total_stats);
+    Xapian::Weight::Internal remote_stats;
     if (!db->get_remote_stats(nowait, remote_stats)) RETURN(false);
     total_stats += remote_stats;
     RETURN(true);
@@ -48,22 +53,24 @@ void
 RemoteSubMatch::start_match(Xapian::doccount first,
 			    Xapian::doccount maxitems,
 			    Xapian::doccount check_at_least,
-			    const Stats & total_stats)
+			    const Xapian::Weight::Internal & total_stats)
 {
-    DEBUGCALL(MATCH, void, "RemoteSubMatch::start_match",
-	      first << ", " << maxitems << ", " << check_at_least);
+    LOGCALL_VOID(MATCH, "RemoteSubMatch::start_match", first | maxitems | check_at_least | total_stats);
     db->send_global_stats(first, maxitems, check_at_least, total_stats);
 }
 
 PostList *
 RemoteSubMatch::get_postlist_and_term_info(MultiMatch *,
-	map<string, Xapian::MSet::Internal::TermFreqAndWeight> * termfreqandwts)
+	map<string, Xapian::MSet::Internal::TermFreqAndWeight> * termfreqandwts,
+	Xapian::termcount * total_subqs_ptr)
 {
-    DEBUGCALL(MATCH, PostList *, "RemoteSubMatch::get_postlist_and_term_info",
-	      "[matcher], " << (void*)termfreqandwts);
+    LOGCALL(MATCH, PostList *, "RemoteSubMatch::get_postlist_and_term_info", "[matcher]" | termfreqandwts | total_subqs_ptr);
     Xapian::MSet mset;
-    db->get_mset(mset);
+    db->get_mset(mset, matchspies);
     percent_factor = mset.internal->percent_factor;
     if (termfreqandwts) *termfreqandwts = mset.internal->termfreqandwts;
+    // For remote databases we report percent_factor rather than counting the
+    // number of subqueries.
+    (void)total_subqs_ptr;
     return new MSetPostList(mset, decreasing_relevance);
 }
