@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2003,2004,2005,2006,2007,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,7 +27,7 @@
 
 #include "progclient.h"
 #include <xapian/error.h>
-#include "omdebug.h"
+#include "debuglog.h"
 
 #include <string>
 #include <vector>
@@ -72,15 +72,13 @@ ProgClient::ProgClient(const string &progname, const string &args,
 			 get_progcontext(progname, args),
 			 writable)
 {
-    DEBUGCALL(DB, void, "ProgClient::ProgClient", progname << ", " << args <<
-	      ", " << msecs_timeout << ", " << writable);
+    LOGCALL_VOID(DB, "ProgClient::ProgClient", progname | args | msecs_timeout | writable);
 }
 
 string
 ProgClient::get_progcontext(const string &progname, const string &args)
 {
-    DEBUGCALL_STATIC(DB, string, "ProgClient::get_progcontext", progname <<
-		     ", " << args);
+    LOGCALL_STATIC(DB, string, "ProgClient::get_progcontext", progname | args);
     RETURN("remote:prog(" + progname + " " + args);
 }
 
@@ -92,8 +90,7 @@ ProgClient::run_program(const string &progname, const string &args
 			)
 {
 #if defined HAVE_SOCKETPAIR && defined HAVE_FORK
-    DEBUGCALL_STATIC(DB, int, "ProgClient::run_program", progname << ", " <<
-		     args << ", [&pid]");
+    LOGCALL_STATIC(DB, int, "ProgClient::run_program", progname | args | "[&pid]");
     /* socketpair() returns two sockets.  We keep sv[0] and give
      * sv[1] to the child process.
      */
@@ -112,7 +109,7 @@ ProgClient::run_program(const string &progname, const string &args
     if (pid != 0) {
 	// parent
 	// close the child's end of the socket
-	close(sv[1]);
+	::close(sv[1]);
 	return sv[0];
     }
 
@@ -121,16 +118,18 @@ ProgClient::run_program(const string &progname, const string &args
      */
 
     // replace stdin and stdout with the socket
-    // FIXME: check return values.
-    close(0);
-    close(1);
-    dup2(sv[1], 0);
-    dup2(sv[1], 1);
+    // FIXME: check return values from dup2.
+    if (sv[1] != 0) {
+	dup2(sv[1], 0);
+    }
+    if (sv[1] != 1) {
+	dup2(sv[1], 1);
+    }
 
     // close unnecessary file descriptors
-    // FIXME: Probably a bit excessive...
-    for (int fd = 2; fd < 256; ++fd) {
-	close(fd);
+    int maxfd = static_cast<int>(sysconf(_SC_OPEN_MAX));
+    for (int fd = 2; fd < maxfd; ++fd) {
+	::close(fd);
     }
 
     // Redirect stderr to /dev/null
@@ -141,7 +140,7 @@ ProgClient::run_program(const string &progname, const string &args
     if (stderrfd != 2) {
 	// Not sure why it wouldn't be 2, but handle the situation anyway.
 	dup2(stderrfd, 2);
-	close(stderrfd);
+	::close(stderrfd);
     }
 
     vector<string> argvec;
@@ -167,8 +166,7 @@ ProgClient::run_program(const string &progname, const string &args
     return 0;
 #endif
 #elif defined __WIN32__
-    DEBUGCALL_STATIC(DB, int, "ProgClient::run_program", progname << ", " <<
-		     args);
+    LOGCALL_STATIC(DB, int, "ProgClient::run_program", progname | args);
 
     static unsigned int pipecount = 0;
     char pipename[256];
