@@ -1,8 +1,8 @@
 /** @file remoteserver.cc
  *  @brief Xapian remote backend server base class
  */
-/* Copyright (C) 2006,2007,2008 Olly Betts
- * Copyright (C) 2006,2007 Lemur Consulting Ltd
+/* Copyright (C) 2006,2007,2008,2009,2010 Olly Betts
+ * Copyright (C) 2006,2007,2009,2010 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,14 +22,15 @@
 #include <config.h>
 #include "remoteserver.h"
 
-#include <xapian/database.h>
-#include <xapian/enquire.h>
-#include <xapian/error.h>
-#include <xapian/valueiterator.h>
+#include "xapian/database.h"
+#include "xapian/enquire.h"
+#include "xapian/error.h"
+#include "xapian/matchspy.h"
+#include "xapian/valueiterator.h"
 
 #include "safeerrno.h"
 #include <signal.h>
-#include <stdlib.h>
+#include <cstdlib>
 
 #include "autoptr.h"
 #include "multimatch.h"
@@ -37,8 +38,8 @@
 #include "omtime.h"
 #include "serialise.h"
 #include "serialise-double.h"
-#include "stats.h"
-#include "utils.h"
+#include "str.h"
+#include "weightinternal.h"
 
 /// Class to throw when we receive the connection closing message.
 struct ConnectionClosed { };
@@ -47,32 +48,32 @@ RemoteServer::RemoteServer(const std::vector<std::string> &dbpaths,
 			   int fdin_, int fdout_,
 			   Xapian::timeout active_timeout_,
 			   Xapian::timeout idle_timeout_,
-			   bool writable)
-    : RemoteConnection(fdin_, fdout_, ""),
-      db(NULL), wdb(NULL),
+			   bool writable_)
+    : RemoteConnection(fdin_, fdout_, std::string()),
+      db(NULL), wdb(NULL), writable(writable_),
       active_timeout(active_timeout_), idle_timeout(idle_timeout_)
 {
     // Catch errors opening the database and propagate them to the client.
     try {
-	if (writable) {
-	    AssertEq(dbpaths.size(), 1); // Expecting exactly one database.
-	    wdb = new Xapian::WritableDatabase(dbpaths[0], Xapian::DB_CREATE_OR_OPEN);
-	    db = wdb;
-	} else {
-	    db = new Xapian::Database;
-	    vector<std::string>::const_iterator i;
-	    for (i = dbpaths.begin(); i != dbpaths.end(); ++i)
-		db->add_database(Xapian::Database(*i));
-	}
-
-	// Build a better description than Database::get_description() gives.
-	// FIXME: improve Database::get_description() and then just use that
-	// instead.
+	Assert(!dbpaths.empty());
+	// We always open the database read-only to start with.  If we're
+	// writable, the client can ask to be upgraded to write access once
+	// connected if it wants it.
+	db = new Xapian::Database(dbpaths[0]);
+	// Build a better description than Database::get_description() gives
+	// in the variable context.  FIXME: improve Database::get_description()
+	// and then just use that instead.
 	context = dbpaths[0];
-	vector<std::string>::const_iterator i(dbpaths.begin());
-	for (++i; i != dbpaths.end(); ++i) {
-	    context += ' ';
-	    context += *i;
+
+	if (!writable) {
+	    vector<std::string>::const_iterator i(dbpaths.begin());
+	    for (++i; i != dbpaths.end(); ++i) {
+		db->add_database(Xapian::Database(*i));
+		context += ' ';
+		context += *i;
+	    }
+	} else {
+	    AssertEq(dbpaths.size(), 1); // Expecting exactly one database.
 	}
     } catch (const Xapian::Error &err) {
 	// Propagate the exception to the client.
@@ -94,29 +95,22 @@ RemoteServer::RemoteServer(const std::vector<std::string> &dbpaths,
     message += char(XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION);
     message += encode_length(db->get_doccount());
     message += encode_length(db->get_lastdocid());
+    message += encode_length(db->get_doclength_lower_bound());
+    message += encode_length(db->get_doclength_upper_bound());
     message += (db->has_positions() ? '1' : '0');
-    message += serialise_double(db->get_avlength());
+    // FIXME: clumsy to reverse calculate total_len like this:
+    totlen_t total_len = totlen_t(db->get_avlength() * db->get_doccount() + .5);
+    message += encode_length(total_len);
+    //message += encode_length(db->get_total_length());
+    string uuid = db->get_uuid();
+    message += uuid;
     send_message(REPLY_GREETING, message);
-
-    // Register weighting schemes.
-    Xapian::Weight * weight;
-    weight = new Xapian::BM25Weight();
-    wtschemes[weight->name()] = weight;
-    weight = new Xapian::BoolWeight();
-    wtschemes[weight->name()] = weight;
-    weight = new Xapian::TradWeight();
-    wtschemes[weight->name()] = weight;
 }
 
 RemoteServer::~RemoteServer()
 {
     delete db;
     // wdb is either NULL or equal to db, so we shouldn't delete it too!
-
-    map<string, Xapian::Weight*>::const_iterator i;
-    for (i = wtschemes.begin(); i != wtschemes.end(); ++i) {
-	delete i->second;
-    }
 }
 
 message_type
@@ -133,14 +127,14 @@ RemoteServer::get_message(Xapian::timeout timeout, string & result,
     if (type == MSG_SHUTDOWN) throw ConnectionClosed();
     if (type >= MSG_MAX) {
 	string errmsg("Invalid message type ");
-	errmsg += om_tostring(type);
+	errmsg += str(type);
 	throw Xapian::NetworkError(errmsg);
     }
     if (required_type != MSG_MAX && type != unsigned(required_type)) {
 	string errmsg("Expecting message type ");
-	errmsg += om_tostring(int(required_type));
+	errmsg += str(int(required_type));
 	errmsg += ", got ";
-	errmsg += om_tostring(int(type));
+	errmsg += str(int(type));
 	throw Xapian::NetworkError(errmsg);
     }
     return static_cast<message_type>(type);
@@ -174,6 +168,7 @@ RemoteServer::run()
 		&RemoteServer::msg_document,
 		&RemoteServer::msg_termexists,
 		&RemoteServer::msg_termfreq,
+		&RemoteServer::msg_valuestats,
 		&RemoteServer::msg_keepalive,
 		&RemoteServer::msg_doclength,
 		&RemoteServer::msg_query,
@@ -184,22 +179,25 @@ RemoteServer::run()
 		&RemoteServer::msg_update,
 		&RemoteServer::msg_adddocument,
 		&RemoteServer::msg_cancel,
-		&RemoteServer::msg_deletedocument_pre_30_2,
 		&RemoteServer::msg_deletedocumentterm,
-		&RemoteServer::msg_flush,
+		&RemoteServer::msg_commit,
 		&RemoteServer::msg_replacedocument,
 		&RemoteServer::msg_replacedocumentterm,
-		NULL, // MSG_GETMSET - used during a conversation.
-		NULL, // MSG_SHUTDOWN - handled by get_message().
-		&RemoteServer::msg_deletedocument
+		&RemoteServer::msg_deletedocument,
+		&RemoteServer::msg_writeaccess,
+		&RemoteServer::msg_getmetadata,
+		&RemoteServer::msg_setmetadata,
+		&RemoteServer::msg_addspelling,
+		&RemoteServer::msg_removespelling,
+		// MSG_GETMSET - used during a conversation.
+		// MSG_SHUTDOWN - handled by get_message().
 	    };
 
 	    string message;
 	    size_t type = get_message(idle_timeout, message);
-	    if (type >= sizeof(dispatch)/sizeof(dispatch[0]) ||
-		dispatch[type] == NULL) {
+	    if (type >= sizeof(dispatch)/sizeof(dispatch[0])) {
 		string errmsg("Unexpected message type ");
-		errmsg += om_tostring(type);
+		errmsg += str(type);
 		throw Xapian::InvalidArgumentError(errmsg);
 	    }
 	    (this->*(dispatch[type]))(message);
@@ -214,7 +212,7 @@ RemoteServer::run()
 	    // And rethrow it so our caller can log it and close the
 	    // connection.
 	    throw;
-	} catch (const Xapian::NetworkError) {
+	} catch (const Xapian::NetworkError &) {
 	    // All other network errors mean we are fatally confused and are
 	    // unlikely to be able to communicate further across this
 	    // connection.  So we don't try to propagate the error to the
@@ -229,7 +227,7 @@ RemoteServer::run()
 	    return;
 	} catch (...) {
 	    // Propagate an unknown exception to the client.
-	    send_message(REPLY_EXCEPTION, "");
+	    send_message(REPLY_EXCEPTION, string());
 	    // And rethrow it so our caller can log it and close the
 	    // connection.
 	    throw;
@@ -251,7 +249,7 @@ RemoteServer::msg_allterms(const string &message)
 	send_message(REPLY_ALLTERMS, item);
     }
 
-    send_message(REPLY_DONE, "");
+    send_message(REPLY_DONE, string());
 }
 
 void
@@ -261,7 +259,7 @@ RemoteServer::msg_termlist(const string &message)
     const char *p_end = p + message.size();
     Xapian::docid did = decode_length(&p, p_end, false);
 
-    send_message(REPLY_DOCLENGTH, serialise_double(db->get_doclength(did)));
+    send_message(REPLY_DOCLENGTH, encode_length(db->get_doclength(did)));
     const Xapian::TermIterator end = db->termlist_end(did);
     for (Xapian::TermIterator t = db->termlist_begin(did); t != end; ++t) {
 	string item = encode_length(t.get_wdf());
@@ -270,7 +268,7 @@ RemoteServer::msg_termlist(const string &message)
 	send_message(REPLY_TERMLIST, item);
     }
 
-    send_message(REPLY_DONE, "");
+    send_message(REPLY_DONE, string());
 }
 
 void
@@ -290,7 +288,7 @@ RemoteServer::msg_positionlist(const string &message)
 	lastpos = pos;
     }
 
-    send_message(REPLY_DONE, "");
+    send_message(REPLY_DONE, string());
 }
 
 void
@@ -312,20 +310,24 @@ RemoteServer::msg_postlist(const string &message)
 	Xapian::docid newdocid = *i;
 	string reply = encode_length(newdocid - lastdocid - 1);
 	reply += encode_length(i.get_wdf());
-	// FIXME: get_doclength should always return an integer value, but
-	// Xapian::doclength is a double.  We could improve the compression
-	// here by casting to an int and serialising that instead, but it's
-	// probably not worth doing since the plan is to stop storing the
-	// document length in the posting lists anyway, at which point the
-	// remote protocol should stop passing it since it will be more
-	// expensive to do so.
-	reply += serialise_double(i.get_doclength());
 
 	send_message(REPLY_POSTLISTITEM, reply);
 	lastdocid = newdocid;
     }
 
-    send_message(REPLY_DONE, "");
+    send_message(REPLY_DONE, string());
+}
+
+void
+RemoteServer::msg_writeaccess(const string & msg)
+{
+    if (!writable) 
+	throw Xapian::InvalidOperationError("Server is read-only");
+
+    wdb = new Xapian::WritableDatabase(context, Xapian::DB_OPEN);
+    delete db;
+    db = wdb;
+    msg_update(msg);
 }
 
 void
@@ -344,11 +346,34 @@ RemoteServer::msg_update(const string &)
 
     string message = encode_length(db->get_doccount());
     message += encode_length(db->get_lastdocid());
+    message += encode_length(db->get_doclength_lower_bound());
+    message += encode_length(db->get_doclength_upper_bound());
     message += (db->has_positions() ? '1' : '0');
-    message += serialise_double(db->get_avlength());
+    // FIXME: clumsy to reverse calculate total_len like this:
+    totlen_t total_len = totlen_t(db->get_avlength() * db->get_doccount() + .5);
+    message += encode_length(total_len);
+    //message += encode_length(db->get_total_length());
+    string uuid = db->get_uuid();
+    message += uuid;
     send_message(REPLY_UPDATE, message);
 }
 
+/** Structure holding a list of match spies.
+ *
+ *  The main reason for the existence of this structure is to make it easy to
+ *  ensure that the match spies are all deleted after use.
+ */
+struct MatchSpyList {
+    vector<Xapian::MatchSpy *> spies;
+
+    ~MatchSpyList() {
+	vector<Xapian::MatchSpy *>::const_iterator i;
+	for (i = spies.begin(); i != spies.end(); ++i) {
+	    delete *i;
+	}
+    }
+};
+
 void
 RemoteServer::msg_query(const string &message_in)
 {
@@ -358,13 +383,16 @@ RemoteServer::msg_query(const string &message_in)
 
     // Unserialise the Query.
     len = decode_length(&p, p_end, true);
-    AutoPtr<Xapian::Query::Internal> query(Xapian::Query::Internal::unserialise(string(p, len)));
+    AutoPtr<Xapian::Query::Internal> query(Xapian::Query::Internal::unserialise(string(p, len), reg));
     p += len;
 
     // Unserialise assorted Enquire settings.
     Xapian::termcount qlen = decode_length(&p, p_end, false);
 
-    Xapian::valueno collapse_key = decode_length(&p, p_end, false);
+    Xapian::valueno collapse_max = decode_length(&p, p_end, false);
+
+    Xapian::valueno collapse_key = Xapian::BAD_VALUENO;
+    if (collapse_max) collapse_key = decode_length(&p, p_end, false);
 
     if (p_end - p < 4 || *p < '0' || *p > '2') {
 	throw Xapian::NetworkError("bad message (docid_order)");
@@ -397,47 +425,54 @@ RemoteServer::msg_query(const string &message_in)
 
     // Unserialise the Weight object.
     len = decode_length(&p, p_end, true);
-    map<string, Xapian::Weight *>::const_iterator i;
-    i = wtschemes.find(string(p, len));
-    if (i == wtschemes.end()) {
-	throw Xapian::InvalidArgumentError("Weighting scheme " + string(p, len) + " not registered");
-    }
+    string wtname(p, len);
     p += len;
 
+    const Xapian::Weight * wttype = reg.get_weighting_scheme(wtname);
+    if (wttype == NULL) {
+	// Note: user weighting schemes should be registered by adding them to
+	// a Registry, and setting the context using
+	// RemoteServer::set_registry().
+	throw Xapian::InvalidArgumentError("Weighting scheme " +
+					   wtname + " not registered");
+    }
+
     len = decode_length(&p, p_end, true);
-    AutoPtr<Xapian::Weight> wt(i->second->unserialise(string(p, len)));
+    AutoPtr<Xapian::Weight> wt(wttype->unserialise(string(p, len)));
     p += len;
 
     // Unserialise the RSet object.
-    Xapian::RSet rset = unserialise_rset(string(p, p_end - p));
+    len = decode_length(&p, p_end, true);
+    Xapian::RSet rset = unserialise_rset(string(p, len));
+    p += len;
 
-    Stats local_stats;
-    MultiMatch match(*db, query.get(), qlen, &rset, collapse_key,
+    // Unserialise any MatchSpy objects.
+    MatchSpyList matchspies;
+    while (p != p_end) {
+	len = decode_length(&p, p_end, true);
+	string spytype(p, len);
+	const Xapian::MatchSpy * spyclass = reg.get_match_spy(spytype);
+	if (spyclass == NULL) {
+	    throw Xapian::InvalidArgumentError("Match spy " + spytype +
+					       " not registered");
+	}
+	p += len;
+
+	len = decode_length(&p, p_end, true);
+	matchspies.spies.push_back(spyclass->unserialise(string(p, len), reg));
+	p += len;
+    }
+
+    Xapian::Weight::Internal local_stats;
+    MultiMatch match(*db, query.get(), qlen, &rset, collapse_max, collapse_key,
 		     percent_cutoff, weight_cutoff, order,
 		     sort_key, sort_by, sort_value_forward, NULL,
-		     local_stats, wt.get(), false, false);
+		     local_stats, wt.get(), matchspies.spies, false, false);
 
     send_message(REPLY_STATS, serialise_stats(local_stats));
 
     string message;
-#if 0 // Reinstate this when major protocol version increases to 31.
     get_message(active_timeout, message, MSG_GETMSET);
-#else
-    char type = get_message(active_timeout, message);
-    if (rare(type != MSG_GETMSET)) {
-	if (type != MSG_GETMSET_PRE_30_5 && type != MSG_GETMSET_PRE_30_3) {
-	    string errmsg("Expecting message type ");
-	    errmsg += om_tostring(MSG_GETMSET_PRE_30_3);
-	    errmsg += " or ";
-	    errmsg += om_tostring(MSG_GETMSET_PRE_30_5);
-	    errmsg += " or ";
-	    errmsg += om_tostring(MSG_GETMSET);
-	    errmsg += ", got ";
-	    errmsg += om_tostring(type);
-	    throw Xapian::NetworkError(errmsg);
-	}
-    }
-#endif
     p = message.c_str();
     p_end = p + message.size();
 
@@ -445,21 +480,24 @@ RemoteServer::msg_query(const string &message_in)
     Xapian::termcount maxitems = decode_length(&p, p_end, false);
 
     Xapian::termcount check_at_least = 0;
-    if (type != MSG_GETMSET_PRE_30_3) {
-	check_at_least = decode_length(&p, p_end, false);
-    }
+    check_at_least = decode_length(&p, p_end, false);
 
     message.erase(0, message.size() - (p_end - p));
-    Stats total_stats(unserialise_stats(message));
+    Xapian::Weight::Internal total_stats(unserialise_stats(message));
+    total_stats.set_bounds_from_db(*db);
 
     Xapian::MSet mset;
     match.get_mset(first, maxitems, check_at_least, mset, total_stats, 0, 0, 0);
 
-    if (type == MSG_GETMSET_PRE_30_3 || type == MSG_GETMSET_PRE_30_5) {
-	send_message(REPLY_RESULTS_PRE_30_5, serialise_mset_pre_30_5(mset));
-    } else {
-	send_message(REPLY_RESULTS, serialise_mset(mset));
+    message.resize(0);
+    vector<Xapian::MatchSpy *>::const_iterator i;
+    for (i = matchspies.spies.begin(); i != matchspies.spies.end(); ++i) {
+	string spy_results = (*i)->serialise_results();
+	message += encode_length(spy_results.size());
+	message += spy_results;
     }
+    message += serialise_mset(mset);
+    send_message(REPLY_RESULTS, message);
 }
 
 void
@@ -479,7 +517,7 @@ RemoteServer::msg_document(const string &message)
 	item += *i;
 	send_message(REPLY_VALUE, item);
     }
-    send_message(REPLY_DONE, "");
+    send_message(REPLY_DONE, string());
 }
 
 void
@@ -487,13 +525,13 @@ RemoteServer::msg_keepalive(const string &)
 {
     // Ensure *our* database stays alive, as it may contain remote databases!
     db->keep_alive();
-    send_message(REPLY_DONE, "");
+    send_message(REPLY_DONE, string());
 }
 
 void
 RemoteServer::msg_termexists(const string &term)
 {
-    send_message((db->term_exists(term) ? REPLY_TERMEXISTS : REPLY_TERMDOESNTEXIST), "");
+    send_message((db->term_exists(term) ? REPLY_TERMEXISTS : REPLY_TERMDOESNTEXIST), string());
 }
 
 void
@@ -508,26 +546,44 @@ RemoteServer::msg_termfreq(const string &term)
     send_message(REPLY_TERMFREQ, encode_length(db->get_termfreq(term)));
 }
 
+void
+RemoteServer::msg_valuestats(const string & message)
+{
+    const char *p = message.data();
+    const char *p_end = p + message.size();
+    while (p != p_end) {
+	Xapian::valueno valno = decode_length(&p, p_end, false);
+	string message_out;
+	message_out += encode_length(db->get_value_freq(valno));
+	string bound = db->get_value_lower_bound(valno);
+	message_out += encode_length(bound.size());
+	message_out += bound;
+	bound = db->get_value_upper_bound(valno);
+	message_out += encode_length(bound.size());
+	message_out += bound;
+
+	send_message(REPLY_VALUESTATS, message_out);
+    }
+}
+
 void
 RemoteServer::msg_doclength(const string &message)
 {
     const char *p = message.data();
     const char *p_end = p + message.size();
     Xapian::docid did = decode_length(&p, p_end, false);
-    // FIXME: get_doclength should always return an integer, but
-    // Xapian::doclength is a double...
-    send_message(REPLY_DOCLENGTH, serialise_double(db->get_doclength(did)));
+    send_message(REPLY_DOCLENGTH, encode_length(db->get_doclength(did)));
 }
 
 void
-RemoteServer::msg_flush(const string &)
+RemoteServer::msg_commit(const string &)
 {
     if (!wdb)
 	throw Xapian::InvalidOperationError("Server is read-only");
 
-    wdb->flush();
+    wdb->commit();
 
-    send_message(REPLY_DONE, "");
+    send_message(REPLY_DONE, string());
 }
 
 void
@@ -553,9 +609,8 @@ RemoteServer::msg_adddocument(const string & message)
     send_message(REPLY_ADDDOCUMENT, encode_length(did));
 }
 
-// FIXME: eliminate this method when we move to remote major 31.
 void
-RemoteServer::msg_deletedocument_pre_30_2(const string & message)
+RemoteServer::msg_deletedocument(const string & message)
 {
     if (!wdb)
 	throw Xapian::InvalidOperationError("Server is read-only");
@@ -565,14 +620,8 @@ RemoteServer::msg_deletedocument_pre_30_2(const string & message)
     Xapian::docid did = decode_length(&p, p_end, false);
 
     wdb->delete_document(did);
-}
 
-void
-RemoteServer::msg_deletedocument(const string & message)
-{
-    msg_deletedocument_pre_30_2(message);
-
-    send_message(REPLY_DONE, "");
+    send_message(REPLY_DONE, string());
 }
 
 void
@@ -613,3 +662,45 @@ RemoteServer::msg_replacedocumentterm(const string & message)
 
     send_message(REPLY_ADDDOCUMENT, encode_length(did));
 }
+
+void
+RemoteServer::msg_getmetadata(const string & message)
+{
+    send_message(REPLY_METADATA, db->get_metadata(message));
+}
+
+void
+RemoteServer::msg_setmetadata(const string & message)
+{
+    if (!wdb)
+	throw Xapian::InvalidOperationError("Server is read-only");
+    const char *p = message.data();
+    const char *p_end = p + message.size();
+    size_t keylen = decode_length(&p, p_end, false);
+    string key(p, keylen);
+    p += keylen;
+    string val(p, p_end - p);
+    wdb->set_metadata(key, val);
+}
+
+void
+RemoteServer::msg_addspelling(const string & message)
+{
+    if (!wdb)
+	throw Xapian::InvalidOperationError("Server is read-only");
+    const char *p = message.data();
+    const char *p_end = p + message.size();
+    Xapian::termcount freqinc = decode_length(&p, p_end, false);
+    wdb->add_spelling(string(p, p_end - p), freqinc);
+}
+
+void
+RemoteServer::msg_removespelling(const string & message)
+{
+    if (!wdb)
+	throw Xapian::InvalidOperationError("Server is read-only");
+    const char *p = message.data();
+    const char *p_end = p + message.size();
+    Xapian::termcount freqdec = decode_length(&p, p_end, false);
+    wdb->remove_spelling(string(p, p_end - p), freqdec);
+}
