@@ -26,7 +26,7 @@
 
 #include "termgenerator_internal.h"
 
-#include "utils.h"
+#include "str.h"
 
 using namespace std;
 using namespace Xapian;
@@ -129,7 +129,7 @@ TermGenerator::get_description() const
 	s += ", doc=";
 	s += internal->doc.get_description();
 	s += ", termpos=";
-	s += om_tostring(internal->termpos);
+	s += str(internal->termpos);
     }
     s += ")";
     return s;
