@@ -2,6 +2,7 @@
  * @brief BackendManager subclass for remoteprog databases.
  */
 /* Copyright (C) 2007,2009 Olly Betts
+ * Copyright (C) 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +23,7 @@
 #define XAPIAN_INCLUDED_BACKENDMANAGER_REMOTEPROG_H
 
 #include "backendmanager.h"
+#include "backendmanager_remote.h"
 
 #include <string>
 
@@ -32,7 +34,7 @@
 #include "positionlist.h"
 
 /// BackendManager subclass for remoteprog databases.
-class BackendManagerRemoteProg : public BackendManager {
+class BackendManagerRemoteProg : public BackendManagerRemote {
     /// Don't allow assignment.
     void operator=(const BackendManagerRemoteProg &);
 
@@ -42,17 +44,16 @@ class BackendManagerRemoteProg : public BackendManager {
     /// The path of the last writable database used.
     std::string last_wdb_name;
 
+  private:
+    /// Create a Xapian::Database object indexing multiple files.
+    Xapian::Database do_get_database(const std::vector<std::string> & files);
+
   public:
-    BackendManagerRemoteProg() { }
+    BackendManagerRemoteProg(const std::string & remote_type_)
+	: BackendManagerRemote(remote_type_) { }
 
     /// Return a string representing the current database type.
-    const char * get_dbtype() const;
-
-    /// Create a RemoteProg Xapian::Database object indexing multiple files.
-    Xapian::Database get_database(const std::vector<std::string> & files);
-
-    /// Create a RemoteProg Xapian::Database object indexing a single file.
-    Xapian::Database get_database(const std::string & file);
+    std::string get_dbtype() const;
 
     /// Create a RemoteProg Xapian::WritableDatabase object indexing a single file.
     Xapian::WritableDatabase get_writable_database(const std::string & name,
