@@ -1,7 +1,7 @@
 /* testsuite.h: a generic test suite engine
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2005,2006,2007,2008 Olly Betts
+ * Copyright 2002,2003,2005,2006,2007,2008,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,7 +34,7 @@
 #include <string>
 #include <vector>
 
-#include <float.h> // For DBL_DIG.
+#include <cfloat> // For DBL_DIG.
 
 /** Class which is thrown when a test case fails.
  */
@@ -244,9 +244,6 @@ class test_driver {
 /// Test a condition, without an additional explanation for failure.
 #define TEST(a) TEST_AND_EXPLAIN(a, "")
 
-// Crude wrapper for compatibility with tests backported from 1.1.x.
-#define TEST_REL(A,R,B) TEST((A) R (B))
-
 /// Test for equality of two things.
 #define TEST_EQUAL(a, b) TEST_AND_EXPLAIN(((a) == (b)), \
 	"Expected `"STRINGIZE(a)"' and `"STRINGIZE(b)"' to be equal:" \
@@ -279,24 +276,9 @@ extern bool TEST_EQUAL_DOUBLE_(double a, double b);
 	"Expected `"STRINGIZE(a)"' and `"STRINGIZE(b)"' not to be equal:" \
 	" were " << (a) << " and " << (b))
 
-/// Test for one thing being >= another.
-#define TEST_GREATER_OR_EQUAL(a, b) TEST_AND_EXPLAIN(((a) >= (b)), \
-	"Expected `"STRINGIZE(a)"' to be greater than or equal to `"STRINGIZE(b)"':" \
-	" were " << (a) << " and " << (b))
+#define DEFINE_TESTCASE(S,COND) bool test_##S()
 
-/// Test for one thing being > another.
-#define TEST_GREATER(a, b) TEST_AND_EXPLAIN(((a) > (b)), \
-	"Expected `"STRINGIZE(a)"' to be greater than `"STRINGIZE(b)"':" \
-	" were " << (a) << " and " << (b))
-
-/// Test for one thing being <= another.
-#define TEST_LESSER_OR_EQUAL(a, b) TEST_AND_EXPLAIN(((a) <= (b)), \
-	"Expected `"STRINGIZE(a)"' to be less than or equal to `"STRINGIZE(b)"':" \
-	" were " << (a) << " and " << (b))
-
-/// Test for one thing being < another.
-#define TEST_LESSER(a, b) TEST_AND_EXPLAIN(((a) < (b)), \
-	"Expected `"STRINGIZE(a)"' to be less than than `"STRINGIZE(b)"':" \
-	" were " << (a) << " and " << (b))
+// Newer test macros:
+#include "testmacros.h"
 
 #endif // OM_HGUARD_TESTSUITE_H
